function ep_chanGrp(EPdata,factorData)
% ep_chanGrp(EPdata,factorData) -
% Defines channel groups for windowing data.
% Strips EPdata of adds (combined channels, cells, subjects, and factors) first.
%
%Input:
%  EPdata         : Structured array with the data and accompanying information in EP file format.  See readData.
%  factorData       : Structured array with information about PCA datasets.
%       .name       : Name of the dataset.
%       .FacPat     : Factor pattern matrix.
%       .facNames   : Labels for each of the factors.
%
%Output: (via global variable)
%   EPchanGrp       : Structured array with information about channel groups
%       .

%History
%  by Joseph Dien (6/13/09)
%  jdien07@mac.com
%
%  bugfix 9/5/09 JD
%  Location of windows appearing partly off screen on some systems due to offset from having Dock (on Mac) or Task Bar
%  (on PC) at the bottom of the screen.
%
%  bugfix 1/17/10 JD
%  Workaround for Matlab bug that appears to prevent buttons from being colored when pressed on Macs using version 2009b
%  under at least some conditions.
%
%  bugfix 2/27/10 JD
%  Applied workaround to PCA guided option too.
%
% bugfix 7/2/10 JD
% In order fix crash due to Matlab bug on Windows, put in brief pause after spec name requestor diaglog box.
%
% bugfix 1/22/11 JD
% Fixed channel controls not working on Matlab versions prior to 7.8.
% Fixed some controls going off screen on a laptop screen.

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

global EPchanGrp

scrsz = get(0,'ScreenSize');
if scrsz(4) > 900
    windowHeight=900;
else
    windowHeight=750;
end;

chanGrpFigure=findobj('Name', 'Channel Group Window');

if nargin > 0
    [EPdata]=ep_stripAdds(EPdata);
    if isempty(EPdata.data)
        msg{1}='Error: The file had no data left after additions were removed.';
        [msg]=ep_errorMsg(msg);
        return
    end;

    if ~isempty(EPchanGrp)  
        eloc=EPdata.eloc;
        
        if length(eloc) ~= length(EPchanGrp.eloc)
            EPchanGrp =[]; %if the new dataset has a different number of channels than the old one, reinitialize this window.
        else
            EPchanGrp.factorData=factorData;
            EPchanGrp.chanNames=EPdata.chanNames;
            EPchanGrp.eloc=eloc;
        end;
    end;
end;

if isempty(EPchanGrp)    
    if ~isempty(chanGrpFigure)
        close(chanGrpFigure)
    end;
    EPchanGrp.handles.window = figure('Name', 'Channel Group Window', 'NumberTitle', 'off', 'Position',[201 scrsz(4)-windowHeight 700 windowHeight]);
    
    EPchanGrp.activeArea=1;
    EPchanGrp.activeGrp=1;

    EPchanGrp.factorData=factorData;
    EPchanGrp.chanNames=EPdata.chanNames;
    EPchanGrp.eloc=EPdata.eloc;
    
    EPchanGrp.numAreas=12;
    EPchanGrp.leftMargin=.15;
    EPchanGrp.rightMargin=.1;    
    EPchanGrp.topMargin=.1;    
    EPchanGrp.bottomMargin=.5; 
    EPchanGrp.work.group(EPchanGrp.activeGrp).channel(1:length(EPchanGrp.chanNames))=EPchanGrp.numAreas+1;
    EPchanGrp.areaColors=[0 0 1;0 1 0;1 0 0;0 1 1;1 0 1;1 1 0;0 0 .5;0 .5 0;.5 0 0;0 .5 .5;.5 0 .5;.5 .5 0;0 0 0];
    
    grpName = inputdlg('Name of new electrode grouping?');
    pause(1); %pause to avoid crash on Windows due to bug in Matlab 2009
    if isempty(grpName)
        close(EPchanGrp.handles.window);
        EPchanGrp=[];
        return
    end;
    EPchanGrp.work.group(EPchanGrp.activeGrp).name=char(grpName);
    EPchanGrp.work.group(EPchanGrp.activeGrp).threshold=[];
    for area =1:12
        EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{area}=['area' num2str(area)];
    end;
    EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{13}='none';
    EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac=0;
    EPchanGrp.work.group(EPchanGrp.activeGrp).threshold=.6;
else
    if isempty(chanGrpFigure)
        EPchanGrp.handles.window = figure('Name', 'Channel Group Window', 'NumberTitle', 'off', 'Position',[201 windowHeight 700 windowHeight]);
    else
        figure(EPchanGrp.handles.window);
    end;
end;

numChans=length(EPchanGrp.chanNames);

%compute screen locations of channels
EPchanGrp.X=zeros(numChans,1);
EPchanGrp.Y=zeros(numChans,1);
hasLoc=zeros(numChans,1);
% for chan=1:numChans    
%     theta=((EPchanGrp.eloc(chan).theta+90)/180)*pi;
%     radius=EPchanGrp.eloc(chan).radius;
%     if isempty(theta)
%         EPchanGrp.X(chan)=[];
%         EPchanGrp.Y(chan)=[];
%     else
%         [EPchanGrp.X(chan),EPchanGrp.Y(chan)] = pol2cart(theta,radius);
%         hasLoc(chan)=1;
%     end;    
% end;

if ~isempty(EPchanGrp.eloc) %if there are electrode coordinates
    nonLoc=0;
    for chan=1:numChans
        theta=((EPchanGrp.eloc(chan).theta+90)/180)*pi;
        radius=EPchanGrp.eloc(chan).radius;
        if isempty(theta)
            nonLoc=nonLoc+1;
            EPchanGrp.X(chan)=.02;
            EPchanGrp.Y(chan)=.02+((.05)*nonLoc);
        else
            [EPchanGrp.X(chan),EPchanGrp.Y(chan)] = pol2cart(theta,radius);
            hasLoc(chan)=1;
        end;
    end;
else %if there are no electrode coordinates
    graphSize=ceil(sqrt(numChans));
    for chan=1:numChans
        EPchanGrp.X(chan)=1-(mod(chan-1,graphSize)+1)*(.05);
        EPchanGrp.Y(chan)=1-(floor((chan-1)/graphSize)+1)*(.05);
        hasLoc(chan)=1;
    end;
end;

EPchanGrp.X=(1-EPchanGrp.X)/2;
EPchanGrp.Y=(EPchanGrp.Y+1)/2;

EPchanGrp.X=((EPchanGrp.X-min(EPchanGrp.X))/(max(EPchanGrp.X-min(EPchanGrp.X)))*(1-EPchanGrp.leftMargin-EPchanGrp.rightMargin))+EPchanGrp.leftMargin;
EPchanGrp.Y=((EPchanGrp.Y-min(EPchanGrp.Y))/(max(EPchanGrp.Y-min(EPchanGrp.Y)))*(1-EPchanGrp.topMargin-EPchanGrp.bottomMargin))+EPchanGrp.bottomMargin;

EPchanGrp.handles.chans=zeros(numChans,1);

MATLABver=ver('MATLAB');
[a b]=strtok(MATLABver.Version,'.');
b=b(2:end);

if (str2num(a) < 7) || ((str2num(a) == 7) && (str2num(b) > 8)) && ismac %workaround for Matlab bug
    
    for chan=1:numChans
        if hasLoc(chan)
            EPchanGrp.handles.chans(chan)=uicontrol('units','normalized','position',[EPchanGrp.X(chan) EPchanGrp.Y(chan) .06 .02],...
                'Style', 'pushbutton', 'String', EPchanGrp.chanNames(chan),'ForegroundColor',EPchanGrp.areaColors(EPchanGrp.work.group(EPchanGrp.activeGrp).channel(chan),:),...
                'BackgroundColor',[1 1 1],'Callback', {@clickChannel,chan});
        end;
    end;
    
else
    
    
    for chan=1:numChans
        if hasLoc(chan)
            EPchanGrp.handles.chans(chan)=uicontrol('units','normalized','position',[EPchanGrp.X(chan) EPchanGrp.Y(chan) .06 .02],...
                'Style', 'pushbutton', 'String', EPchanGrp.chanNames(chan),'BackgroundColor',EPchanGrp.areaColors(EPchanGrp.work.group(EPchanGrp.activeGrp).channel(chan),:),...
                'Callback', {@clickChannel,chan});
        end;
    end;
    
end;

if scrsz(4) > 900
    for area =1:EPchanGrp.numAreas+1
        EPchanGrp.handles.group(EPchanGrp.activeGrp).areaName(area) = uicontrol('Style','edit','ForegroundColor', EPchanGrp.areaColors(area,:),...
            'String',EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{area},'Position',[20 windowHeight-500-area*25 100 25],...
            'Callback', ['global EPchanGrp;','EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{' num2str(area) '}=get(EPchanGrp.handles.group(EPchanGrp.activeGrp).areaName(' num2str(area) '),''String'');']);
        EPchanGrp.handles.area.channel(area)=uicontrol('position',[130 windowHeight-500-area*25 20 20],...
            'Min',0,'Max',1,...
            'Style', 'radiobutton', 'ForegroundColor', EPchanGrp.areaColors(area,:),...
            'Callback', ['global EPchanGrp;','set(EPchanGrp.handles.area.channel(EPchanGrp.activeArea),''Value'',0);','EPchanGrp.activeArea=' num2str(area) ';','set(EPchanGrp.handles.area.channel(EPchanGrp.activeArea),''Value'',1);']);
    end;
else
    for area =1:EPchanGrp.numAreas+1
        EPchanGrp.handles.group(EPchanGrp.activeGrp).areaName(area) = uicontrol('Style','edit','ForegroundColor', EPchanGrp.areaColors(area,:),...
            'String',EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{area},'Position',[20+floor((area-1)/8)*200 windowHeight-500-(rem((area-1),8)+1)*25 100 25],...
            'Callback', ['global EPchanGrp;','EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{' num2str(area) '}=get(EPchanGrp.handles.group(EPchanGrp.activeGrp).areaName(' num2str(area) '),''String'');']);
        EPchanGrp.handles.area.channel(area)=uicontrol('position',[130+floor((area-1)/8)*200 windowHeight-500-(rem((area-1),8)+1)*25 20 20],...
            'Min',0,'Max',1,...
            'Style', 'radiobutton', 'ForegroundColor', EPchanGrp.areaColors(area,:),...
            'Callback', ['global EPchanGrp;','set(EPchanGrp.handles.area.channel(EPchanGrp.activeArea),''Value'',0);','EPchanGrp.activeArea=' num2str(area) ';','set(EPchanGrp.handles.area.channel(EPchanGrp.activeArea),''Value'',1);']);
    end;
end;

set(EPchanGrp.handles.area.channel(EPchanGrp.activeArea),'Value',1);


set(EPchanGrp.handles.group(EPchanGrp.activeGrp).areaName(13),'Style','text');

EPchanGrp.handles.new = uicontrol('Style', 'pushbutton', 'String', 'New',...
    'Position', [20 windowHeight-110 80 35], 'Callback', @newGroup);

EPchanGrp.handles.delete = uicontrol('Style', 'pushbutton', 'String', 'Delete',...
    'Position', [20 windowHeight-150 80 35], 'Callback', @deleteGroup);

EPchanGrp.handles.load = uicontrol('Style', 'pushbutton', 'String', 'Load',...
    'Position', [20 windowHeight-190 80 35], 'Callback', @loadGroup);

EPchanGrp.handles.save = uicontrol('Style', 'pushbutton', 'String', 'Save',...
    'Position', [20 windowHeight-230 80 35], 'Callback', @saveGroup);

EPchanGrp.handles.cancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
    'Position', [20 windowHeight-270 80 35], 'Callback', ['global EPchanGrp; close(EPchanGrp.handles.window); ep(''start'');']);

EPchanGrp.handles.done = uicontrol('Style', 'pushbutton', 'String', 'Done',...
    'Position', [20 windowHeight-310 80 35], 'Callback', ['global EPchanGrp EPmain; EPchanGrp.group=EPchanGrp.work.group; close(EPchanGrp.handles.window); ep(''start''); ']);

h = uicontrol('Style','text',...
    'String','Channel Group','HorizontalAlignment','left',...
    'Position',[105 windowHeight-95 90 20]);

EPchanGrp.handles.chanGrp = uicontrol('Style','popupmenu',...
    'String',{EPchanGrp.work.group.name},...
    'Value',EPchanGrp.activeGrp,'Position',[100 windowHeight-115 100 20], 'Callback', ['global EPchanGrp;','EPchanGrp.activeGrp=get(EPchanGrp.handles.chanGrp,''Value''); ep_chanGrp;']);



EPchanGrp.handles.factor = uicontrol('Style', 'pushbutton', 'String', 'Factor',...
    'Position', [20 windowHeight-350 80 35], 'Callback', ['global EPchanGrp; EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac=1; ep_chanGrp; ']);

if isempty(EPchanGrp.factorData)
    set(EPchanGrp.handles.factor,'enable','off');
end;


if EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac
    h = uicontrol('Style','text',...
        'String','Factor File','HorizontalAlignment','left',...
        'Position',[20 windowHeight-370 90 20]);
    
    EPchanGrp.handles.facFile = uicontrol('Style','popupmenu',...
        'String',{EPchanGrp.factorData.name},...
        'Value',EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac,'Position',[20 windowHeight-390 100 20], 'Callback', ['global EPchanGrp;','EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac=get(EPchanGrp.handles.facFile,''Value'');']);

    h = uicontrol('Style','text',...
        'String','Threshold','HorizontalAlignment','left',...
        'Position',[20 windowHeight-410 90 20]);
    
    EPchanGrp.handles.threshold = uicontrol('Style','edit',...
        'String',EPchanGrp.work.group(EPchanGrp.activeGrp).threshold,'Position',[20 windowHeight-430 100 20]);
    
    EPchanGrp.handles.apply = uicontrol('Style', 'pushbutton', 'String', 'Apply',...
        'Position', [20 windowHeight-470 80 35], 'Callback', @applyFactor);

end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function loadGroup(src,eventdata) %load an electrode group
global EPchanGrp

[FileName,PathName,FilterIndex] = uigetfile('','Load Channel Group','Group1');
if FileName ~= 0
    eval(['load ' PathName FileName]);
    if length(EPgroup.channel) == length(EPchanGrp.eloc)
        EPchanGrp.work.group(end+1)=EPgroup;
        EPchanGrp.activeGrp=length(EPchanGrp.work.group);
    else
        warndlg('Number of electrodes different from current dataset.');
    end;
    ep_chanGrp;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function saveGroup(src,eventdata) %save an electrode group
global EPchanGrp

[FileName,PathName,FilterIndex] = uiputfile('','Save Channel Group','Group1');
EPgroup=EPchanGrp.work.group(EPchanGrp.activeGrp);
eval(['save ' PathName FileName ' EPgroup']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function newGroup(src,eventdata) %Add a new electrode group
global EPchanGrp

grpName = inputdlg('Name of new electrode grouping?');
pause(1); %pause to avoid crash on Windows due to bug in Matlab 2009
if ~isempty(grpName)
    EPchanGrp.work.group(end+1).name=char(grpName);
    EPchanGrp.activeGrp=length(EPchanGrp.work.group);
    EPchanGrp.work.group(EPchanGrp.activeGrp).channel(1:length(EPchanGrp.eloc))=EPchanGrp.numAreas+1;
    for area =1:EPchanGrp.numAreas
        EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{area}=['area' num2str(area)];
    end;
    EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{EPchanGrp.numAreas+1}='none';
    EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac=0;
    EPchanGrp.work.group(EPchanGrp.activeGrp).threshold=.6;
    
    clf(EPchanGrp.handles.window)
    ep_chanGrp;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function clickChannel(src,eventdata,chan) %Click on electrode
global EPchanGrp

if EPchanGrp.activeArea == EPchanGrp.work.group(EPchanGrp.activeGrp).channel(chan)
    EPchanGrp.work.group(EPchanGrp.activeGrp).channel(chan)=EPchanGrp.numAreas+1;
else
    EPchanGrp.work.group(EPchanGrp.activeGrp).channel(chan)=EPchanGrp.activeArea;
end;

set(EPchanGrp.handles.chans(chan),'ForegroundColor',EPchanGrp.areaColors(EPchanGrp.work.group(EPchanGrp.activeGrp).channel(chan),:));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function applyFactor(src,eventdata) %Use factor results to form channel areas
global EPchanGrp

numChans=length(EPchanGrp.eloc);
threshold=str2num(get(EPchanGrp.handles.threshold,'string'));

%select factors to apply
[factorSelection,ok] = listdlg('PromptString',['Choose up to' num2str(EPchanGrp.numAreas) ' factors'],'ListString',EPchanGrp.factorData(EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac).facNames);
numSelectedFacs=length(factorSelection);
if ok && ~isempty(factorSelection) && numSelectedFacs <=EPchanGrp.numAreas
    EPchanGrp.work.group(EPchanGrp.activeGrp).factorSelection=factorSelection;
    EPchanGrp.work.group(EPchanGrp.activeGrp).factorData=EPchanGrp.factorData(EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac);
    for area =1:numSelectedFacs
        EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{area}=EPchanGrp.factorData(EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac).facNames(area);
    end;
    %assign each channel to factor with greatest absolute loading that is higher than the threshold
    facSigns=zeros(numChans,numSelectedFacs);
    for chan=1:numChans
        [theLoading biggestFac]= max(abs(EPchanGrp.work.group(EPchanGrp.activeGrp).factorData(EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac).FacPat(chan,factorSelection)));
        if theLoading >= threshold
            facSigns(chan,biggestFac)=sign(EPchanGrp.work.group(EPchanGrp.activeGrp).factorData(EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac).FacPat(chan,biggestFac));
            EPchanGrp.work.group(EPchanGrp.activeGrp).channel(chan)=biggestFac;
        end;
    end;
    %if a factor has both positive and negative channels then keep only channels with the same sign factor loading
    for factor =1:numSelectedFacs
        if any(facSigns(:,factor)>0) && any(facSigns(:,factor)<0)
            if length(find(facSigns(:,factor)>0)) >= length(find(facSigns(:,factor)<0))
                EPchanGrp.work.group(EPchanGrp.activeGrp).channel(find(facSigns(:,factor)<0))=EPchanGrp.numAreas+1;
            else
                EPchanGrp.work.group(EPchanGrp.activeGrp).channel(find(facSigns(:,factor)>0))=EPchanGrp.numAreas+1;
            end;
        end;
    end;
    for chan=1:numChans
        %set(EPchanGrp.handles.chans(chan),'BackgroundColor',EPchanGrp.areaColors(EPchanGrp.work.group(EPchanGrp.activeGrp).channel(chan),:));
        set(EPchanGrp.handles.chans(chan),'ForegroundColor',EPchanGrp.areaColors(EPchanGrp.work.group(EPchanGrp.activeGrp).channel(chan),:));
    end;
    drawnow
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function deleteGroup(src,eventdata) %Delete current electrode group
global EPchanGrp

numGroups=length(EPchanGrp.work.group);
if  numGroups == 1 %if only one group, then need to replace it with a new one
    grpName = inputdlg('Name of new electrode grouping?');
    pause(1); %pause to avoid crash on Windows due to bug in Matlab 2009
    if ~isempty(grpName)
        EPchanGrp.work.group(1).name=char(grpName);
        EPchanGrp.activeGrp=1;
        EPchanGrp.work.group(EPchanGrp.activeGrp).channel(1:length(EPchanGrp.eloc))=EPchanGrp.numAreas+1;
        for area =1:EPchanGrp.numAreas
            EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{area}=['area' num2str(area)];
        end;
        EPchanGrp.work.group(EPchanGrp.activeGrp).areaName{EPchanGrp.numAreas+1}='none';
        EPchanGrp.work.group(EPchanGrp.activeGrp).activeFac=0;
        EPchanGrp.work.group(EPchanGrp.activeGrp).threshold=.6;
    else
        return
    end;
else
    EPchanGrp.work.group=EPchanGrp.work.group(setdiff([1:numGroups],EPchanGrp.activeGrp));
    EPchanGrp.activeGrp=max(1,EPchanGrp.activeGrp-1);
end;

clf(EPchanGrp.handles.window)
ep_chanGrp;




