function ep_printADFResults(RESULTS, alpha, followup, outfid);
%ep_printADFResults(RESULTS, alpha, followup, outfid);
%Prints out statistical results with appropriate formatting.
%
%Inputs
%  alpha :   The alpha thresholds for significance
%    .uncorrected:  Alpha for a priori interest (.05)
%    .corrected:  Alpha for posthoc multiple comparison corrected.
%  RESULTS	: (1) = test statistic
%  RESULTS	: (2) = Numerator DF
%  RESULTS	: (3) = Denominator DF
%  RESULTS	: (4) = Significance
%  followup: How many levels deep a follow-up this is with zero being not a follow-up.
%  outfid:  The fid of the output file.  Will print to screen if zero.
%Outputs
%  Prints out statistical results.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%History
%  by Joseph Dien (11/8/08)
%  jdien07@mac.com
%
% modified 2/12/09 JD
% Correctly handles NaN results.
%
% modified 3/23/09 JD
% Highlights significant results.  Added option to print to file.
%
% modified 4/21/09 JD
% Green now corresponds to significant 1-tailed test (two times the uncorrected alpha).
%
% modified 7/26/09 JD
% Added indentation for follow-up ANOVAs.

if nargin < 4
    outfid =0;
end;

P=sprintf('%10.8f', RESULTS(4));
Pp=findstr('.',P);
Pz=findstr(P(Pp+1:length(P)),'0');
for i=1:length(Pz)
    if i ~= Pz(i)
        break
    end
end
if isempty(i)
    i=1;
end;

if RESULTS(4) >= .00000001
    pSign='=';
    pVal=sprintf(['%1.' num2str(i+1) 'f'], RESULTS(4));
elseif isnan(RESULTS(4))
    pSign='=';
    pVal=' could not be calculated.';
else
    pSign='<';
    pVal='0.00000001';
end

if outfid
    %print to file
    if RESULTS(4) < alpha.corrected
        fprintf(outfid,[repmat('&nbsp;',1,followup*2) '<font color="red">T<sub>WJt</sub>/c(%1.1f,%1.1f)=%1.2f, p%s%s\n</font></BR>'], RESULTS(2), RESULTS(3), RESULTS(1), pSign, pVal);
    elseif RESULTS(4) < alpha.uncorrected
        fprintf(outfid,[repmat('&nbsp;',1,followup*2) '<font color="orange">T<sub>WJt</sub>/c(%1.1f,%1.1f)=%1.2f, p%s%s\n</font></BR>'], RESULTS(2), RESULTS(3), RESULTS(1), pSign, pVal);
    elseif RESULTS(4) < 2*alpha.uncorrected
        fprintf(outfid,[repmat('&nbsp;',1,followup*2) '<font color="green">T<sub>WJt</sub>/c(%1.1f,%1.1f)=%1.2f, p%s%s\n</font></BR>'], RESULTS(2), RESULTS(3), RESULTS(1), pSign, pVal);
    else
        fprintf(outfid,[repmat('&nbsp;',1,followup*2) 'T<sub>WJt</sub>/c(%1.1f,%1.1f)=%1.2f, p%s%s\n</BR>'], RESULTS(2), RESULTS(3), RESULTS(1), pSign, pVal);
    end;
else
    %print to screen
    if RESULTS(4) < alpha.corrected
        fprintf([repmat('&nbsp;',1,followup*2) '**TWJt/c(%1.1f,%1.1f)=%1.2f, p%s%s**\n'], RESULTS(2), RESULTS(3), RESULTS(1), pSign, pVal);
    elseif RESULTS(4) < alpha.uncorrected
        fprintf([repmat('&nbsp;',1,followup*2) '*TWJt/c(%1.1f,%1.1f)=%1.2f, p%s%s*\n'], RESULTS(2), RESULTS(3), RESULTS(1), pSign, pVal);
    else
        fprintf([repmat('&nbsp;',1,followup*2) 'TWJt/c(%1.1f,%1.1f)=%1.2f, p%s%s\n'], RESULTS(2), RESULTS(3), RESULTS(1), pSign, pVal);
    end; 
end;
