function newDataset=ep_addToEPworkCache(EPdata);
%  newDataset=addToEPworkCache(EPdata);
%       Generates the EPwork cache entry for the data file.
%
%Inputs:
%  EPdata      : Structured array with the data and accompanying information.  See readData.
%
%Outputs:
%  newDataset      : Structured array with the list of files in the work directory
%     .EPwork     : The path of the work directory.
%     .dataName   : The list of dataset names.
%
%History:
%  by Joseph Dien (11/1/09)
%  jdien07@mac.com
%
%  modified 1/24/10 JD
%  Added implicit, baseline, and Fs to cache contents.
%  For continuous files, calculates plotMVmin and max for each one second epoch.
%
%  modified 5/12/10 JD
%  Added ced to cache contents.
%
%  modified 6/15/10 JD
%  Added saved flag to cache contents.
%
%  bufix 7/24/10 JD
%  Fixed min and max of factors sometimes not being calculated correctly.

%     Copyright (C) 1999-2011  Joseph Dien
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

SGLfacs=size(EPdata.data,5);
if ~isempty(EPdata.facData)
    CMBfacs=size(EPdata.facData,5);
end;

if strcmp(EPdata.dataType,'continuous')
    numEpochs=ceil(length(EPdata.timeNames)/EPdata.Fs);
    newDataset.plotMVmin=zeros(numEpochs,1,max(length(EPdata.facNames),1));
    newDataset.plotMVmax=zeros(numEpochs,1,max(length(EPdata.facNames),1));
    for theEpoch=1:numEpochs
        for theFactor=1:SGLfacs
            timeMin=1;
            timeMax=1;
            firstSample=1+(theEpoch-1)*EPdata.Fs;
            lastSample=min(theEpoch*EPdata.Fs,length(EPdata.timeNames));
            if ~isempty(EPdata.facVecT)
                timeMin=min(EPdata.facVecT(:,theFactor));
                timeMax=max(EPdata.facVecT(:,theFactor));
                firstSample=1;
                lastSample=1;
            end;
            spaceMin=1;
            spaceMax=1;
            if ~isempty(EPdata.facVecS)
                spaceMin=min(EPdata.facVecS(:,theFactor));
                spaceMax=max(EPdata.facVecS(:,theFactor));
            end;
            minFac=min(min(EPdata.data(:,firstSample:lastSample,:,:,theFactor)));
            maxFac=max(max(EPdata.data(:,firstSample:lastSample,:,:,theFactor)));
            minMax=kron(kron([timeMin timeMax],[spaceMin spaceMax]),[minFac maxFac]);
            newDataset.plotMVmin(theEpoch,1,theFactor)=min(minMax);
            newDataset.plotMVmax(theEpoch,1,theFactor)=max(minMax);
        end;
        if ~isempty(EPdata.facData)
            for theFactor=1:CMBfacs
                newDataset.plotMVmin(theEpoch,1,theFactor+SGLfacs)=min(min(EPdata.facData(:,firstSample:lastSample,:,:,theFactor)));
                newDataset.plotMVmax(theEpoch,1,theFactor+SGLfacs)=max(max(EPdata.facData(:,firstSample:lastSample,:,:,theFactor)));
            end;
        end;
    end;
else
    newDataset.plotMVmin=zeros(length(EPdata.cellNames),length(EPdata.subNames),max(length(EPdata.facNames),1));
    newDataset.plotMVmax=zeros(length(EPdata.cellNames),length(EPdata.subNames),max(length(EPdata.facNames),1));
    for theCell=1:length(EPdata.cellNames)
        for theSub=1:length(EPdata.subNames)
            for theFactor=1:SGLfacs
                timeMin=1;
                timeMax=1;
                if ~isempty(EPdata.facVecT)
                    timeMin=min(EPdata.facVecT(:,theFactor));
                    timeMax=max(EPdata.facVecT(:,theFactor));
                end;
                spaceMin=1;
                spaceMax=1;
                if ~isempty(EPdata.facVecS)
                    spaceMin=min(EPdata.facVecS(:,theFactor));
                    spaceMax=max(EPdata.facVecS(:,theFactor));
                end;
                minFac=min(min(EPdata.data(:,:,theCell,theSub,theFactor)));
                maxFac=max(max(EPdata.data(:,:,theCell,theSub,theFactor)));
                minMax=kron(kron([timeMin timeMax],[spaceMin spaceMax]),[minFac maxFac]);
                newDataset.plotMVmin(theCell,theSub,theFactor)=min(minMax);
                newDataset.plotMVmax(theCell,theSub,theFactor)=max(minMax);
            end;
            if ~isempty(EPdata.facData)
                for theFactor=1:CMBfacs
                    newDataset.plotMVmin(theCell,theSub,theFactor+SGLfacs)=min(min(EPdata.facData(:,:,theCell,theSub,theFactor)));
                    newDataset.plotMVmax(theCell,theSub,theFactor+SGLfacs)=max(max(EPdata.facData(:,:,theCell,theSub,theFactor)));
                end;
            end;
        end;
    end;
end;

newDataset.dataName=EPdata.dataName;
newDataset.chanNames=EPdata.chanNames;
newDataset.timeNames=EPdata.timeNames;
newDataset.cellNames=EPdata.cellNames;
newDataset.trialNames=EPdata.trialNames;
newDataset.subNames=EPdata.subNames;
newDataset.facNames=EPdata.facNames;
newDataset.eloc=EPdata.eloc;
newDataset.dataType=EPdata.dataType;
newDataset.chanTypes=EPdata.chanTypes;
newDataset.subTypes=EPdata.subTypes;
newDataset.cellTypes=EPdata.cellTypes;
newDataset.facTypes=EPdata.facTypes;
newDataset.implicit=EPdata.implicit;
newDataset.baseline=EPdata.baseline;
newDataset.Fs=EPdata.Fs;
newDataset.ced=EPdata.ced;
newDataset.saved='yes';
