function [err]=ep_checkEPfile(data);
%  [err]=ep_checkEPfile(data);
%       Checks EP data file for problems.
%
%Inputs:
%  data         : Structured array with the data and accompanying information.  See readData.
%
%Outputs:
%  err        : Array of detected error codes.  If no errors, returns zero.

%History:
%  by Joseph Dien (3/4/09)
%  jdien07@mac.com
%
%     Copyright (C) 1999-2011  Joseph Dien
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

err=[];

if ~isfield(data,'data')
    err = [err ;1];
    disp('The .data field is missing.');
    data.data=[];
end

if ~isfield(data,'montage')
    err = [err ;2];
    disp('The .montage field is missing.');
end;

if ~isfield(data,'ced')
    err = [err ;50];
    disp('The .ced field is missing.');
end;

if ~isfield(data,'fileFormat')
    err = [err ;3];
    disp('The .fileFormat field is missing.');
end;

if ~isfield(data,'dataType')
    err = [err ;4];
    disp('The .dataType field is missing.');
    data.dataType=[];
end;

if ~isfield(data,'chanTypes')
    err = [err ;46];
    disp('The .chanTypes field is missing.');
end;

if ~isfield(data,'subTypes')
    err = [err ;47];
    disp('The .subTypes field is missing.');
end;

if ~isfield(data,'cellTypes')
    err = [err ;48];
    disp('The .cellTypes field is missing.');
end;

if ~isfield(data,'facTypes')
    err = [err ;49];
    disp('The .facTypes field is missing.');
end;

if ~isfield(data,'chanNames')
    err = [err ;5];
    disp('The .chanNames field is missing.');
end;

if ~isfield(data,'timeNames')
    err = [err ;6];
    disp('The .timeNames field is missing.');
end;

if ~isfield(data,'subNames')
    err = [err ;7];
    disp('The .subNames field is missing.');
end;

if ~isfield(data,'cellNames')
    err = [err ;8];
    disp('The .cellNames field is missing.');
end;

if ~isfield(data,'facNames')
    err = [err ;45];
    disp('The .facNames field is missing.');
end;

if ~isfield(data,'trialNames')
    err = [err ;9];
    disp('The .trialNames field is missing.');
end;

if ~isfield(data,'EPver')
    err = [err ;10];
    disp('The .EPver field is missing.');
end;

if ~isfield(data,'ver')
    err = [err ;11];
    disp('The .ver field is missing.');
end;

if ~isfield(data,'date')
    err = [err ;12];
    disp('The .date field is missing.');
end;

if ~isfield(data,'Fs')
    err = [err ;13];
    disp('The .Fs field is missing.');
    data.Fs=[];
end;

if ~isfield(data,'baseline')
    err = [err ;14];
    disp('The .baseline field is missing.');
end;

if ~isfield(data,'ename')
    err = [err ;15];
    disp('The .ename field is missing.');
end;

if ~isfield(data,'dataName')
    err = [err ;68];
    disp('The .dataName field is missing.');
end;

if ~isfield(data,'trialSpecs')
    err = [err ;16];
    disp('The .trialSpecs field is missing.');
end;

if ~isfield(data,'trialSpecNames')
    err = [err ;17];
    disp('The .trialSpecNames field is missing.');
end;

if ~isfield(data,'subjectSpecs')
    err = [err ;18];
    disp('The .subjectSpecs field is missing.');
end;

if ~isfield(data,'subjectSpecNames')
    err = [err ;19];
    disp('The .subjectSpecNames field is missing.');
end;

if ~isfield(data,'events')
    err = [err ;20];
    disp('The .events field is missing.');
end;

if ~isfield(data,'avgNum')
    err = [err ;21];
    disp('The .avgNum field is missing.');
end;

if ~isfield(data,'subNum')
    err = [err ;39];
    disp('The .subNum field is missing.');
end;

if ~isfield(data,'fileName')
    err = [err ;31];
    disp('The .fileName field is missing.');
end;

if ~isfield(data,'history')
    err = [err ;32];
    disp('The .history field is missing.');
end;

if ~isfield(data,'facVecS')
    err = [err ;56];
    disp('The .facVecS field is missing.');
end;

if ~isfield(data,'facVecT')
    err = [err ;57];
    disp('The .facVecT field is missing.');
end;

if ~isfield(data,'facData')
    err = [err ;62];
    disp('The .facData field is missing.');
end;

if ~isfield(data,'analysis')
    err = [err ;63];
    disp('The .analysis field is missing.');
else
    if ~isfield(data.analysis,'blinkTrial')
        err = [err ;64];
        disp('The .analysis.blinkTrial field is missing.');
    end;
    if ~isfield(data.analysis,'saccadeTrial')
        err = [err ;69];
        disp('The .analysis.saccadeTrial field is missing.');
    end;
    if ~isfield(data.analysis,'saccadeOnset')
        err = [err ;70];
        disp('The .analysis.saccadeOnset field is missing.');
    end;
    if ~isfield(data.analysis,'moveTrial')
        err = [err ;65];
        disp('The .analysis.moveTrial field is missing.');
    end;
    if ~isfield(data.analysis,'badTrials')
        err = [err ;66];
        disp('The .analysis.badTrials field is missing.');
    end;
    if ~isfield(data.analysis,'badChans')
        err = [err ;67];
        disp('The .analysis.badChans field is missing.');
    end;
end;

if ~isempty(data.facVecS)
    if (size(data.facVecS,1) ~= length(data.chanNames))
        err = [err ;22];
        disp('Number of channel names does not match the data.');
    end;
else
    if length(data.chanNames) ~= size(data.data,1)
        err = [err ;22];
        disp('Number of channel names does not match the data.');
    end;
end;

if ~isempty(data.facData)
    if length(data.chanNames) ~= size(data.facData,1)
        err = [err ;58];
        disp('Number of channel names does not match the combined factor data.');
    end;
    if length(data.timeNames) ~= size(data.facData,2)
        err = [err ;59];
        disp('Number of sample names does not match the combined factor data.');
    end;
    if length(data.cellNames) ~= size(data.facData,3)
        err = [err ;60];
        disp('Number of cell names does not match the combined factor data.');
    end;
    
    if length(data.subNames) ~= size(data.facData,4)
        err = [err ;61];
        disp('Number of subject names does not match the combined factor data.');
    end;
end;

if ~isempty(data.facVecT)
    if (size(data.facVecT,1) ~= length(data.timeNames)) || (size(data.data,2) > 1)
        err = [err ;51];
        disp('Number of sample names does not match the data.');
    end;
else
    if length(data.timeNames) ~= size(data.data,2)
        err = [err ;51];
        disp('Number of sample names does not match the data.');
    end;
end;

if length(data.cellNames) ~= size(data.data,3)
    err = [err ;52];
    disp('Number of cell names does not match the data.');
end;

if length(data.subNames) ~= size(data.data,4)
    err = [err ;53];
    disp('Number of subject names does not match the data.');
end;

numFacData=0;
if ~isempty(data.facData)
    numFacData=size(data.facData,5);
else
    numFacData=0;
end;
% elseif length(find(strcmp(data.facTypes,'CMB'))) == 1
%     numFacData=1;
%end;

if length(data.facNames) ~= (size(data.data,5)+numFacData) && not (isempty(data.facNames) && size(data.data,5)==1)
    err = [err ;44];
    disp('Number of factor names does not match the data.');
end;

if (size(data.subjectSpecs,1) ~= size(data.data,4)) && ~isempty(data.subjectSpecs)
    err = [err ;27];
    disp('Number of subject specs does not match the data.');
end;

if size(data.subjectSpecs,2) ~= length(data.subjectSpecNames)
    err = [err ;28];
    disp('Number of subject spec names does not match the subject specs.');
end;

if size(data.events,1) ~= size(data.data,4) || size(data.events,2) ~= size(data.data,3)
    err = [err ;29];
    disp('Number of events does not match the data.');
end;

if size(data.avgNum,1) ~= size(data.data,4) || size(data.avgNum,2) ~= size(data.data,3)
    err = [err ;30];
    disp('Number of trials going into averages does not match the data.');
end;

if size(data.subNum,1) ~= size(data.data,4) || size(data.subNum,2) ~= size(data.data,3)
    err = [err ;40];
    disp('Number of subjects going into averages does not match the data.');
end;

if ~isfield(data,'eloc')
    err = [err ;38];
    disp('The .eloc field is missing.');
elseif ~isempty(data.eloc)
    if ~isempty(data.facVecS)
        if (size(data.facVecS,1) ~= length(data.eloc))
            err = [err ;35];
            disp('Number of electrode coordinates does not match the data.');
        end;
    else
        if length(data.eloc) ~= size(data.data,1)
            err = [err ;35];
            disp('Number of electrode coordinates does not match the data.');
        end;
    end;
end;

if ~isempty(data.facVecS)
    if size(data.facVecS,2) ~= size(data.data,5)
        err = [err ;54];
        disp('Number of factors does not match spatial factor vector.');
    end;
end;

if ~isempty(data.facVecT)
    if size(data.facVecT,2) ~= size(data.data,5)
        err = [err ;55];
        disp('Number of factors does not match temporal factor vector.');
    end;
end;

if ~isempty(data.implicit)
    implicitRef=sum(strcmp('REF',{data.implicit.type}));
else
    implicitRef=0;
end;

if sum(strcmp('REF',{data.chanTypes}))+ implicitRef > 2
    err = [err ;36];
    disp('There cannot be more than two reference channels.');
end;

if ~isempty(data.facVecS)
    if (size(data.facVecS,1) ~= length(data.chanTypes))
        err = [err ;37];
        disp('Number of channel types does not match the data.');
    end;
else
    if length(data.chanTypes) ~= size(data.data,1)
        err = [err ;37];
        disp('Number of channel types does not match the data.');
    end;
end;

if length(data.subTypes) ~= size(data.data,4)
    err = [err ;41];
    disp('Number of subject types does not match the data.');
end;

if length(data.cellTypes) ~= size(data.data,3)
    err = [err ;42];
    disp('Number of cell types does not match the data.');
end;

if (length(data.facTypes) ~= (size(data.data,5)+numFacData)) && not (isempty(data.facTypes) && size(data.data,5)==1)
    err = [err ;43];
    disp('Number of factor types does not match the data.');
end;

numEpochs=size(data.data,3);
if strcmp(data.dataType,'continuous')
    numEpochs=floor(size(data.data,2)/data.Fs); %excess time points are tacked onto final epoch
    if numEpochs == 0
        numEpochs =1;
    end;
end;

if size(data.analysis.blinkTrial,2) ~= numEpochs || size(data.analysis.blinkTrial,1) ~= size(data.data,4)
    err = [err ;44];
    disp('Number of blink trials does not match the data.');
end;

if size(data.analysis.saccadeTrial,2) ~= numEpochs || size(data.analysis.saccadeTrial,1) ~= size(data.data,4)
    err = [err ;71];
    disp('Number of saccade trials does not match the data.');
end;

if size(data.analysis.saccadeOnset,2) ~= numEpochs || size(data.analysis.saccadeOnset,1) ~= size(data.data,4)
    err = [err ;72];
    disp('Number of saccade onset trials does not match the data.');
end;

if size(data.analysis.moveTrial,2) ~= numEpochs || size(data.analysis.moveTrial,1) ~= size(data.data,4)
    err = [err ;45];
    disp('Number of movement trials does not match the data.');
end;

if size(data.analysis.badTrials,2) ~= numEpochs || size(data.analysis.badTrials,1) ~= size(data.data,4)
    err = [err ;46];
    disp('Number of bad trials does not match the data.');
end;

if ((size(data.analysis.badChans,3) ~= size(data.data,1)) && (size(data.analysis.badChans,3) ~= size(data.facVecS,1))) || (size(data.analysis.badChans,2) ~= numEpochs) || size(data.analysis.badChans,1) ~= size(data.data,4)
    err = [err ;47];
    disp('Number of bad channels does not match the data.');
end;


switch data.dataType
    case 'continuous'
        if length(data.subNames) ~= 1
            err = [err ;69];
            disp('Continuous data should have only one subject waveform.');
        end;
    case 'single_trial'
        if length(data.trialNames) ~= size(data.data,3)
            err = [err ;23];
            disp('Number of trial codes does not match the data.');
        end;
        if size(data.trialSpecs,1) ~= size(data.data,3)
            err = [err ;25];
            disp('Number of trial specs does not match the data.');
        end;
        if size(data.trialSpecs,2) ~= length(data.trialSpecNames)
            err = [err ;26];
            disp('Number of trial spec names does not match the trial specs.');
        end;
    case 'average'
        if ~isempty(data.trialNames)
            err = [err ;68];
            disp('Average files should not have trial names.');
        end;
    otherwise
        err = [err ;24];
        disp(['Data type ' data.dataType ' is not a recognized type.']); 
end;

if isempty(err)
    err=0;
else
    beep;
end;

