function EPdataset=ep_checkEPworkCache(EPdataset);
%  EPdataset=ep_checkEPworkCache(EPdataset);
%       Checks integrity of the EPwork cache and regenerates it from scratch if out-of-date or corrupted.
%
%Inputs:
%  EPdataset      : Structured array with the list of files in the work directory
%     .EPwork     : The path of the work directory.
%     .dataName   : The list of dataset names.
%
%Outputs:
%  EPdataset      : Structured array with the list of files in the work directory

%History:
%  by Joseph Dien (11/1/09)
%  jdien07@mac.com
%
%  modified 1/20/10 JD
%  Added implicit, baseline, and Fs to cache contents.
%
%  modified 5/12/10 JD
%  Added ced to cache contents.
%
%  modified 6/15/10 JD
%  Added saved flag to cache contents.

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

cacheBad=0;

if isempty(EPdataset.dataset)
    EPdataset.dataset='';
end;
if ~isempty(EPdataset.dataset)
    if ~isfield(EPdataset.dataset,'dataName')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'chanNames')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'timeNames')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'cellNames')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'trialNames')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'subNames')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'facNames')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'eloc')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'dataType')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'chanTypes')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'subTypes')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'cellTypes')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'facTypes')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'plotMVmin')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'plotMVmax')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'implicit')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'baseline')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'Fs')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'ced')
        cacheBad=1;
    end;
    if ~isfield(EPdataset.dataset,'saved')
        cacheBad=1;
    end;
end;

if cacheBad
    disp('EPwork cache out-of-date or corrupted.  Regenerating EPwork cache.');
    
    newCache=[];
    
    for i=1:length(EPdataset.dataset)
        oldEPdata=ep_loadEPdataset(EPdataset,i);
        newDataset=ep_addToEPworkCache(oldEPdata);
        newCache=[newCache newDataset];
    end;
    EPdataset.dataset=newCache;
    
    eval(['save ''' EPdataset.EPwork filesep 'EPwork' filesep 'EPdataset'' EPdataset']);
    
    disp('Done.')
end;