function [ALLEEG]=ep_ep2alleeg(EPdata)
% ep_ep2alleeg - [ALLEEG]=ep_ep2alleeg(EPdata) -
% Convert EPdata format to EEGlab's ALLEEG format.  Doesn't convert factor files.
%
%Inputs
%   EPdata        : See ep_readData header.
%
%Outputs:
%  ALLEEG         : EEGlab's ALLEG format.
%
%History
%  by Joseph Dien, with help by Grega Repovs (3/28/09)
%  jdien07@mac.com
%
% bugfix 5/24/10 JD
% Now separates each cell into a separate dataset, not just each subject.

ALLEEG=[];

uniqueCells=unique(EPdata.cellNames);

count=0;
for subject=1:length(EPdata.subNames)
    for theCell=1:length(uniqueCells)
        count=count+1;
        
        ALLEEG(count).setname=[EPdata.subNames{subject} '-' EPdata.cellNames{theCell}];
        ALLEEG(count).filename=[];
        ALLEEG(count).filepath=[];
        ALLEEG(count).subject=EPdata.subNames{subject};
        ALLEEG(count).group='';
        ALLEEG(count).condition=EPdata.cellNames{theCell};
        ALLEEG(count).session=[];
        ALLEEG(count).comments=[];
        ALLEEG(count).nbchan=length(EPdata.chanNames);
        ALLEEG(count).trials=length(strmatch(uniqueCells{theCell},EPdata.cellNames,'exact'));
        ALLEEG(count).pnts=length(EPdata.timeNames);
        ALLEEG(count).srate=EPdata.Fs;
        ALLEEG(count).xmin=min(EPdata.timeNames)/1000;
        ALLEEG(count).xmax=max(EPdata.timeNames)/1000;
        ALLEEG(count).times=EPdata.timeNames;
        ALLEEG(count).data=squeeze(EPdata.data(:,:,theCell,subject));
        ALLEEG(count).icaact=[];
        ALLEEG(count).icawinv=[];
        ALLEEG(count).icasphere=[];
        ALLEEG(count).icaweights=[];
        ALLEEG(count).icachansind=[];
        ALLEEG(count).chanlocs=EPdata.eloc;
        ALLEEG(count).urchanlocs=[];
        ALLEEG(count).chaninfo.icachansind=[];
        ALLEEG(count).ref=[];
        
        clear events;
        if ~isempty(EPdata.events)
            trialList=strmatch(uniqueCells{theCell},EPdata.cellNames,'exact');
            for trial=trialList
                for theEvent=1:length(EPdata.events{subject,trial})
                    if exist('events', 'var')
                        events(end+1)=EPdata.events{subject,trial}(theEvent);
                    else
                        events(1)=EPdata.events{subject,trial}(theEvent);
                    end;
                end;
            end;
            if exist('events', 'var')
                elength = size(EPdata.data,2);
                for f = 1:length(events)
                    events(f).latency=events(f).sample;
                    events(f).epoch = ceil(events(f).sample/elength);
                    events(f).code = lower(events(f).type);
                    events(f).type = events(f).value;
                end
                events = rmfield(events, 'value');
                events = rmfield(events,'sample');
                events = rmfield(events,'offset');
            end;
        end;
        
        if ~exist('events', 'var')
            events = [];
        end;
        
        ALLEEG(count).event=events;
        ALLEEG(count).urevent=events;
        ALLEEG(count).eventdescription=[];
        
        epoch(1)=struct('event',[],'eventduration',[],'eventlatency',[],'eventtype',[],'eventvalue',[]);
        for trial=1:length(trialList)
            epoch(trial).event=trial;
            epoch(trial).eventduration=length(EPdata.timeNames)*(1000/EPdata.Fs);
            epoch(trial).eventlatency=0;
            epoch(trial).eventtype=EPdata.cellNames{trialList(trial)};
            epoch(trial).eventvalue='trial';
        end;
        
        ALLEEG(count).epoch=epoch;
        ALLEEG(count).epochdescription=cell(0);
        ALLEEG(count).reject=[];
        ALLEEG(count).stats=[];
        ALLEEG(count).specdata=[];
        ALLEEG(count).specicaact=[];
        ALLEEG(count).splinefile='';
        ALLEEG(count).icasplinefile='';
        ALLEEG(count).dipfit=[];
        ALLEEG(count).history=[];
        ALLEEG(count).saved='yes';
        ALLEEG(count).etc=[];
        ALLEEG(count).datfile=[];
        
    end;
end;
