function [expandedData]=ep_expandFacs(EPdata,chans,points,cells,subs,facs);
%  [expandedData]=ep_expandFacs(EPdata,chans,points,cells,subs,facs);
%       Uncompresses factor data by multiplying loadings (in facVecS and facVecT) by the factor scores.
%
%Inputs:
%  EPdata         : Structured array with the data and accompanying information in EP file format.  See readData.
%  chans          : Channels to expand.
%  points         : Time points to expand.
%  cells          : Cells to expand.
%  subs           : Subjects to expand.
%  facs           : Factors to expand.
%
%Outputs:
%  expandedData        : Uncompressed voltage data, as specified by the inputs

%History:
%  by Joseph Dien (7/17/09)
%  jdien07@mac.com

% bugfix 2/3/11 JD
% Fixed only final factor being expanded for spatial PCAs.

%
%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 2
    chans=[1:length(EPdata.chanNames)];
end;

if nargin < 3
    points=[1:length(EPdata.timeNames)];
end;

if nargin < 4
    cells=[1:length(EPdata.cellNames)];
end;

if nargin < 5
    subs=[1:length(EPdata.subNames)];
end;

if nargin < 6
    facs=[1:length(EPdata.facNames)];
end;

if isempty(chans)
    chans=[1:length(EPdata.chanNames)];
end;

if isempty(points)
    points=[1:length(EPdata.timeNames)];
end;

if isempty(cells)
    cells=[1:length(EPdata.cellNames)];
end;

if isempty(subs)
    subs=[1:length(EPdata.subNames)];
end;

if isempty(facs)
    facs=[1:length(EPdata.facNames)];
end;

if isempty(facs)
    facs=1;
end;

SGLfacs=facs(find(facs <= size(EPdata.data,5)));
if ~isempty(EPdata.facData)
    CMBfacs=facs(find(facs > size(EPdata.data,5)))-size(EPdata.data,5);
end;

expandedData=zeros(length(chans),length(points),length(cells),length(subs),length(facs));

if isempty(EPdata.facVecS) && isempty(EPdata.facVecT)
    expandedData=EPdata.data(chans,points,cells,subs,SGLfacs);
else
    for i=1:length(chans)
        theChan=chans(i);
        for k=1:length(points)
            thePoint=points(k);
            if ~isempty(EPdata.facVecS) && ~isempty(EPdata.facVecT)
                for m=1:length(SGLfacs)
                    theFac=SGLfacs(m);
                    expandedData(i,k,:,:,m)=EPdata.data(:,:,cells,subs,theFac)*EPdata.facVecS(theChan,theFac)*EPdata.facVecT(thePoint,theFac);
                end;
            end;
            if ~isempty(EPdata.facVecS) && isempty(EPdata.facVecT)
                for m=1:length(SGLfacs)
                    theFac=SGLfacs(m);
                    expandedData(i,k,:,:,m)=EPdata.data(1,thePoint,cells,subs,theFac)*EPdata.facVecS(theChan,theFac);
                end;
            end;
            if isempty(EPdata.facVecS) && ~isempty(EPdata.facVecT)
                for m=1:length(SGLfacs)
                    theFac=SGLfacs(m);
                    expandedData(i,k,:,:,m)=EPdata.data(theChan,1,cells,subs,theFac)*EPdata.facVecT(thePoint,theFac);
                end;
            end;
        end;
    end;
end;

if ~isempty(EPdata.facData)
    expandedData(:,:,:,:,length(SGLfacs)+1:length(SGLfacs)+length(CMBfacs))=EPdata.facData(chans,points,cells,subs,CMBfacs);
end;