function EPdata=ep_loadEPdataset(EPdataset,theDataset);
%  EPdata=ep_loadEPdataset(EPdataset,theDataset);
%       Loads a dataset from the work directory.
%
%Inputs:
%  EPdataset      : Structured array with the list of files in the work directory
%     .EPwork     : The path of the work directory.
%     .dataName   : The list of dataset names.
%  theDataset     : Which dataset to load in.
%
%Outputs:
%  EPdata         : Structured array with the data and accompanying information.  See readData.

%History:
%  by Joseph Dien (7/11/09)
%  jdien07@mac.com
%
%  bugfix 8/3/09 JD
%  File names with multiple periods would confuse Matlab about the file type,
%  resulting in it trying to load the file as an ASCII file rather than as a .mat file.
%
%  modified 2/24/10 JD
%  Backward compatibility for changes in EPdata format.
%
%  modified 10/12/10 JD
%  Backward compatibility for shift to epochwise format for analysis fields for continuous data files.
% 
%  modified 10/17/10 JD
%  Added support for saccadeTrials and saccadeOnset fields.
%
%     Copyright (C) 1999-2011  Joseph Dien
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if theDataset > length(EPdataset.dataset)
    msg{1}='Error: there is no dataset corresponding to the one requested.';
    [msg]=ep_errorMsg(msg);
    return
end;

if isempty(theDataset)
    msg{1}='Error: which dataset to load in was not specified.';
    [msg]=ep_errorMsg(msg);
    return
end;

eval(['load ''' EPdataset.EPwork filesep 'EPwork' filesep EPdataset.dataset(theDataset).dataName '.mat'';']);

%backward compatibility
if isfield(EPdata,'implicit')
    if isfield(EPdata.implicit,'chantype')
        EPdata.implicit=rmfield(EPdata.implicit,'chantype');
    end;
end;

if ~isfield(EPdata,'analysis')
    EPdata.analysis=[];
end;

numSubs=length(EPdata.subNames);
numWaves=length(EPdata.cellNames);
numChan=length(EPdata.chanNames);

numEpochs=numWaves;
if strcmp(EPdata.dataType,'continuous')
    numEpochs=floor(size(EPdata.data,2)/EPdata.Fs); %excess time points are tacked onto final epoch
    if numEpochs == 0
        numEpochs =1;
    end;
end;

if ~isfield(EPdata.analysis,'blinkTrial')
    EPdata.analysis.blinkTrial=zeros(numSubs,numEpochs);
end;
if ~isfield(EPdata.analysis,'saccadeTrial')
    EPdata.analysis.saccadeTrial=zeros(numSubs,numEpochs);
end;
if ~isfield(EPdata.analysis,'saccadeOnset')
    EPdata.analysis.saccadeOnset=zeros(numSubs,numEpochs);
end;
if ~isfield(EPdata.analysis,'moveTrial')
    EPdata.analysis.moveTrial=zeros(numSubs,numEpochs);
end;
if ~isfield(EPdata.analysis,'badTrials')
    EPdata.analysis.badTrials=zeros(numSubs,numEpochs);
end;
if ~isfield(EPdata.analysis,'badChans')
    EPdata.analysis.badChans=zeros(numSubs,numEpochs,numChan);
end;

if (size(EPdata.data,2) >= EPdata.Fs*2) && size(EPdata.analysis.blinkTrial,2) ==1 && strcmp(EPdata.dataType,'continuous')
    EPdata.analysis.blinkTrial=zeros(numSubs,numEpochs);
    EPdata.analysis.saccadeTrial=zeros(numSubs,numEpochs);
    EPdata.analysis.saccadeOnset=zeros(numSubs,numEpochs);
    EPdata.analysis.moveTrial=zeros(numSubs,numEpochs);
    EPdata.analysis.badTrials=zeros(numSubs,numEpochs);
    EPdata.analysis.badChans=zeros(numSubs,numEpochs,numChan);
    disp('Resetting artifact correction fields to zero to update to epochwise approach to continuous files.');
end;



