function EPdataset=ep_saveEPdataset(EPdataset,EPdata,entryNumber,fileSaved);
%  EPdataset=ep_saveEPdataset(EPdataset,EPdata,entryNumber,fileSaved);
%       Adds a new dataset to the work directory.
%
%Inputs:
%  EPdataset      : Structured array with the list of files in the work directory
%     .EPwork     : The path of the work directory.
%     .dataName   : The list of dataset names.
%  EPdata         : Structured array with the data and accompanying information.  See readData.
%  entryNumber    : Where to enter the dataset in the dataset list.  Defaults to the end of the list.
%  saved          : Has the latest version of the file been saved?  Defaults to yes.
%
%Outputs:
%  EPdataset        : Structured array with the list of files in the work directory

%History:
%  by Joseph Dien (7/29/09)
%  jdien07@mac.com
%
%  bugfix 8/3/09 JD
%  Was dropping suffixes even though readData had already dropped the file suffix, resulting in truncation of file names
%  with multiple periods in the name.  Also, file names with multiple periods would confuse Matlab about the file type,
%  resulting in it saving the file as a ASCII file rather than as a .mat file.
%
%  modified 10/31/09 JD
%  Added more information to EPdataset to speed up main pane and preprocess pane.
%
% modified 6/15/10 JD
% Marks when a file isn't saved yet.

%
%     Copyright (C) 1999-2011  Joseph Dien
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin<3
    entryNumber=length(EPdataset.dataset)+1;
end;

if nargin<4
    fileSaved='yes';
end;

if isempty(EPdata.dataName)
    EPdata.dataName='PCA';
end;
sameName=1;
suffix=0;
tempName=EPdata.dataName;
while sameName
    sameName=0;
    for i=1:length(EPdataset.dataset)
        if strcmp(EPdataset.dataset(i).dataName,tempName)
            sameName=1;
        end;
    end;
    if sameName
        suffix=suffix+1;
        tempName=[tempName '-' num2str(suffix)];
    end;
end;
EPdata.dataName=tempName;

newDataset=ep_addToEPworkCache(EPdata);

if ~exist([EPdataset.EPwork filesep 'EPwork'],'dir')
    msg{1}='The work directory cannot be found.';
    [msg]=ep_errorMsg(msg);
    return
end;

EPdataset=ep_checkEPworkCache(EPdataset);

EPdataset.dataset=[EPdataset.dataset(1:entryNumber-1) newDataset EPdataset.dataset(entryNumber:end)];

EPdataset.dataset(entryNumber).saved = fileSaved;

eval(['save ''' EPdataset.EPwork filesep 'EPwork' filesep 'EPdataset'' EPdataset']);
eval(['save ''' EPdataset.EPwork filesep 'EPwork' filesep EPdata.dataName '.mat'' EPdata;']);


