function ep_initEP;

% ep_initEP - ep_initEP -
%          Set up EP Toolkit and check for installed software packages.

%History
%  by Joseph Dien (1/31/09)
%  jdien07@mac.com
%
%  bugfix 8/3/10 JD
%  Fixed crash when EP Toolkit folder name has been changed.

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

global EP_COPYRIGHT_NOTE EP_VER

try
    
    EPpath=which('ep.m');
    [pathstr, name, ext]=fileparts(EPpath);
    [a b]=strtok(pathstr,filesep);
    b=b(2:end);
    EPver=ver(b);
    
    MATLABver=ver('MATLAB');
    [a b]=strtok(MATLABver.Version,'.');
    b=b(2:end);
    
    if (str2num(a) < 7) || ((str2num(a) == 7) && (str2num(b) < 3)) %only support Matlab 7.3 onwards
        disp(' ');
        disp('**************************************************************');
        disp('Warning: The EP Toolkit is only supported for versions 7.3 onwards.');
        disp('You are welcome to try it out but please do not ask me for help with any problems you may encounter.');
        disp('**************************************************************');
        disp(' ');
    end;
catch
    EPver.Version='';
    MATLABver='unavailable'; %workaround for bug in earlier version of Matlab
end;

if ~exist('EP_VER','var')
    EP_VER='';
end;

if isempty(EP_COPYRIGHT_NOTE) || ~strcmp(EPver.Version,EP_VER)
    disp(' ');
    disp(['ERP PCA Toolbox ' EPver.Version ' Copyright (C) 1999-2011  Joseph Dien']);
    disp('This program comes with ABSOLUTELY NO WARRANTY.');
    disp('This is free software, and you are welcome to redistribute it');
    disp('under certain conditions; see http://www.gnu.org/licenses/ for details.');
    disp(' ');
    eval('which ft_read_header')
    eval('which runica')
    
    EP_COPYRIGHT_NOTE='Y';
    EP_VER=EPver.Version; %ensure that the copyright notice will be printed again if the version is updated during a session.
end;

if ~exist('ft_read_header','file') && exist('read_header','file')
    disp(' ');
    disp('**************************************************************');
    disp('Warning: Fieldtrip needs to be updated a current version.');
    disp('**************************************************************');
    disp(' ');
end;

if ~exist('ft_read_header','file') || ~exist('ft_read_data','file') || ~exist('ft_read_event','file')
    disp(' ');
    disp('**************************************************************');
    disp(['Warning: Fieldtrip has either not been installed or put in the path, which may cause problems importing files.']);
    disp('Remember to add its subdirectories too, not just the main directory.');
    disp(['See tutorial file for details.']);
    disp('**************************************************************');
    disp(' ');
else
    fieldTripLoc=which('ft_read_header');
    if ~isempty(findstr(fieldTripLoc,['external' filesep 'fieldtrip']))
        disp(' ');
        disp('**************************************************************');
        disp('Warning: You appear to be using the version of Fieldtrip that comes with EEGlab (in its external directory).');
        disp('It is recommended that you delete it and download the current version of FieldTrip.');
        disp(['See tutorial file for details.']);
        disp('**************************************************************');
        disp(' ');
    end;
    theLoc=findstr(which('ft_read_header'),'fieldtrip');
    if str2num(fieldTripLoc(theLoc+10:theLoc+17)) < 20110326
        disp(' ');
        disp('**************************************************************');
        disp('Warning: You appear to be using a version of FieldTrip that is too old to work properly with the EP Toolkit.');
        disp(['It dates back to: ' datestr([str2num(fieldTripLoc(theLoc+10:theLoc+13)) str2num(fieldTripLoc(theLoc+14:theLoc+15)) str2num(fieldTripLoc(theLoc+16:theLoc+17)) 0 0 0])])
        disp('This may cause problems importing files.');
        disp('It is recommended that you delete it and download the current version of FieldTrip.');
        disp('If you just downloaded it, try looking again.  If the file names were ordered by name rather than date and you took the top one, then that was the oldest version.')
        disp(['See tutorial file for details.']);
        disp('**************************************************************');
        disp(' ');
    end;
    cellfunLoc=which('cellfun');
    if ~isempty(findstr(cellfunLoc,['compat' filesep 'R']))
        disp(' ');
        disp('**************************************************************');
        disp('Warning: There is a copy of cellfun.m in the Fieldtrip folder that needs to be deleted immediately or it will destabilize Matlab.');
        disp('It is located in the comp directory, in folders named R13 and/or R14.');
        disp('It is recommended that you delete it immediately.');
        disp(['See tutorial file for details.']);
        disp('**************************************************************');
        disp(' ');
    end;
end;

if exist('icadefs','file')
    icadefs;
    if exist(ICABINARY) == 2
        [pathstr, name, ext]=fileparts(ICABINARY);
        if ismac && ~(strcmp(name,'ica_osx') || strcmp(name,'ica_osxG5'))
            disp(['Warning: You have a binary ICA program installed.  The file is called: ' name]);
            disp('It is not one of the two that is appropriate for OS X.');
        end;
        try
            if isunix && ~ismac && ~(strcmp(name,'ica_linux') || strcmp(name,'ica_bsd'))
                disp(['Warning: You have a binary ICA program installed.  The file is called: ' name]);
                disp('It is not one of the two that is appropriate for UNIX.');
            end;
        catch
        end
        if ispc
            disp(['Warning: You have a binary ICA program installed.  The file is called: ' name]);
            disp('There is not currently one that is appropriate for Windows.');
        end;
        if ~isempty(findstr(which('binica'),['external' filesep 'eeglab']))
            disp(' ');
            disp('**************************************************************');
            disp('Warning: You appear to be using the version of binica that comes with Fieldtrip (in its external directory).');
            disp('It is recommended that you delete it and download the current version of EEGlab.');
            disp(['See tutorial file for details.']);
            disp('**************************************************************');
            disp(' ');
        end;
    else
        if ~isempty(findstr(which('runica'),['external' filesep 'eeglab']))
            disp(' ');
            disp('**************************************************************');
            disp('Warning: You appear to be using the version of runica that comes with Fieldtrip (in its external directory).');
            disp('It is recommended that you delete it and download the current version of EEGlab.');
            disp(['See tutorial file for details.']);
            disp('**************************************************************');
            disp(' ');
        end;
    end
else
    disp(' ');
    disp('**************************************************************');
    disp(['Warning: ICA is not installed.  You will need to install it and add it to the path,']);
    disp(['otherwise you will not be able to use the Infomax ICA rotation.  See tutorial.']);
    disp('Remember to add its subdirectories too, not just the main directory.');
    disp('**************************************************************');
    disp(' ');
    beep();
end;

if exist('biosig/NaN','dir')
    disp(' ');
    disp('**************************************************************');
    disp(['Warning: The Biosig directory "NaN" is in the path and may cause problems.']);
    disp(['It is recommended that it be removed from the path.']);
    disp('**************************************************************');
    disp(' ');
end;

if exist('biosig/maybe-missing','dir')
    disp(' ');
    disp('**************************************************************');
    disp(['Warning: The Biosig directory "maybe-missing" is in the path and may cause problems.']);
    disp(['It is recommended that it be removed from the path.']);
    disp('**************************************************************');
    disp(' ');
end;

scrsz = get(0,'ScreenSize');

if scrsz(4) < 500
    disp(' ');
    disp('**************************************************************');
    disp(['The current vertical screen height of ' num2str(scrsz(4)) ' pixels is too low.  It needs to be at least 500.']);
    disp('**************************************************************');
    disp(' ');
end;

