function theTable=ep_noTable(theTable);

% theTable=ep_noTable(theTable);
%          Provides a replacement for uitable for earlier versions of
%          Matlab that don't support it.

%History
%  by Joseph Dien (7/26/09)
%  jdien07@mac.com
%

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

Xloc=theTable.Xloc; %the X-coordinate of the table
Yloc=theTable.Yloc; %the Y-coordinate of the table
nRows=theTable.nRows; %number of rows in the table
nCols=theTable.nCols; %number of columns in the table
colWidth=theTable.colWidth; %width of columns in the table
rowNames=theTable.rowNames;
tableNames=theTable.tableNames;
tableData=theTable.tableData;
columnEditable=theTable.columnEditable;
ColumnFormat=theTable.ColumnFormat;
selRows=theTable.selRows; %which rows should be displayed
selCols=theTable.selCols; %which columns should be displayed
chgRows=theTable.chgRows; %which rows can be changed
chgCols=theTable.chgCols; %which columns can be changed
name=theTable.name;
endHeaderCallback=theTable.endHeaderCallback;
endTableCallback=theTable.endTableCallback;

%put up column headers
for theCol=1:nCols
    if chgCols(theCol)
        theTable.handles.colHeader(theCol)=uicontrol('Style','popupmenu',...
            'String',tableNames,'HorizontalAlignment','left',...
            'Value',selCols(theCol),...
            'Callback', ['global EPtable;','EPtable.' name '.selCols(' num2str(theCol) ')=get(EPtable.' name '.handles.colHeader(' num2str(theCol) '),''Value'');',endHeaderCallback],...
            'Position',[Xloc+theCol*(colWidth+5) Yloc colWidth 20]);
    else
        theTable.handles.colHeader(theCol)=uicontrol('Style','text',...
            'String',tableNames{theCol},'HorizontalAlignment','left',...
            'Position',[Xloc+theCol*(colWidth+5) Yloc colWidth 20]);
    end;
end;

for theRow=1:nRows
    %put up the row headers
    if chgRows(theRow)
    theTable.handles.rowHeader(theRow)=uicontrol('Style','popupmenu',...
        'String',rowNames,'HorizontalAlignment','left',...
        'Value',selRows(theRow),...
        'Callback', ['global EPtable;','EPtable.' name '.selRows(' num2str(theRow) ')=get(EPtable.' name '.handles.rowHeader(' num2str(theRow) '),''Value'');',endHeaderCallback],...
        'Position',[Xloc Yloc-theRow*20 colWidth+5 20]);
    else
    theTable.handles.rowHeader(theRow)=uicontrol('Style','text',...
        'String',rowNames{theRow},'HorizontalAlignment','left',...
        'Position',[Xloc Yloc-theRow*20 colWidth+5 20]);
    end;
    %put up the table contents
    for theCol=1:nCols
        if columnEditable(theCol)
            switch ColumnFormat{theCol}
                case 'char'
                    theTable.handles.tableData(theRow,theCol)=uicontrol('Style','edit',...
                        'String',tableData{selRows(theRow),selCols(theCol)},'HorizontalAlignment','left',...
                        'Callback', ['global EPtable;','EPtable.' name '.tableData{' num2str(selRows(theRow)) ',' num2str(selCols(theCol)) '}=(get(EPtable.' name '.handles.tableData(' num2str(theRow) ',' num2str(theCol) '),''String''));',endTableCallback],...
                        'Position',[Xloc+theCol*(colWidth+5) Yloc-theRow*20 colWidth 20]);
                case 'logical'
                    theTable.handles.tableData(theRow,theCol)=uicontrol('Style','checkbox',...
                        'Value',tableData{selRows(theRow),selCols(theCol)},...
                        'Callback', ['global EPtable;','EPtable.' name '.tableData{' num2str(selRows(theRow)) ',' num2str(selCols(theCol)) '}=(get(EPtable.' name '.handles.tableData(' num2str(theRow) ',' num2str(theCol) '),''Value''));',endTableCallback],...
                        'Position',[Xloc+theCol*(colWidth+5) Yloc-theRow*20 colWidth 20]);
                case 'numeric'
                    theTable.handles.tableData(theRow,theCol)=uicontrol('Style','edit',...
                        'String',num2str(tableData{selRows(theRow),selCols(theCol)}),...
                        'Callback', ['global EPtable;','EPtable.' name '.tableData{' num2str(selRows(theRow)) ',' num2str(selCols(theCol)) '}=str2num(get(EPtable.' name '.handles.tableData(' num2str(theRow) ',' num2str(theCol) '),''String''));',endTableCallback],...
                        'Position',[Xloc+theCol*(colWidth+5) Yloc-theRow*20 colWidth 20]);
            end;
        else
            theData=tableData{selRows(theRow),selCols(theCol)};
            if isnumeric(theData)
                theData=num2str(theData);
            end;
            uicontrol('Style','text',...
                'String',tableData{selRows(theRow),selCols(theCol)},'HorizontalAlignment','left',...
                'Position',[Xloc+theCol*(colWidth+5) Yloc-theRow*20 colWidth 20]);
        end
    end
end
