% optFIR plugin
%
% Plugin for EEGlab to implement optimal FIR filter via Cook and Miller 
% Optimal Digital Filtering approach (with hamming option applied 
% when truncating filter to final filter order). See: 
%
% Cook, E.W., & Miller, G.A. (1992).  Digital Filtering: Background and tutorial for 
%   psychophysiologists. Psychophysiology, 3, 350-367
%
% Also allows comparison to EEGlab built in filters eegfilt and iirfilt.
%
% Requires Signal Processing Toolbox to be installed.  
%
%
% Version 0.9
%
% Written July, 2011, by John JB Allen jallen@u.arizona.edu
%
% See Also:  pop_optFIR, pop_optFIR_compare, pop_about_optFIR_compare, optFIR.m, optFIR_compare, optFIR_design, FIR_filt, FIR_freq_rsps, IIR_freq_rsps


function eegplugin_optFIR( fig, try_strings, catch_strings);

% create menu
filtmenu = findobj(fig, 'tag', 'filter'); % Nests under filter submenu of tools menu
% toolsmenu = findobj(fig, 'tag', 'tools'); % would put it within tools main menu


% construct commands
optFIR_cmd = [try_strings.no_check  '[EEG LASTCOM]= pop_optFIR(EEG);' catch_strings.new_and_hist];
optFIR_compare_cmd = [try_strings.no_check  '[filtspecs LASTCOM]= pop_optFIR_compare(EEG);' catch_strings.add_to_hist];
about_optFIR_cmd = [try_strings.no_check  'pop_about_optFIR_compare;' catch_strings.add_to_hist];

% add menu items
menutextcolor = [0.5 0 1];
optFIR_menu=uimenu(filtmenu,'label','Optimal FIR Filtering (plugin)','separator','on','ForegroundColor', menutextcolor, 'tag','Opt FIR Tools'); 
uimenu(optFIR_menu, 'label', 'Filter data with Optimal FIR','separator','on','ForegroundColor', menutextcolor, ...
    'callback', optFIR_cmd);
uimenu(optFIR_menu, 'label', 'Compare Optimal FIR to Basic FIR and Short IIR','ForegroundColor', menutextcolor, ...
    'callback', optFIR_compare_cmd);
uimenu(optFIR_menu, 'label', 'About optFIR plugin','ForegroundColor', menutextcolor, ...
    'callback', about_optFIR_cmd);
