% Simple function that takes all the outputs from optFIR_compare and puts
%   them neatly in a single structure
%
% Usage:
%   filtspecs = filtspecs2struc(filtspecs, field_name, filtered_data_structure, fs, m)
%
% Where:
%   filtspecs is structure that will hold data (can be empty or have fields already)
%   fieldname is a string that will become the field in the structure
%       (should match name of the filtered data structure you pass in)
%   filtered_data_structure is EEG type structure that has extra fields
%       generated during the optFIR_compare procedures.
%
% Written July, 2011, by John JB Allen jallen@u.arizona.edu
%
% See Also:  pop_optFIR_compare, pop_optFIR_design, optFIR_design, pop_optFIR, optFIR


function filtspecs = filtspecs2struc(filtspecs, field_name, filtered_data_structure, fs, m, lowcut, highcut, notch)

% all the usual suspects...
filtspecs.(genvarname(field_name)).lowcut = lowcut;
filtspecs.(genvarname(field_name)).highcut = highcut;
filtspecs.(genvarname(field_name)).notch = notch;
filtspecs.(genvarname(field_name)).f_axis = filtered_data_structure.f_axis;
filtspecs.(genvarname(field_name)).gain = filtered_data_structure.gain;
filtspecs.(genvarname(field_name)).h = filtered_data_structure.h;
filtspecs.(genvarname(field_name)).w = filtered_data_structure.w;
filtspecs.(genvarname(field_name)).ideal_actual_corr = filtered_data_structure.ideal_actual_corr;
filtspecs.(genvarname(field_name)).fs = fs;
filtspecs.(genvarname(field_name)).m = m;
filtspecs.(genvarname(field_name)).time_to_filter = filtered_data_structure.time_to_filter;




if any(strcmp('cmd',fieldnames(filtered_data_structure)))% only coded for optFIR currently
    filtspecs.(genvarname(field_name)).cmd = filtered_data_structure.cmd;
end

% one complication
if any(strcmp('fweights',fieldnames(filtered_data_structure)))% won't exist for iirfilt
filtspecs.(genvarname(field_name)).fweights = filtered_data_structure.fweights;
end

