function ep(varargin);
% ep - function ep(varargin);
% GUI interface for the EP Toolkit.
%

%History
%  by Joseph Dien (7/30/09)
%  jdien07@mac.com
%
%  bugfix 8/2/09 JD
%  EPdataset.dataset initialized as an empty string rather than as a cell, which was causing a crash when adding the
%  first dataset.
%
%  modified 8/3/09 JD
%  Consolidated EGI EGIS average and session entries in the file format list.
%
%  bugfix 8/11/09 JD
%  For artifact correction, bug resulted in EGI EGIS session format always being used as input format.  Thanks
%  to Grega Repov.
%
%  bugfix 8/27/09 JD
%  Location of windows appearing partly off screen on some systems.
%  ANOVA preferences pane crash fixed and crash when doing ANOVA with only one between group and add option on.  Thanks to Tim Curran.
%  File title missing from first ANOVA in output files.  Thanks to Tim Curran.
%
%  modified 8/30/09 JD
%  Added preferences for importing text files.
%
%  bugfix and modified 9/5/09 JD
%  Crash when generating spatial PCA.  Thanks to Tim Curran.
%  Move factors preference were not used for preprocessing. 
%  Preprocess controls now remember settings.
%  Single cell files option for preprocessing were not functional.
%  Added reference channel field to preprocessing pane to deal with ICA problems when processing mean mastoid data.
%  Blink file was not passed on to blink correction function for fileTemplate option.
%  Crash when saving preferences file on a non-Mac.
%  Dataset information like the cells were not updating when changing datasets in the Window Data Pane.
%
%  bugfix 9/10/09 JD
%  Fixed not saving file when averaging session files and outputting as EGIS format.
%  Fixed crash when preprocessing data and using autoTemplate for blinks.
%
%  bugfix 9/23/09 JD
%  Not finding EPwork directory when in the search path but not in the active directory.
%
%  bugfix 10/12/09 JD
%  Fixed crash when using single file mode with Read function.
%
%  modified & bugfix 10/28/09 JD
%  Added option to disable preprocessing figure for low memory situations.
%  Movement correction preferences not registering.
%
%  modified 10/31/09 JD
%  When reading multiple files, assume all have the same electrode location and montage information rather than
%  repeatedly asking for them.
%  Cached more information in EPdataset to avoid slowdowns in main pane and preprocess pane.
%
%  modified 10/31/09 JD
%  checks to see if EPwork cache is current and regenerates it if not.
%
%  bugfix 11/2/09 JD
%  Fixed crash when using single file mode with Read function.
%
%  bugfix 11/4/09 JD
%  Fixed crash when running PCA scree and no noise PCA was conducted.  Thanks to Jia Wu.
%
%  bugfix 11/12/09 JD
%  Fixed bug where Toolkit was saving all variables into EPwork cache, not just EPdataset, resulting in erratic problems
%  when old values for variables were loaded back into the workspace.
%
%  bugfix 12/3/09 JD
%  Added workaround for sporadic Matlab menu bug that was causing menu to disappear or crash when menu item that is selected is
%  same as before.
%
%  bugfix 12/10/09 JD
%  Fixed crash when viewing Window pane for a dataset with less than four cells under Matlab versions prior to 2008.
%
%  bugfix 1/8/10 JD
%  Fixed crash when entering between group name that is not three letters long.
%  Fixed failure due to Matlab bug to redraw View pane when changing the dataset for one of the colors.
%
%  bugfix & modified 1/17/10 JD
%  Fixed crash when examining a file in the View pane with less than four cells.
%  Modified ANOVA module to accommodate non-ERP data by adding support for "behavioral" keyword.
%
%  modified 1/28/10 JD
%  Can leave out subject and cell fields when reading files in single file mode
%  (will use default values instead and will assume all the subjects/cells are the same).
%  Can accomodate file formats such as Neuroscan .avg where channels can be deleted, not just zeroed out, if bad and
%  channels can appear in an order different than that of the .ced file.  When importing a file with missing channels,
%  will now add them in as zeroed out channels and will mark them as bad.  Will also reorder channels to match that of
%  the ced file.
%  The type field in the .ced files is now used since the EEGlab bug was apparently fixed and it is now functional.  The
%  type field must now be present in the .ced file and assumptions will no longer be made about which ones are REF or FID types.
%  Added ability to plot data from continuous data files by showing only one second at a time.
%  Epoch ms now displayed as from beginning of first sample to end of last sample.
%  Fixed bug where checkboxes and subject and cell fields on Read and Preprocess panes impossible to deselect.
%  Refreshes Edit pane when Done button is pressed so when Data Name is changed, the name in the list of datasets is updated.
%
%  bugfix and modified 2/28/10 JD
%  Average function now correctly checks to see if output file name already exists for EP format files.
% analysis fields no longer optional.
% Now has option to use file's default reference settings in the preprocess pane.
% When running ANOVAs with between subject factors, fixed the addition of grand averages corresponding to the levels of the factors to the dataset.
% Made autoPCA more memory efficient so it wouldn't run out of memory.
% Fixed crash in ANOVA function when level names of between group factors were of different lengths.
% Close files after running batch of ANOVAs to avoid "too many files open" error.
% When running ANOVAs with between subject factors using the output of the autoPCA option, fixed the addition of grand averages corresponding to the levels of the factors to the dataset.
% Corrected the peak channel and peak time point identification of factors made by the autoPCA option of the window function.
% Fixed bug that prevented one from fully deleting the contents of edit fields, such as "baseline" on the preprocessing pane.
% Fixed bug that prevented one from changing the contents of the edit fields on the postprocessing pane.
% Fixed inability to set minimum variance setting for autoPCA preference to anything other than zero.
% Added option to turn off adding montage information to EGIS file format files due to incompatibility issues with some
% versions of NetStation.
%
% bugfix 3/6/10 JD
% When running ANOVAs with between subject factors, fixed having only the first between group waveform being added to
% the datafile.
%
% modified 3/27/10 JD
% Added ability to save data using .set file format.
%
% bugfix 4/26/10 JD
% Fixed wrong peak chans and time points for AutoPCA when not two-step PCA.
% Added topos function to the View Pane.
%
% bugfix 5/5/10 JD
% Fixed crash when in Window Data pane using noTable mode (older versions of Matlab) and there are less than five
% subject specs.
% Fixed not changing table of cells and table of specs when in Window Data pane using noTable mode (older versions of
% Matlab).
% Fixed crash when using Topo button of View EEG pane and not all four colors are being used.
% Added option to use unrotated solution for PCAs.
%
% modified 5/22/09 JD
% Number of factors and title of PCA no longer being changed to blank when changing other PCA settings.
% Added support for EEGlab .study files.
%
% bugfix 6/5/10 JD
% Fixed crash when postprocessing.
% View function no longer crashes when average file erroneously has non-empty trial names field.
% Fixed files sometimes not being recognized as being selected when names are in uppercase.
%
% modified & bugfix 6/17/10 JD
% Marks when a file isn't saved yet.
% Changed the suffix of EP files to ".ept".
% Fixed crash when windowing adds regional channel and there is no electrode coordinate information (eloc).
% Fixed crash when conducting robust ANOVA and there are no spec columns in the ANOVA data file.
%
% bugfix 6/28/10 JD
% Fixed when no within factors for ANOVAs, pane indicated needed one within cell rather than zero wthin cells.
%
% bugfix 7/2/10 JD
% In order fix crash due to Matlab bug on Windows, put in brief pause after spec name requestor diaglog box.
%
% bugfix 7/23/10 JD
% Fixed view pane listing all the trial names of the dataset rather than just those for a specific cell, resulting in
% crashes when they were selected.
% Fixed crash when viewing Waves and data are all zero.
% Fixed grand averages, which are added when computing ANOVAs with between factors, being computed incorrectly (only
% affected waveforms, not the ANOVA results).
%
% bugfix 8/27/10 JD
% Fixed unable to output averages in EGIS format.
% 
%  modified 10/17/10 JD
%  Added support for saccadeTrials and saccadeOnset fields.
% 
%  modified 10/28/10 JD
%  Added support for saccade preprocessing preference settings.  Removed the timepoints option from the preprocessing
%  pane.
%
% bugfix 12/7/10 JD
% Fixed crash in Window function after switching to a dataset with fewer cells.
% Fixed crash in ANOVA function when performing ANOVA with no between group factors under Matlab 2008b.
%
% bugfix 1/5/11 JD
% Fixed crash in ANOVA function when used prior to data being loaded into the work set yet (due to badly initialized variable).
% Now putting preferences file and work directory at default user directory if old one cannot be found.
% 
% modified 1/18/11 JD
% Added support for selecting timepoints in preprocessing (previous implementation was non-functional).
% 
% modified 1/20/11 JD
% Added support for manually specifying EOG channels in the preferences.
%
% bugfix 2/3/11 JD
% Fixed crash when starting program on a computer with spaces in the default user path, as in "Documents and Settings"
%
% bugfix 2/4/11 JD
% Hopefully fixed crash when starting program with certain configurations of EPwork and EPprefs.
%
% bugfix 2/8/11 JD
% Hopefully fixed crashes from buggy reset of preferences.
%
% bugfix 2/10/11 JD
% Fixed crash when performing eyeblink correction due to change in preferences file.
% Fixed not using saccade preference settings.
% Fixed crashing when loading EPdataset files.
% 
% modified 1/20/11 JD
% Added commands to change or create EPwork directory from the EP menu.
% Only stores EPpref files in EPwork directories.
%
% modified 2/26/11 JD
% Added support for EGI Matlab files

%     Copyright (C) 1999-2011  Joseph Dien
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

global EPdataset EPmain EPoverview EPwaves EPscree EPchanGrp EPtable EPtopos

fileFormatReadList={'EGI EGIS (.egis)','EGI Simple Binary (.sbin)','EEGlab (.set/.study)','Biosig EDF (.edf)','Neuroscan (.cnt/.eeg/.avg)','text (.txt)','EP (.ept)','EGI Matlab (.nsf)'};
fileFormatSaveList={'EGI EGIS (.egis)','EGI Simple Binary (.sbin)','EEGlab (.set/.study)','text (.txt)','EP (.ept)'};
refList={'ave ref','no ref','1-2 refs','default'};

if isempty(varargin)
    
    if ~exist('ep_initEP','file')
        msg{1}=['Error: The rest of the EP Toolkit is not on the path.  Did you use'];
        msg{2}=['the ''Add with Subfolders'' button rather than the ''Add Folder'' button?'];
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    if ~isempty(EPmain)
        if isfield(EPmain,'mode')
            if ~any(strcmp(EPmain.mode,{'agree','disagree'}))
                EPmain=[];
            end;
        else
            EPmain=[];
        end;
    end;
    if isempty(EPmain)
        splash
        return
    elseif strcmp(EPmain.mode,'disagree')
        close('About EP Toolkit')
        EPmain=[];
        return
    else
        close('About EP Toolkit')
        EPmain=[];
    end
    
    ep_initEP;
    
    EPmain=[];
    
    PathName=userpath;
    if any(strcmp(PathName(end),{';',':'}))
        PathName=PathName(1:end-1);
    end;
    
    %set up work directory
    if exist([pwd filesep 'EPwork' filesep 'EPdataset.mat'],'file') %if there is already a work directory in the active directory, use it.
        eval(['load EPwork' filesep 'EPdataset.mat']);
        EPdataset.EPwork=pwd;
    elseif exist(['~/Documents/EPwork' filesep 'EPdataset.mat'],'file') %on a Mac, work directories kept in this location
        eval(['load ~/Documents/EPwork' filesep 'EPdataset.mat']); %if there is already a work directory, use it.
        EPdataset.EPwork='~/Documents';
    elseif exist([PathName filesep 'EPwork' filesep 'EPdataset.mat'],'file')
        eval(['load ''' PathName filesep 'EPwork' filesep 'EPdataset.mat''']); %if there is already a work directory, use it.
        EPdataset.EPwork=PathName;
    else
        if isempty(PathName)
            userpath('reset')
            disp('Resetting userpath as it was set to root');
        end;
        mkdir([PathName filesep 'EPwork']);
        EPdataset=[];
        EPdataset.EPwork=PathName;
        EPdataset.dataset=cell(0);
        eval(['save ''' PathName filesep 'EPwork' filesep 'EPdataset'' EPdataset']);
    end;
    
    %set up preferences
%     if exist('~/Library/Preferences/EPprefs.mat','file') %on a Mac, preferences file kept in this location
%         load '~/Library/Preferences/EPprefs.mat';
%         EPmain.preferences=prefs;
%     elseif exist('EPprefs.mat','file') %if the preferences file is being kept in the active directory
%         load EPprefs;
%         EPmain.preferences=prefs;
    if exist([PathName filesep 'EPwork' filesep 'EPprefs.mat'],'file')    
        load([PathName filesep 'EPwork' filesep 'EPprefs.mat']);
        EPmain.preferences=prefs; 
    else
        resetPrefs;
        prefs=EPmain.preferences;
        eval(['save ''' PathName filesep 'EPwork' filesep 'EPprefs'' prefs']);
    end;
    
    err=checkPrefs;
    
    if err
        disp('Preferences are corrupt or out-of-date.  Resetting to default values.');
        resetPrefs;
    end;
    
    EPdataset=ep_checkEPworkCache(EPdataset);
    
    varargin{1}='start';
    EPoverview=[];
    EPmain.handles.hMainWindow =[];
    EPmain.noTable=0;
    EPmain.mode='main';
    EPmain.average.importFormat=EPmain.preferences.general.sessionImportFormat;
    EPmain.average.type=2;
    EPmain.average.outputFormat=EPmain.preferences.general.outputFormat;
    EPmain.average.method=EPmain.preferences.average.method;
    EPmain.postprocess.importFormat=EPmain.preferences.general.importFormat;
    EPmain.postprocess.type=3;
    EPmain.postprocess.outputFormat=EPmain.preferences.general.outputFormat;
    EPmain.postprocess.reference=EPmain.preferences.postprocess.reference;
    EPmain.postprocess.refChan1=EPmain.preferences.postprocess.refChan1;
    EPmain.postprocess.refChan2=EPmain.preferences.postprocess.refChan2;
    EPmain.postprocess.baseline=EPmain.preferences.postprocess.baseline;
    EPmain.read.check=0;
    EPmain.read.subject='';
    EPmain.read.cell='';
    EPmain.read.format=EPmain.preferences.general.importFormat;
    EPmain.read.type=3;
    EPmain.preprocess.importFormat=EPmain.preferences.general.sessionImportFormat;
    EPmain.preprocess.outputFormat=EPmain.preferences.general.sessionOutputFormat;
    EPmain.preprocess.timepoints='';
    EPmain.preprocess.baseline='';
    EPmain.preprocess.detrend=0;
    EPmain.preprocess.blinkTemplate=1;
    EPmain.preprocess.saccadeTemplate=2;
    EPmain.preprocess.channelMode=1;
    EPmain.preprocess.trialMode=1;
    EPmain.preprocess.check=0;
    EPmain.preprocess.subject='';
    EPmain.preprocess.cell='';
    EPmain.preprocess.type=2;
    EPmain.preprocess.reference='';
    EPmain.preprocess.refType=4;
    
    EPmain.view=[];
    
    EPmain.pca.facNum=0;
    EPmain.pca.name='';
    EPmain.pca.mode=EPmain.preferences.pca.mode;
    EPmain.pca.rotation=EPmain.preferences.pca.rotation;
    EPmain.pca.rotopt=EPmain.preferences.pca.rotopt;
    EPmain.pca.rel=EPmain.preferences.pca.rel;
    EPmain.pca.loadings=EPmain.preferences.pca.loadings;
    EPmain.pca.parametric=false;
    EPscree=[];
    
    EPmain.window.minFacVar=[];
    
    EPmain.anova.numComps=0;
    EPmain.anova.data=[];
    
    EPmain.save.format=EPmain.preferences.general.outputFormat;
    EPmain.save.SGLchan=1;
    EPmain.save.REGchan=1;
    EPmain.save.SGLcell=1;
    EPmain.save.CMBcell=1;
    EPmain.save.RAW=1;
    EPmain.save.AVG=1;
    EPmain.save.GAV=1;
    EPmain.save.SGLfac=1;
    EPmain.save.CMBfac=1;
    
    EPchanGrp=[];
    
    EPtable=[];
    
    if err
        ep('savePrefs'); %save reset preference settings
    end;
end;

scrsz = get(0,'ScreenSize');

switch varargin{1}
    case 'start'
        if ~isempty(EPmain.handles.hMainWindow)
            clf(EPmain.handles.hMainWindow)
            figure(EPmain.handles.hMainWindow)
        else
            EPmain.handles.hMainWindow = figure('Name', 'EP Toolkit', 'NumberTitle', 'off', 'Position',[1 scrsz(4)-550 200 500], 'MenuBar', 'none');
            drawnow
        end;
        
        switch EPmain.mode
            case 'main'
                ep('startMain');
                
            case 'preprocess'
                ep('startPreprocess');
                
            case 'average'
                ep('startAverage');
                
            case 'postprocess'
                ep('startPostprocess');
                
            case 'read'
                ep('startRead');
                
            case 'edit'
                ep('startEdit');
                
            case 'view'
                ep('startView');
                
            case 'PCA'
                ep('startPCA');
                
            case 'window'
                ep('startWindow');
                
            case 'ANOVA'
                ep('startANOVA');
                
            case 'save'
                ep('startSave');
                
            case 'preferenceMain'
                ep('startPreferenceMain');
                
            case 'preferenceGeneral'
                ep('startPreferenceGeneral');
                
            case 'preferencePreprocess'
                ep('startPreferencePreprocess');
                
            case 'preferenceAverage'
                ep('startPreferenceAverage');
                
            case 'preferencePostprocess'
                ep('startPreferencePostprocess');
                
            case 'preferenceView'
                ep('startPreferenceView');
                
            case 'preferencePCA'
                ep('startPreferencePCA');
                
            case 'preferenceWindow'
                ep('startPreferenceWindow');
                
            case 'preferenceANOVA'
                ep('startPreferenceANOVA');
                
            otherwise
                error('Not a valid mode.');
        end;
        
    case 'startMain'
        
        screeFigure=findobj('Name', 'ScreeWindow');
        if ~isempty(screeFigure)
            close(screeFigure)
        end;
        
        chanGrpFigure=findobj('Name', 'Channel Group Window');
        if ~isempty(chanGrpFigure)
            close(chanGrpFigure)
        end;
        
        editDataFigure=findobj('Name', 'EditData');
        if ~isempty(editDataFigure)
            close(editDataFigure)
        end;
        
        set(EPmain.handles.hMainWindow,'Name', 'EP Toolkit');
        
        EPmain.handles.hMenu = uimenu('Label','File');
        uimenu(EPmain.handles.hMenu,'Label','About EP Toolkit','Callback',@splash);
        uimenu(EPmain.handles.hMenu,'Label','Preferences','Callback',['global EPmain;','EPmain.mode=''preferenceMain'';','ep(''start'');']);
        uimenu(EPmain.handles.hMenu,'Label','Change Work Directory','Callback',@changeWork);
        uimenu(EPmain.handles.hMenu,'Label','Create Work Directory','Callback',@createWork);
        uimenu(EPmain.handles.hMenu,'Label','Quit','Callback',@quit);
        
        EPmain.handles.main.preprocess = uicontrol('Style', 'pushbutton', 'String', 'Preprocess',...
            'Position', [20 450 100 40], 'Callback', ['global EPmain;','EPmain.mode=''preprocess'';','ep(''start'');']);
        
        EPmain.handles.main.average = uicontrol('Style', 'pushbutton', 'String', 'Average',...
            'Position', [20 400 100 40], 'Callback', ['global EPmain;','EPmain.mode=''average'';','ep(''start'');']);
        
        EPmain.handles.main.postprocess = uicontrol('Style', 'pushbutton', 'String', 'Postprocess',...
            'Position', [20 350 100 40], 'Callback', ['global EPmain;','EPmain.mode=''postprocess'';','ep(''start'');']);
        
        EPmain.handles.main.read = uicontrol('Style', 'pushbutton', 'String', 'Read',...
            'Position', [20 300 100 40], 'Callback', ['global EPmain;','EPmain.mode=''read'';','ep(''start'');']);
        
        EPmain.handles.main.edit = uicontrol('Style', 'pushbutton', 'String', 'Edit',...
            'Position', [20 250 100 40], 'Callback', ['global EPmain;','EPmain.mode=''edit'';','ep(''start'');']);
        
        if isempty(EPdataset.dataset)
            set(EPmain.handles.main.edit,'enable','off');
        end;
        
        EPmain.handles.main.view = uicontrol('Style', 'pushbutton', 'String', 'View',...
            'Position', [20 200 100 40], 'Callback', ['global EPmain;','EPmain.mode=''view'';','ep(''start'');']);
        
        if isempty(EPdataset.dataset)
            set(EPmain.handles.main.view,'enable','off');
        end;
        
        EPmain.handles.main.PCA = uicontrol('Style', 'pushbutton', 'String', 'PCA',...
            'Position', [20 150 100 40], 'Callback', ['global EPmain;','EPmain.mode=''PCA'';','ep(''start'');']);
        
        if isempty(EPdataset.dataset)
            set(EPmain.handles.main.PCA,'enable','off');
        end;
        
        EPmain.handles.main.window = uicontrol('Style', 'pushbutton', 'String', 'Window',...
            'Position', [20 100 100 40], 'Callback', ['global EPmain;','EPmain.window=[];','EPmain.mode=''window'';','ep(''start'');']);
        
        if ~isempty(EPdataset.dataset)
            isAve=false;
            for i=1:length(EPdataset.dataset)
                if strcmp(EPdataset.dataset(i).dataType,'average') && (any(strcmp(EPdataset.dataset(i).chanTypes,'EEG')) || any(strcmp(EPdataset.dataset(i).chanTypes,'MEG'))) && any(strcmp(EPdataset.dataset(i).cellTypes,'SGL')) && any(strcmp(EPdataset.dataset(i).subTypes,'AVG')) && (any(strcmp(EPdataset.dataset(i).facTypes,'SGL')) || isempty(EPdataset.dataset(i).facNames))
                    isAve=true;
                end;
            end;
            if ~isAve
                set(EPmain.handles.main.window,'enable','off');
            end;
        else
            set(EPmain.handles.main.window,'enable','off');
        end;
        
        EPmain.handles.main.ANOVA = uicontrol('Style', 'pushbutton', 'String', 'ANOVA',...
            'Position', [20 50 100 40], 'Callback', ['global EPmain;','EPmain.mode=''ANOVA'';','ep(''start'');']);
        
        EPmain.handles.main.save = uicontrol('Style', 'pushbutton', 'String', 'Save',...
            'Position', [20 00 100 40], 'Callback', ['global EPmain;','EPmain.mode=''save'';','ep(''start'');']);
        
        if isempty(EPdataset.dataset)
            set(EPmain.handles.main.save,'enable','off');
        end;
        
        
    case 'startPreprocess'
        
        set(EPmain.handles.hMainWindow,'Name', 'Preprocess Data');
        
        uicontrol('Style','text',...
            'String','Import File Format','HorizontalAlignment','left',...
            'Position',[25 470 150 20]);
        
        EPmain.handles.preprocess.importFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatReadList,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.importFormat,''Value'');','if temp ~=0,EPmain.preprocess.importFormat=temp;end;','if isempty(temp),EPmain.preprocess.importFormat=temp;end;','ep(''start'');'],...
            'Value',EPmain.preprocess.importFormat,'Position',[20 450 160 20]);
        
        uicontrol('Style','text',...
            'String','File Type','HorizontalAlignment','left',...
            'Position',[25 430 150 20]);
        
        EPmain.handles.preprocess.type= uicontrol('Style','popupmenu',...
            'String',{'continuous','single_trial','average','grand_average','factors'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.type,''Value'');','if temp ~=0,EPmain.preprocess.type=temp;end;','if isempty(temp),EPmain.preprocess.type=temp;end;','ep(''start'');'],...
            'Value',EPmain.preprocess.type,'Position',[20 410 160 20]);
        
        [importSuffix,importFormatName,importFormat]=ep_fileFormats('average',fileFormatReadList{EPmain.preprocess.importFormat});
        if strcmp(importFormat,'ep_mat')
            set(EPmain.handles.preprocess.type,'enable','off');
        end;
        
        uicontrol('Style','text',...
            'String','Output File Format','HorizontalAlignment','left',...
            'Position',[25 390 150 20]);
        
        EPmain.handles.preprocess.outputFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatSaveList,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.outputFormat,''Value'');','if temp ~=0,EPmain.preprocess.outputFormat=temp;end;','if isempty(temp),EPmain.preprocess.outputFormat=temp;end;','ep(''start'');'],...
            'Value',EPmain.preprocess.outputFormat,'Position',[20 370 160 20]);
        
        EPmain.handles.preprocess.timepointsLabel = uicontrol('Style','text','HorizontalAlignment','left','String', 'Points',...
            'Position',[80 350 50 20]);
        
        if isempty(EPmain.preprocess.timepoints)
            set(EPmain.handles.preprocess.timepointsLabel,'enable','off');
        elseif isempty(str2num(EPmain.preprocess.timepoints))
            set(EPmain.handles.preprocess.timepointsLabel,'enable','off');
        end;
        
        EPmain.handles.preprocess.timepoints = uicontrol('Style','edit','HorizontalAlignment','left','String', EPmain.preprocess.timepoints,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.timepoints,''String''),','if temp ~=0,EPmain.preprocess.timepoints=temp;end;','if isempty(temp),EPmain.preprocess.timepoints=temp;end;','ep(''start'');'],...
            'Position',[80 330 50 20],'TooltipString','For retaining a subset of timepoints - example 1:250');
        
        EPmain.handles.preprocess.baselineLabel=uicontrol('Style','text','HorizontalAlignment','left','String', 'Baseline',...
            'Position',[25 350 50 20]);
        
        if isempty(EPmain.preprocess.baseline)
            set(EPmain.handles.preprocess.baselineLabel,'enable','off');
        elseif isempty(str2num(EPmain.preprocess.baseline))
            set(EPmain.handles.preprocess.baselineLabel,'enable','off');
        end;
        
        EPmain.handles.preprocess.baseline = uicontrol('Style','edit','HorizontalAlignment','left','String', EPmain.preprocess.baseline,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.baseline,''String'');','if temp ~=0,EPmain.preprocess.baseline=temp;end;','if isempty(temp),EPmain.preprocess.baseline=temp;end;','ep(''start'');'],...
            'Position',[25 330 50 20],'TooltipString','Recommended - example 1:50 (if timepoints were dropped, 1st retained sample is sample #1)');
        
        uicontrol('Style','text',...
            'String','Blink Correction','HorizontalAlignment','left',...
            'Position',[25 310 150 20]);
        
        EPmain.handles.preprocess.blinkTemplate = uicontrol('Style','popupmenu',...
            'String',{'fileTemplate','autoTemplate','bothTemplate','none'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.blinkTemplate,''Value'');','if temp ~=0,EPmain.preprocess.blinkTemplate=temp;end;','if isempty(temp),EPmain.preprocess.blinkTemplate=temp;end;','ep(''start'');'],...
            'Value',EPmain.preprocess.blinkTemplate,'Position',[20 290 160 20]);
        
        uicontrol('Style','text',...
            'String','Saccade Correction','HorizontalAlignment','left',...
            'Position',[25 270 150 20]);
        
        EPmain.handles.preprocess.saccadeTemplate = uicontrol('Style','popupmenu',...
            'String',{'fileTemplate','none'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.saccadeTemplate,''Value'');','if temp ~=0,EPmain.preprocess.saccadeTemplate=temp;end;','if isempty(temp),EPmain.preprocess.saccadeTemplate=temp;end;','ep(''start'');'],...
            'Value',EPmain.preprocess.saccadeTemplate,'Position',[20 250 160 20]);
        
        EPmain.handles.preprocess.detrend= uicontrol('Style','checkbox',...
            'String','Detrend (not rec''d)',...
            'CallBack',['global EPmain;','EPmain.preprocess.detrend=get(EPmain.handles.preprocess.detrend,''Value'');','ep(''start'');'],...
            'Value',EPmain.preprocess.detrend,'Position',[20 230 160 20]);
        
        uicontrol('Style','text',...
            'String','Bad Channel Correction','HorizontalAlignment','left',...
            'Position',[25 210 150 20]);
        
        EPmain.handles.preprocess.channelMode = uicontrol('Style','popupmenu',...
            'String',{'replace','mark','none'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.channelMode,''Value'');','if temp ~=0,EPmain.preprocess.channelMode=temp;end;','if isempty(temp),EPmain.preprocess.channelMode=temp;end;','ep(''start'');'],...
            'Value',EPmain.preprocess.channelMode,'Position',[20 190 160 20]);
        
        uicontrol('Style','text',...
            'String','Movement Correction','HorizontalAlignment','left',...
            'Position',[25 170 150 20]);
        
        EPmain.handles.preprocess.trialMode = uicontrol('Style','popupmenu',...
            'String',{'fix','none'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.trialMode,''Value'');','if temp ~=0,EPmain.preprocess.trialMode=temp;end;','if isempty(temp),EPmain.preprocess.trialMode=temp;end;','ep(''start'');'],...
            'Value',EPmain.preprocess.trialMode,'Position',[20 150 160 20]);
        
        EPmain.handles.preprocess.refType = uicontrol('Style','popupmenu',...
            'String',refList,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.refType,''Value'');','if temp ~=0,EPmain.preprocess.refType=temp;end;','if isempty(temp),EPmain.preprocess.refType=temp;end;','ep(''start'');'],...
            'Value',EPmain.preprocess.refType,'Position',[20 130 100 20],'TooltipString','Reference channels explicitly represented as a waveform in the data file.');
        
        EPmain.handles.preprocess.reference = uicontrol('Style','edit','HorizontalAlignment','left','String', EPmain.preprocess.reference,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.preprocess.reference,''String'');','if temp ~=0,EPmain.preprocess.reference=temp;end;','if isempty(temp),EPmain.preprocess.reference=temp;end;','ep(''start'');'],...
            'Position',[120 130 55 20],'TooltipString','example: 20 21');
        
        uicontrol('Style','frame',...
            'Position',[20 35 170 90]);
        
        EPmain.handles.preprocess.check= uicontrol('Style','checkbox',...
            'String','Single Cell Files',...
            'CallBack',['global EPmain;','EPmain.preprocess.check=get(EPmain.handles.preprocess.check,''Value'');','ep(''start'');'],...
            'Value',EPmain.preprocess.check,'Position',[30 100 160 20]);
        
        EPmain.handles.preprocess.subject= uicontrol('Style','edit',...
            'String',EPmain.preprocess.subject,...
            'CallBack',['global EPmain;','EPmain.preprocess.subject=get(EPmain.handles.preprocess.subject,''Value'');','ep(''start'');'],...
            'Position',[30 70 50 20],'TooltipString','example 4:6');
        
        EPmain.handles.preprocess.subjectLabel=uicontrol('Style','text',...
            'String','Subject','HorizontalAlignment','left',...
            'Position',[90 70 50 20]);
        
        if isempty(EPmain.preprocess.subject)
            set(EPmain.handles.preprocess.subjectLabel,'enable','off');
        elseif isempty(str2num(EPmain.preprocess.subject))
            set(EPmain.handles.preprocess.subjectLabel,'enable','off');
        end;
        
        EPmain.handles.preprocess.cell= uicontrol('Style','edit',...
            'String',EPmain.preprocess.cell,...
            'CallBack',['global EPmain;','EPmain.preprocess.cell=get(EPmain.handles.preprocess.cell,''Value'');','ep(''start'');'],...
            'Position',[30 40 50 20],'TooltipString','example 7:9');
        
        EPmain.handles.preprocess.cellLabel=uicontrol('Style','text',...
            'String','Cell','HorizontalAlignment','left',...
            'Position',[90 40 50 20]);
        
        if isempty(EPmain.preprocess.cell)
            set(EPmain.handles.preprocess.cellLabel,'enable','off');
        elseif isempty(str2num(EPmain.preprocess.cell))
            set(EPmain.handles.preprocess.cellLabel,'enable','off');
        end;
        
        EPmain.handles.preprocess.template = uicontrol('Style', 'pushbutton', 'String', 'Template',...
            'Position', [10 0 60 35], 'Callback', ['global EPtemplate;','EPtemplate=[];','ep_template']);
        
        noData=true;
        for theData=1:length(EPdataset.dataset)
            if any(strcmp(EPdataset.dataset(theData).dataType,{'single_trial','continuous'})) && ~isempty(EPdataset.dataset(theData).eloc)
                noData=false;
            end;
        end;
        if noData
            set(EPmain.handles.preprocess.template,'enable','off');
        end;
        
        EPmain.handles.preprocess.preprocess = uicontrol('Style', 'pushbutton', 'String', 'Run',...
            'Position', [70 0 60 35], 'Callback', 'ep(''preprocessData'')');
        
        EPmain.handles.preprocess.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [130 0 60 35], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
    case 'preprocessData' %start preprocessing the data
        
        set(EPmain.handles.preprocess.subject,'ForegroundColor','black');
        set(EPmain.handles.preprocess.cell,'ForegroundColor','black');
        
        textPrefs.firstRow=EPmain.preferences.general.firstRow;
        textPrefs.firstCol=EPmain.preferences.general.firstCol;
        textPrefs.lastCol=EPmain.preferences.general.lastCol;
        textPrefs.orientation=EPmain.preferences.general.orientation;
        
        typeNum = get(EPmain.handles.preprocess.type,'value');
        switch typeNum
            case 1
                dataType='continuous';
            case 2
                dataType='single_trial';
            case 3
                dataType='average';
            case 4
                dataType='grand_average';
            case 5
                dataType='factors';
        end;
        
        importFormatNum = get(EPmain.handles.preprocess.importFormat,'value');        
        [importSuffix,importFormatName,importFormat]=ep_fileFormats(dataType,fileFormatReadList{importFormatNum});

        
        outputFormatNum = get(EPmain.handles.preprocess.outputFormat,'value');
        [outputSuffix,outputFormatName,outputFormat]=ep_fileFormats(dataType,fileFormatSaveList{outputFormatNum});
        
        EPmain.preprocess.format=importFormatNum;
        EPmain.preprocess.type=typeNum;
        
        timePoints=str2num(get(EPmain.handles.preprocess.timepoints,'string'));
        baseline=str2num(get(EPmain.handles.preprocess.baseline,'string'));
        switch get(EPmain.handles.preprocess.blinkTemplate,'value')
            case 1
                blinkTemplate='fileTemplate';
            case 2
                blinkTemplate='autoTemplate';
            case 3
                blinkTemplate='bothTemplate';
            case 4
                blinkTemplate='none';
        end;
        switch get(EPmain.handles.preprocess.saccadeTemplate,'value')
            case 1
                saccadeTemplate='fileTemplate';
            case 2
                saccadeTemplate='none';
        end;
        detrend=get(EPmain.handles.preprocess.detrend,'value');
        switch get(EPmain.handles.preprocess.channelMode,'value')
            case 1
                channelMode='replace';
            case 2
                channelMode='mark';
            case 3
                channelMode='none';
        end;
        switch get(EPmain.handles.preprocess.trialMode,'value')
            case 1
                trialMode='fix';
            case 2
                trialMode='none';
        end;
        
        [sessionFiles, pathname] = uigetfile('*.*','Open:','MultiSelect','on');
        activeDirectory=pathname;
        if ~iscell(sessionFiles)
            temp=sessionFiles;
            sessionFiles=[];
            sessionFiles{1}=temp;
        end;
        if (sessionFiles{1}==0) & ~ischar(sessionFiles{1})
            msg{1}='No filenames selected. You have to click on a name.';
            [msg]=ep_errorMsg(msg);
            return
        end
        
        for theFile=1:size(sessionFiles,2)
            sessionFiles{theFile}=[activeDirectory sessionFiles{theFile}];
        end;
        
        if any(strcmp(blinkTemplate, {'fileTemplate','bothTemplate'}))
            [FileName,PathName,FilterIndex] = uigetfile('','Blink Template','blinks');
            if FileName == 0
                warndlg('No blink template file selected.');
                return;
            end;
            blinkFile=[PathName FileName];
        else
            blinkFile=[];
        end;
        
        if any(strcmp(saccadeTemplate, {'fileTemplate'}))
            [FileName,PathName,FilterIndex] = uigetfile('','Saccade Template','saccades');
            if FileName == 0
                warndlg('No saccade template file selected.');
                return;
            end;
            saccadeFile=[PathName FileName];
        else
            saccadeFile=[];
        end;
        
        inArg=[];
        inArg{1}='files';
        inArg{2}=sessionFiles;
        inArg{3}='format';
        inArg{4}=importFormat;
        inArg{5}='type';
        inArg{6}=dataType;
        inArg{7}='outputFormat';
        inArg{8}=outputFormat;
        inArg{9}='template';
        inArg{10}=blinkTemplate;
        inArg{11}='channelMode';
        inArg{12}=channelMode;
        inArg{13}='saturation';
        inArg{14}=[-EPmain.preferences.preprocess.saturation EPmain.preferences.preprocess.saturation];
        inArg{15}='window';
        inArg{16}=EPmain.preferences.preprocess.window;
        inArg{17}='minmax';
        inArg{18}=EPmain.preferences.preprocess.minmax;
        inArg{19}='badnum';
        inArg{20}=EPmain.preferences.preprocess.badnum;
        inArg{21}='saccademin';
        inArg{22}=EPmain.preferences.preprocess.saccademin;
        inArg{23}='neighbors';
        inArg{24}=EPmain.preferences.preprocess.neighbors;
        inArg{25}='maxneighbor';
        inArg{26}=EPmain.preferences.preprocess.maxneighbor;
        inArg{27}='badchan';
        inArg{28}=EPmain.preferences.preprocess.badchan;
        inArg{29}='blink';
        inArg{30}=EPmain.preferences.preprocess.blink;
        inArg{31}='badtrials';
        inArg{32}=EPmain.preferences.preprocess.badtrials;
        inArg{33}='chunkSize';
        inArg{34}=EPmain.preferences.preprocess.chunkSize;
        inArg{35}='minTrialsPerCell';
        inArg{36}=EPmain.preferences.preprocess.minTrialsPerCell;
        inArg{37}='noadjacent';
        inArg{38}=EPmain.preferences.preprocess.noadjacent;
        inArg{39}='trialMode';
        inArg{40}=trialMode;
        inArg{41}='trialminmax';
        inArg{42}=EPmain.preferences.preprocess.trialminmax;
        inArg{43}='movefacs';
        inArg{44}=EPmain.preferences.preprocess.movefacs;
        inArg{45}='textPrefs';
        inArg{46}=textPrefs;
        inArg{47}='noFigure';
        inArg{48}=EPmain.preferences.preprocess.noFigure;
        inArg{49}='sacctemplate';
        inArg{50}=saccadeTemplate;
        inArg{51}='sacctemplate';
        inArg{52}=saccadeTemplate;
        inArg{53}='saccade';
        inArg{54}=EPmain.preferences.preprocess.saccade;
        if any(strcmp(blinkTemplate,{'bothTemplate','fileTemplate'}))
            inArg{end+1}='blinkFile';
            inArg{end+1}=blinkFile;
        end;
        if any(strcmp(saccadeTemplate,{'fileTemplate'}))
            inArg{end+1}='saccadeFile';
            inArg{end+1}=saccadeFile;
        end;
        if ~isempty(EPmain.preferences.preprocess.EOGchans)
            inArg{end+1}='eog';
            inArg{end+1}=EPmain.preferences.preprocess.EOGchans;
        end;
        
        switch EPmain.preprocess.refType
            case {1 2}
                if ~isempty(EPmain.preprocess.reference)
                    msg{1}='Reference channel field should be empty.';
                    [msg]=ep_errorMsg(msg);
                    return
                else
                    inArg{end+1}='reference';
                    inArg{end+1}=[];
                end;
            case 3
                refChan=str2num(EPmain.preprocess.reference);
                if isempty(EPmain.preprocess.reference)
                    msg{1}='Please specify explicit reference channel(s).';
                    [msg]=ep_errorMsg(msg);
                    return
                elseif isempty(refChan)
                    msg{1}='Reference channel(s) need to be numbers.';
                    [msg]=ep_errorMsg(msg);
                    return
                elseif length(refChan) > 2
                    msg{1}='No more than two reference channels can be indicated.';
                    [msg]=ep_errorMsg(msg);
                    return
                end;
                inArg{end+1}='reference';
                inArg{end+1}=refChan;
            case 4
                if ~isempty(EPmain.preprocess.reference)
                    msg{1}='Reference channel field should be empty.';
                    [msg]=ep_errorMsg(msg);
                    return
                end;
        end;
        
%         if ~isempty(timePoints)
%             if timePoints
%                 inArg{end+1}='samples';
%                 inArg{end+1}=timePoints;
%             end;
%         end;
        if ~isempty(baseline)
            if baseline
                inArg{end+1}='baseline';
                inArg{end+1}=baseline;
            end;
        end;
        
        if get(EPmain.handles.preprocess.check,'value') %single cell mode
            subPos=str2num(get(EPmain.handles.preprocess.subject,'string'));
            cellPos=str2num(get(EPmain.handles.preprocess.cell,'string'));
            
            if min(subPos) < 1
                beep
                set(EPmain.handles.preprocess.subject,'ForegroundColor','red');
                drawnow
                return;
            end;
            if min(cellPos) < 1
                beep
                set(EPmain.handles.preprocess.cell,'ForegroundColor','red');
                drawnow
                return;
            end;
            
            for theFile=1:size(sessionFiles,2)
                if ~isempty(subPos)
                    theSubs{theFile}=sessionFiles{theFile}(subPos);
                else
                    theSubs{theFile}='S01';
                end;
                if ~isempty(cellPos)
                    theCells{theFile}=sessionFiles{theFile}(cellPos);
                else
                    theCells{theFile}='C01';
                end;
            end;
            
            uniqueSubs=unique(theSubs);
            
            mergeName = char(inputdlg('Name of new merged dataset?','Dataset name'));
            pause(1); %pause to avoid crash on Windows due to bug in Matlab 2009
            
            inArg{2}=[];
            for sub=1:length(uniqueSubs)
                mergeArg=[];
                theFiles=strcmp(uniqueSubs(sub),theSubs);
                for file=1:length(theFiles)
                    mergeArg{file,1}=sessionFiles{theFiles(file)};
                    mergeArg{file,2}='inputFormat';
                    mergeArg{file,3}=importFormat;
                    mergeArg{file,4}='type';
                    mergeArg{file,5}=dataType;
                    mergeArg{file,6}='labels';
                    mergeArg{file,7}={theCells(theFiles(file)) theSubs(theFiles(file))};
                    mergeArg{file,8}='textPrefs';
                    mergeArg{file,9}=textPrefs;
                end;
                EPdata=ep_mergeEPfiles(mergeArg,mergeName);
                [err]=ep_writeData(EPdata,['EPtempFile' num2str(sub)],'ep_mat');
                if ~isempty(err)
                    inArg{2}{end+1}=['EPtempFile' num2str(sub) '.mat'];
                end;
            end;
        end;
        
        set(EPmain.handles.preprocess.preprocess,'enable','off');
        set(EPmain.handles.preprocess.done,'enable','off');
        drawnow
        
        ep_artifactCorrection(inArg);
        
        if get(EPmain.handles.preprocess.check,'value') %single cell mode
            for i=1:length(inArg{2})
                delete(inArg{2}{i}); %delete temporary merged files
            end;
        end;
        
        set(EPmain.handles.preprocess.preprocess,'enable','on');
        set(EPmain.handles.preprocess.done,'enable','on');
        drawnow
        
        ep('start');
        
    case 'startAverage'
        
        set(EPmain.handles.hMainWindow,'Name', 'Average Data');
        
        uicontrol('Style','text',...
            'String','Input File Format','HorizontalAlignment','left',...
            'Position',[25 450 150 20]);
        
        EPmain.handles.average.importFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatReadList,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.average.importFormat,''Value'');','if temp ~=0,EPmain.average.importFormat=temp;end;','if isempty(temp),EPmain.average.importFormat=temp;end;','ep(''start'');'],...
            'Value',EPmain.average.importFormat,'Position',[20 420 150 20]);
        
        uicontrol('Style','text',...
            'String','Input File Type','HorizontalAlignment','left',...
            'Position',[25 390 150 20]);
        
        EPmain.handles.average.type= uicontrol('Style','popupmenu',...
            'String',{'continuous','single_trial','average','grand_average','factors'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.average.type,''Value'');','if temp ~=0,EPmain.average.type=temp;end;','if isempty(temp),EPmain.average.type=temp;end;','ep(''start'');'],...
            'Value',EPmain.average.type,'Position',[20 360 160 20]);
        
        [importSuffix,importFormatName,importFormat]=ep_fileFormats('average',fileFormatReadList{EPmain.average.importFormat});
        if strcmp(importFormat,'ep_mat')
            set(EPmain.handles.average.type,'enable','off');
        end;
        
        uicontrol('Style','text',...
            'String','Ouput File Format','HorizontalAlignment','left',...
            'Position',[25 330 150 20]);
        
        EPmain.handles.average.outputFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatSaveList,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.average.outputFormat,''Value'');','if temp ~=0,EPmain.average.outputFormat=temp;end;','if isempty(temp),EPmain.average.outputFormat=temp;end;','ep(''start'');'],...
            'Value',EPmain.average.outputFormat,'Position',[20 300 150 20]);
        
        uicontrol('Style','text',...
            'String','Averaging Method','HorizontalAlignment','left',...
            'Position',[25 270 150 20]);
        
        EPmain.handles.average.method= uicontrol('Style','popupmenu',...
            'String',{'Mean','Median','Trimmed Mean'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.average.method,''Value'');','if temp ~=0,EPmain.average.method=temp;end;','if isempty(temp),EPmain.average.method=temp;end;','ep(''start'');'],...
            'Value',EPmain.average.method,'Position',[20 240 150 20]);
        
        EPmain.handles.average.average = uicontrol('Style', 'pushbutton', 'String', 'Average',...
            'Position', [20 50 100 40], 'Callback', 'ep(''averageData'')');
        
        EPmain.handles.average.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [20 0 100 40], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
    case 'averageData'
        
        typeNum = get(EPmain.handles.average.type,'value');
        switch typeNum
            case 1
                dataType='continuous';
                dataOutType='average';
            case 2
                dataType='single_trial';
                dataOutType='average';
            case 3
                dataType='average';
                dataOutType='grand_average';
            case 4
                dataType='grand_average';
                dataOutType='grand_average';
            case 5
                dataType='factors';
                dataOutType='grand_average';
        end;   
        
        importFormatNum = get(EPmain.handles.average.importFormat,'value');        
        [importSuffix,importFormatName,importFormat]=ep_fileFormats(dataType,fileFormatReadList{importFormatNum});

        outputFormatNum = get(EPmain.handles.average.outputFormat,'value');
        [outputSuffix,outputFormatName,outputFormat]=ep_fileFormats(dataOutType,fileFormatSaveList{outputFormatNum});
        
        methodNum = get(EPmain.handles.average.method,'value');
        switch methodNum
            case 1
                averagingMethod='Mean';
            case 2
                averagingMethod='Median';
            case 3
                averagingMethod='Trimmed_Mean';
        end;
        
        EPmain.average.importFormat=importFormatNum;
        EPmain.average.type=typeNum;
        EPmain.average.outputFormat=outputFormatNum;
        EPmain.average.method=methodNum;
        
        [sessionFiles, pathname] = uigetfile('*.*','Open:','MultiSelect','on');
        activeDirectory=pathname;
        if ~iscell(sessionFiles)
            temp=sessionFiles;
            sessionFiles=[];
            sessionFiles{1}=temp;
        end;
        if (sessionFiles{1}==0) & ~ischar(sessionFiles{1})
            msg{1}='No filenames selected. You have to click on a name.';
            [msg]=ep_errorMsg(msg);
            return
        end
        if ~iscell(sessionFiles)
            temp=sessionFiles;
            sessionFiles=[];
            sessionFiles{1}=temp;
        end;
        for theFile=1:size(sessionFiles,2)
            sessionFiles{theFile}=[activeDirectory sessionFiles{theFile}];
        end;
        
        [pathstr, name, ext] = fileparts(sessionFiles{1});
        if strcmp(outputFormat,'ep_mat')
            outFileName=[pathstr filesep name '_avg' ext];
        else
            outFileName=[pathstr filesep name '_avg' ext];
        end;
        
        [outFileName, pathname] = uiputfile('*.*','Save:',outFileName);
        if outFileName == 0
            msg{1}='No output name selected.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        outFileName=[pathname outFileName];
        if exist(outFileName,'file')
            delete(outFileName); %user must have clicked "yes" to whether to replace existing file
        end;
        
        set(EPmain.handles.average.average,'enable','off');
        set(EPmain.handles.average.done,'enable','off');
        drawnow
        averagedData=ep_averageData(sessionFiles,importFormat,dataType,averagingMethod,EPmain.preferences.average.trimLevel);
        if isfield(averagedData,'data')
            [err]=ep_writeData(averagedData,outFileName,outputFormat);
        end
        set(EPmain.handles.average.average,'enable','on');
        set(EPmain.handles.average.done,'enable','on');
        drawnow
        
        ep('start');
        
    case 'startPostprocess'
        
        set(EPmain.handles.hMainWindow,'Name', 'Postprocess Data');
        
        uicontrol('Style','text',...
            'String','Input File Format','HorizontalAlignment','left',...
            'Position',[25 450 150 20]);
        
        EPmain.handles.postprocess.importFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatReadList,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.postprocess.importFormat,''Value'');','if temp ~=0,EPmain.postprocess.importFormat=temp;end;','if isempty(temp),EPmain.postprocess.importFormat=temp;end;','ep(''start'');'],...
            'Value',EPmain.postprocess.importFormat,'Position',[20 430 150 20]);
        
        uicontrol('Style','text',...
            'String','Input File Type','HorizontalAlignment','left',...
            'Position',[25 410 150 20]);
        
        EPmain.handles.postprocess.type= uicontrol('Style','popupmenu',...
            'String',{'continuous','single_trial','average','grand_average','factors'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.postprocess.type,''Value'');','if temp ~=0,EPmain.postprocess.type=temp;end;','if isempty(temp),EPmain.postprocess.type=temp;end;','ep(''start'');'],...
            'Value',EPmain.postprocess.type,'Position',[20 390 160 20]);
        
        [importSuffix,importFormatName,importFormat]=ep_fileFormats('average',fileFormatReadList{EPmain.postprocess.importFormat});
        if strcmp(importFormat,'ep_mat')
            set(EPmain.handles.postprocess.type,'enable','off');
        end;
        
        uicontrol('Style','text',...
            'String','Ouput File Format','HorizontalAlignment','left',...
            'Position',[25 370 150 20]);
        
        EPmain.handles.postprocess.outputFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatSaveList,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.postprocess.outputFormat,''Value'');','if temp ~=0,EPmain.postprocess.outputFormat=temp;end;','if isempty(temp),EPmain.postprocess.outputFormat=temp;end;','ep(''start'');'],...
            'Value',EPmain.postprocess.outputFormat,'Position',[20 350 150 20]);
        
        uicontrol('Style','text',...
            'String','Reference','HorizontalAlignment','left',...
            'Position',[25 330 150 20]);
        
        EPmain.handles.postprocess.reference= uicontrol('Style','popupmenu',...
            'String',{'Average','Traditional','none'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.postprocess.reference,''Value'');','if temp ~=0,EPmain.postprocess.reference=temp;end;','if isempty(temp),EPmain.postprocess.reference=temp;end;','ep(''start'');'],...
            'Value',EPmain.postprocess.reference,'Position',[20 310 150 20]);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Reference Channel 1',...
            'Position',[25 290 150 20]);
        
        EPmain.handles.postprocess.refChan1 = uicontrol('Style','edit','HorizontalAlignment','left','String', EPmain.postprocess.refChan1,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.postprocess.refChan1,''String''),','if temp ~=0,EPmain.postprocess.refChan1=temp;end;','if isempty(temp),EPmain.postprocess.refChan1=temp;end;','ep(''start'');'],...
            'Position',[25 270 150 20]);
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Reference Channel 2',...
            'Position',[25 250 150 20]);
        
        EPmain.handles.postprocess.refChan2 = uicontrol('Style','edit','HorizontalAlignment','left','String', EPmain.postprocess.refChan2,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.postprocess.refChan2,''String'');','if temp ~=0,EPmain.postprocess.refChan2=temp;end;','if isempty(temp),EPmain.postprocess.refChan2=temp;end;','ep(''start'');'],...
            'Position',[25 230 150 20]);
        
        if get(EPmain.handles.postprocess.reference,'Value') ~= 2 %traditional reference
            set(EPmain.handles.postprocess.refChan1,'enable','off');
            set(EPmain.handles.postprocess.refChan2,'enable','off');
        end;
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Baseline Samples',...
            'Position',[25 210 150 20]);
        
        EPmain.handles.postprocess.baseline = uicontrol('Style','edit','HorizontalAlignment','left','String', EPmain.postprocess.baseline,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.postprocess.baseline,''String'');','if temp ~=0,EPmain.postprocess.baseline=temp;end;','if isempty(temp),EPmain.postprocess.baseline=temp;end;','ep(''start'');'],...
            'Position',[25 190 150 20]);
        
        EPmain.handles.postprocess.postprocess = uicontrol('Style', 'pushbutton', 'String', 'Postprocess',...
            'Position', [20 50 100 40], 'Callback', 'ep(''postProcessData'')');
        
        EPmain.handles.postprocess.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [20 0 100 40], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
    case 'postProcessData'
        
        typeNum = get(EPmain.handles.postprocess.type,'value');
        switch typeNum
            case 1
                dataType='continuous';
            case 2
                dataType='single_trial';
            case 3
                dataType='average';
            case 4
                dataType='grand_average';
            case 5
                dataType='factors';
        end;
        
        importFormatNum = get(EPmain.handles.postprocess.importFormat,'value');
        [importSuffix,importFormatName,importFormat]=ep_fileFormats(dataType,fileFormatReadList{importFormatNum});

        
        outputFormatNum = get(EPmain.handles.postprocess.outputFormat,'value');
        [outputSuffix,outputFormatName,outputFormat]=ep_fileFormats(dataType,fileFormatSaveList{outputFormatNum});
                
        referenceNum = get(EPmain.handles.postprocess.reference,'value');
        switch referenceNum
            case 1
                referenceMethod='Average';
            case 2
                referenceMethod='Traditional';
            case 3
                referenceMethod='none';
        end;
        
        EPmain.postprocess.importFormat=importFormatNum;
        EPmain.postprocess.type=typeNum;
        EPmain.postprocess.outputFormat=outputFormatNum;
        EPmain.postprocess.reference=referenceNum;
        EPmain.postprocess.refChan1=str2num(get(EPmain.handles.postprocess.refChan1,'String'));
        EPmain.postprocess.refChan2=str2num(get(EPmain.handles.postprocess.refChan2,'String'));
        EPmain.postprocess.baseline=get(EPmain.handles.postprocess.baseline,'String');
        
        [inputFiles, pathname] = uigetfile('*.*','Open:','MultiSelect','on');
        activeDirectory=pathname;
        if ~iscell(inputFiles)
            temp=inputFiles;
            inputFiles=[];
            inputFiles{1}=temp;
        end;
        if inputFiles{1}==0
            return %user hit cancel on file requestor
        end
        if ~iscell(inputFiles)
            temp=inputFiles;
            inputFiles=[];
            inputFiles{1}=temp;
        end;
        for theFile=1:size(inputFiles,2)
            inputFiles{theFile}=[activeDirectory inputFiles{theFile}];
        end;
        
        set(EPmain.handles.postprocess.postprocess,'enable','off');
        set(EPmain.handles.postprocess.done,'enable','off');
        drawnow
        ep_postprocessData(inputFiles,importFormat,dataType,outputFormat,referenceMethod,EPmain.postprocess.refChan1,EPmain.postprocess.refChan2,str2num(EPmain.postprocess.baseline));
        set(EPmain.handles.postprocess.postprocess,'enable','on');
        set(EPmain.handles.postprocess.done,'enable','on');
        drawnow
        
        ep('start');
        
    case 'startRead'
        
        set(EPmain.handles.hMainWindow,'Name', 'Read Data');
        
        uicontrol('Style','text',...
            'String','File Format','HorizontalAlignment','left',...
            'Position',[25 450 100 20]);
        
        EPmain.handles.read.format = uicontrol('Style','popupmenu',...
            'String',fileFormatReadList,...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.read.format,''Value'');','if temp ~=0,EPmain.read.format=temp;end;','if isempty(temp),EPmain.read.format=temp;end;','ep(''start'');'],...
            'Value',EPmain.read.format,'Position',[20 420 150 20]);
        
        uicontrol('Style','text',...
            'String','File Type','HorizontalAlignment','left',...
            'Position',[25 390 100 20]);
        
        EPmain.handles.read.type= uicontrol('Style','popupmenu',...
            'String',{'continuous','single_trial','average','grand_average','factors'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.read.type,''Value'');','if temp ~=0,EPmain.read.type=temp;end;','if isempty(temp),EPmain.read.type=temp;end;','ep(''start'');'],...
            'Value',EPmain.read.type,'Position',[20 360 150 20]);
        
        [importSuffix,importFormatName,importFormat]=ep_fileFormats('average',fileFormatReadList{EPmain.read.format});
        if strcmp(importFormat,'ep_mat')
            set(EPmain.handles.read.type,'enable','off');
        end;
        
        uicontrol('Style','frame',...
            'Position',[20 265 170 90]);
        
        EPmain.handles.read.check= uicontrol('Style','checkbox',...
            'String','Single Cell Files',...
            'CallBack',['global EPmain;','EPmain.read.check=get(EPmain.handles.read.check,''Value'');','ep(''start'');'],...
            'Value',EPmain.read.check,'Position',[30 330 150 20]);
        
        EPmain.handles.read.subject= uicontrol('Style','edit',...
            'String',EPmain.read.subject,...
            'CallBack',['global EPmain;','EPmain.read.subject=get(EPmain.handles.read.subject,''String'');','ep(''start'');'],...
            'Position',[30 300 50 20],'TooltipString','example 4:6');
        
        EPmain.handles.read.subjectLabel=uicontrol('Style','text',...
            'String','Subject','HorizontalAlignment','left',...
            'Position',[90 300 50 20]);
        
        if isempty(EPmain.read.subject)
            set(EPmain.handles.read.subjectLabel,'enable','off');
        elseif isempty(str2num(EPmain.read.subject))
            set(EPmain.handles.read.subjectLabel,'enable','off');
        end;
        
        EPmain.handles.read.cell= uicontrol('Style','edit',...
            'String',EPmain.read.cell,...
            'CallBack',['global EPmain;','EPmain.read.cell=get(EPmain.handles.read.cell,''String'');','ep(''start'');'],...
            'Position',[30 270 50 20],'TooltipString','example 7:9');
        
        EPmain.handles.read.cellLabel=uicontrol('Style','text',...
            'String','Cell','HorizontalAlignment','left',...
            'Position',[90 270 50 20]);
        
        if isempty(EPmain.read.cell)
            set(EPmain.handles.read.cellLabel,'enable','off');
        elseif isempty(str2num(EPmain.read.cell))
            set(EPmain.handles.read.cellLabel,'enable','off');
        end;
        
        if ~isempty(EPdataset.dataset)
            for i=1:length(EPdataset.dataset)
                fileName=EPdataset.dataset(i).dataName;
                if strcmp(EPdataset.dataset(i).saved,'no')
                    fileName=['*' fileName];
                end;
                tableData{i,1}=fileName;
            end;
        else
            tableData=[];
        end;
        
        tableNames{1}='data';
        columnEditable =  false;
        ColumnFormat{1}=[];
        
        if ~EPmain.noTable
            try
                EPmain.handles.read.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'CellSelectionCallback',@deleteReadData,'ForegroundColor','red',...
                    'ColumnWidth',{135},'Position',[20 100 170 150]);
            catch
                EPmain.noTable=1;
            end;
        end;
        if EPmain.noTable
            if ~isempty(tableData)
                EPmain.handles.read.menu= uicontrol('Style','popupmenu',...
                    'String',tableData,...
                    'Position',[20 160 160 20]);
                
                EPmain.handles.read.chooseMenu = uicontrol('Style', 'pushbutton', 'String', 'Purge',...
                    'Position', [20 130 80 35], 'Callback', @deleteReadData);
                drawnow
            else
                EPmain.handles.read.menu= uicontrol('Style','popupmenu',...
                    'String','none',...
                    'Position',[20 160 160 20]);
                
                EPmain.handles.read.chooseMenu = uicontrol('Style', 'pushbutton', 'String', 'Purge',...
                    'Position', [20 130 80 35], 'Callback', @deleteReadData);
                set(EPmain.handles.read.chooseMenu,'enable','off');
                drawnow
            end;
        end;
        
        EPmain.handles.read.hRead = uicontrol('Style', 'pushbutton', 'String', 'Read',...
            'Position', [20 50 100 40], 'Callback', 'ep(''readData'')');
        
        EPmain.handles.read.hQuitRead = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [20 0 100 40], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
    case 'readData'
        
        set(EPmain.handles.read.subject,'ForegroundColor','black');
        set(EPmain.handles.read.cell,'ForegroundColor','black');
        
        textPrefs.firstRow=EPmain.preferences.general.firstRow;
        textPrefs.firstCol=EPmain.preferences.general.firstCol;
        textPrefs.lastCol=EPmain.preferences.general.lastCol;
        textPrefs.orientation=EPmain.preferences.general.orientation;
        
        typeNum = get(EPmain.handles.read.type,'value');
        switch typeNum
            case 1
                dataType='continuous';
            case 2
                dataType='single_trial';
            case 3
                dataType='average';
            case 4
                dataType='grand_average';
            case 5
                dataType='factors';
        end;
        
        formatNum = get(EPmain.handles.read.format,'value');
        [importSuffix,importFormatName,importFormat]=ep_fileFormats(dataType,fileFormatReadList{formatNum});
        
        EPmain.read.format=formatNum;
        EPmain.read.type=typeNum;
        if strcmp(importFormat,'ep_mat')
            dataType=''; %data type set by data file itself
        end;
        
        if ~get(EPmain.handles.read.check,'value') %not single cell mode
            
            [fileNames, pathname] = uigetfile('*.*','Open:','MultiSelect','on');
            activeDirectory=pathname;
            if ~iscell(fileNames)
                temp=fileNames;
                fileNames=[];
                fileNames{1}=temp;
            end;
            if fileNames{1}==0
                msg{1}='No filenames selected. You have to click on a name.';
                [msg]=ep_errorMsg(msg);
                return
            end
            
            set(EPmain.handles.read.hRead,'enable','off');
            set(EPmain.handles.read.hQuitRead,'enable','off');
            drawnow
            
            ced=[];
            montage=[];
            for theFile=1:size(fileNames,2)
                inArg=[];
                inArg{1}='file';
                inArg{2}=[activeDirectory fileNames{theFile}];
                inArg{3}='format';
                inArg{4}=importFormat;
                if ~strcmp(importFormat,'ep_mat')
                    inArg{5}='type';
                    inArg{6}=dataType;
                end;
                inArg{end+1}='textPrefs';
                inArg{end+1}=textPrefs;
                if ~isempty(ced)
                    inArg{end+1}='ced';
                    inArg{end+1}=ced;
                    disp('Assuming the electrode information is the same as for the last file.');
                end;
                if ~isempty(montage)
                    inArg{end+1}='montage';
                    inArg{end+1}=montage;
                    disp('Assuming the montage information is the same as for the last file.');
                end;
                
                EPdata=ep_readData(inArg);
                if isempty(EPdata)
                    set(EPmain.handles.read.hRead,'enable','on');
                    set(EPmain.handles.read.hQuitRead,'enable','on');
                    beep();
                    return
                end;
                
                if ~isempty(EPdata.ced)
                    ced=EPdata.ced;
                end;
                if ~isempty(EPdata.montage)
                    montage=EPdata.montage;
                end;
                
                EPdataset=ep_saveEPdataset(EPdataset,EPdata);
            end;
            
        else %single cell mode
            
            subPosStr=get(EPmain.handles.read.subject,'string');
            cellPosStr=get(EPmain.handles.read.cell,'string');
            subPos=str2num(subPosStr);
            cellPos=str2num(cellPosStr);
            
            if min(subPos) < 1
                beep
                set(EPmain.handles.read.subjectLabel,'ForegroundColor','red');
                drawnow
                return;
            end;
            if min(cellPos) < 1
                beep
                set(EPmain.handles.read.cellLabel,'ForegroundColor','red');
                drawnow
                return;
            end;
            
            EPmain.read.check=1;
            EPmain.read.subject=subPosStr;
            EPmain.read.cell=cellPosStr;
            
            [sessionFiles, pathname] = uigetfile(importSuffix,'Open:','MultiSelect','on');
            if ~iscell(sessionFiles)
                temp=sessionFiles;
                sessionFiles=[];
                sessionFiles{1}=temp;
            end;
            if (sessionFiles{1}==0) & ~ischar(sessionFiles{1})
                msg{1}='No filenames selected. You have to click on a name.';
                [msg]=ep_errorMsg(msg);
                return
            end
            if ~iscell(sessionFiles)
                temp=sessionFiles;
                sessionFiles=[];
                sessionFiles{1}=temp;
            end;
            for theFile=1:size(sessionFiles,2)
                if ~isempty(subPos)
                    theSubs{theFile}=sessionFiles{theFile}(subPos);
                else
                    if any(strcmp(dataType,{'grand_average','factors'}))
                        theSubs{theFile}='GAV';
                    else
                        theSubs{theFile}='S01';
                    end;
                end;
                if ~isempty(cellPos)
                    theCells{theFile}=sessionFiles{theFile}(cellPos);
                else
                    theCells{theFile}='C01';
                end;
                sessionFiles{theFile}=[pathname sessionFiles{theFile}];
            end;
            
            mergeName = char(inputdlg('Name of new merged dataset?','Dataset name'));
            pause(1); %pause to avoid crash on Windows due to bug in Matlab 2009
            
            if isempty(mergeName)
                msg{1}='No filename given for the new merged file.  Read operation aborted.';
                [msg]=ep_errorMsg(msg);
                return
            end
            
            inArg=[];
            for theFile=1:length(sessionFiles)
                inArg{theFile,1}=sessionFiles{theFile};
                inArg{theFile,2}='format';
                inArg{theFile,3}=importFormat;
                inArg{theFile,4}='type';
                inArg{theFile,5}=dataType;
                inArg{theFile,6}='labels';
                inArg{theFile,7}={theCells(theFile) theSubs(theFile)};
                inArg{theFile,8}='textPrefs';
                inArg{theFile,9}=textPrefs;
            end;
            set(EPmain.handles.read.hRead,'enable','off');
            set(EPmain.handles.read.hQuitRead,'enable','off');
            drawnow
            EPdata=ep_mergeEPfiles(inArg,mergeName);
            if isempty(EPdata)
                set(EPmain.handles.read.hRead,'enable','on');
                set(EPmain.handles.read.hQuitRead,'enable','on');
                beep();
                return
            end;
            EPdataset=ep_saveEPdataset(EPdataset,EPdata);
            
        end;
        set(EPmain.handles.read.hRead,'enable','on');
        set(EPmain.handles.read.hQuitRead,'enable','on');
        drawnow
        
        ep('start');
        
    case 'startEdit'
        
        if strcmp(EPoverview,'done')
            doneFlag=1;
        else
            doneFlag=0;
        end;
        EPoverview=[];
        tableData=[];
        
        if ~isempty(EPdataset.dataset)
            for i=1:length(EPdataset.dataset)
                fileName=EPdataset.dataset(i).dataName;
                if strcmp(EPdataset.dataset(i).saved,'no')
                    fileName=['*' fileName];
                end;
                tableData{i,1}=fileName;
            end;
        else
            tableData=[];
        end;
        
        tableNames{1}='data';
        columnEditable =  false;
        ColumnFormat{1}=[];
        
        if ~EPmain.noTable
            try
                uicontrol('Style','text','HorizontalAlignment','left','String', 'Click on dataset to edit.',...
                    'Position',[20 250 160 20]);
                
                EPmain.handles.edit.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'CellSelectionCallback',@pickEditData,...
                    'ColumnWidth',{135},'Position',[20 100 170 150]);
            catch
                EPmain.noTable=1;
            end;
        end;
        
        if EPmain.noTable
            EPmain.handles.edit.menu= uicontrol('Style','popupmenu',...
                'String',tableData,...
                'Position',[20 160 160 20]);
            
            EPmain.handles.edit.chooseMenu = uicontrol('Style', 'pushbutton', 'String', 'Edit',...
                'Position', [20 130 80 35], 'Callback', @pickEditData);
        end;
        
        EPmain.handles.edit.hQuitRead = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [20 0 100 40], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
        if length(EPdataset.dataset) == 1 && ~doneFlag %if just one dataset, don't need to wait for it to be selected.
            EPoverview.dataset=1;
            EPoverview.mode=[];
            ep_editData
        end;
        
    case 'startView'
        
        set(EPmain.handles.hMainWindow,'Name', 'View EEG');
        refresh
        
        EPwaves=[];
        
        if isempty(EPmain.view)
            
            %no option to switch to this panel unless there are data to view.
            for i=1:length(EPdataset.dataset)
                EPmain.view.theFileNames{i,1}=EPdataset.dataset(i).dataName;
                [uniqueCells waveOrder m]=unique(EPdataset.dataset(i).cellNames,'first'); %waveOrder is the true order of the unique cells (wave numbers of the first occurences)
                [a b cellOrder]=unique(waveOrder,'first'); %cellOrder is the true order of the unique cells (numbered by cells)
                EPmain.view.theCells{i}(cellOrder)=uniqueCells;
            end;
            
            for color=1:4
                if color <= length(EPdataset.dataset(end).cellNames)
                    EPmain.view.dataset(color)=length(EPdataset.dataset);
                    EPmain.view.cell(color)=min(color,length(EPmain.view.theCells{EPmain.view.dataset(color)}));
                    EPmain.view.trial(color)=1;
                    EPmain.view.subject(color)=1;
                    if ~isempty(EPdataset.dataset(end).facNames)
                        EPmain.view.subject(color)=length(EPdataset.dataset(end).subNames);
                    end;
                    EPmain.view.factor(color)=1;
                else
                    EPmain.view.dataset(color)=length(EPdataset.dataset)+1; %all the cells shown so just leave this one blank.
                end;
                EPmain.view.changeDatasetFlag(color)=0;
                EPmain.view.changeFlag(color)=1;
            end;
            EPmain.view.marker1=0;
            EPmain.view.marker2=0;
            EPmain.view.plotMVmin=zeros(4,1);
            EPmain.view.plotMVmax=zeros(4,1);
        end;
        
        if any(EPmain.view.changeDatasetFlag) %if the dataset was just changed
            theColor=find(EPmain.view.changeDatasetFlag);
            if EPmain.view.dataset(theColor) <= length(EPdataset.dataset)
                if strcmp(EPdataset.dataset(EPmain.view.dataset(theColor)).dataType,'single_trial')
                    EPmain.view.cell(theColor)=1;
                else
                    EPmain.view.cell(theColor)=min(theColor,length(EPdataset.dataset(EPmain.view.dataset(theColor)).cellNames));
                end;
                EPmain.view.subject(theColor)=1;
                if ~isempty(EPdataset.dataset(EPmain.view.dataset(theColor)).facNames)
                    EPmain.view.subject(theColor)=length(EPdataset.dataset(EPmain.view.dataset(theColor)).subNames);
                end;
                EPmain.view.factor(theColor)=1;
                EPmain.view.trial(theColor)=1;
            end;
            EPmain.view.plotMVmin(theColor)=0;
            EPmain.view.plotMVmax(theColor)=0;
            EPmain.view.changeDatasetFlag(theColor)=0;
            EPmain.view.changeFlag(theColor)=1;
        end;
        
        marker1='';
        marker2='';
        if EPmain.view.marker1
            marker1=num2str(EPmain.view.marker1);
        end;
        if EPmain.view.marker2
            marker2=num2str(EPmain.view.marker2);
        end;
        
        %compute voltages and epoch for the changed colors
        if any(EPmain.view.changeFlag)
            for color=1:4
                if EPmain.view.changeFlag(color)
                    EPmain.view.changeFlag(color)=0;
                    
                    if EPmain.view.dataset(color) <= length(EPdataset.dataset)
                        
                        if strcmp(EPdataset.dataset(EPmain.view.dataset(color)).dataType,'continuous') %continuous data
                            theCell=EPmain.view.trial(color);
                            EPmain.view.plotMVmin(color)=EPdataset.dataset(EPmain.view.dataset(color)).plotMVmin(theCell,1,EPmain.view.factor(color));
                            EPmain.view.plotMVmax(color)=EPdataset.dataset(EPmain.view.dataset(color)).plotMVmax(theCell,1,EPmain.view.factor(color));
                            
                        elseif strcmp(EPdataset.dataset(EPmain.view.dataset(color)).dataType,'average') %averaged data
                            theCell=EPmain.view.cell(color);
                            EPmain.view.plotMVmin(color)=EPdataset.dataset(EPmain.view.dataset(color)).plotMVmin(theCell,EPmain.view.subject(color),EPmain.view.factor(color));
                            EPmain.view.plotMVmax(color)=EPdataset.dataset(EPmain.view.dataset(color)).plotMVmax(theCell,EPmain.view.subject(color),EPmain.view.factor(color));
                            
                        else  %if single_trial data
                            theCell=intersect(find(strcmp(EPmain.view.theCells{EPmain.view.dataset(color)}(EPmain.view.cell(color)),EPdataset.dataset(EPmain.view.dataset(color)).cellNames)),...
                                find(EPmain.view.trial(color)==EPdataset.dataset(EPmain.view.dataset(color)).trialNames));
                            EPmain.view.plotMVmin(color)=EPdataset.dataset(EPmain.view.dataset(color)).plotMVmin(theCell,1,EPmain.view.factor(color));
                            EPmain.view.plotMVmax(color)=EPdataset.dataset(EPmain.view.dataset(color)).plotMVmax(theCell,1,EPmain.view.factor(color));
                            
                        end;
                    end;
                end;
            end;
        end;
        plotMVmin=min(EPmain.view.plotMVmin);
        plotMVmax=max(EPmain.view.plotMVmax);
        
        startSamp=0;
        endSamp=0;
        % Time is counted for an epoch as, say, -200 to 800 ms.  In this case the samples have some length to them depending on
        % the digitization rate, such as 4ms for a 250Hz sampling rate.  The points prior to the baseline are counted from the
        % left side of the baseline sample and the points following the baseline are counted from the right side of the baseline
        % sample.  Thus, when the actual samples are calculated, one first adds the baseline to the numbers, yielding 0 to
        % 1000ms.  One then must add the length of the baseline to the first number to obtain 4 to 1000ms.  Then one would
        % divide the numbers by the sample size, resulting in 1 to 250 samples, which are the correct samples to use.
        % For continuous data or other such data where there is no baseline, one must by this convention start with 0 ms,
        % as in 0-1000ms and 1000-2000ms.
        for color=1:4
            if EPmain.view.dataset(color) <= length(EPdataset.dataset)
                if ~isnan(startSamp) && ~isnan(endSamp)
                    if ~startSamp && ~endSamp
                        if strcmp(EPdataset.dataset(EPmain.view.dataset(color)).dataType,'continuous') %continuous data
                            startSamp=1000*(EPmain.view.trial(EPmain.view.dataset(color))-1);
                            endSamp=1000*(EPmain.view.trial(EPmain.view.dataset(color)));
                        else
                            startSamp=min([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]);
                            endSamp=max([EPdataset.dataset(EPmain.view.dataset(color)).timeNames])+(1000/EPdataset.dataset(EPmain.view.dataset(color)).Fs);
                        end;
                    else
                        if strcmp(EPdataset.dataset(EPmain.view.dataset(color)).dataType,'continuous') %continuous data
                            if startSamp ~= 1000*(EPmain.view.trial(EPmain.view.dataset(color))-1)
                                startSamp=NaN;
                            end;
                            if endSamp ~= 1000*(EPmain.view.trial(EPmain.view.dataset(color)))
                                endSamp=NaN;
                            end;
                        else
                            if startSamp ~= min([EPdataset.dataset(EPmain.view.dataset(color)).timeNames])
                                startSamp=NaN;
                            end;
                            if endSamp ~= max([EPdataset.dataset(EPmain.view.dataset(color)).timeNames]+(1000/EPdataset.dataset(EPmain.view.dataset(color)).Fs))
                                endSamp=NaN;
                            end;
                        end;
                    end;
                end;
            end;
        end;
        
        %blue
        EPmain.handles.view.dataset(1) = uicontrol('Style','popupmenu',...
            'String',[EPmain.view.theFileNames; {'none'}],'ForegroundColor','blue',...
            'Value',EPmain.view.dataset(1),'Position',[20 480 150 20],...
            'Callback', ['pause(.2);','global EPmain;','temp=get(EPmain.handles.view.dataset(1),''value'');','if temp ~=0,EPmain.view.dataset(1)=temp;end;','if isempty(temp),EPmain.view.dataset(1)=temp;end;','EPmain.view.changeDatasetFlag(1)=1;','ep(''start'')']);
        
        if EPmain.view.dataset(1) <= length(EPdataset.dataset)
            EPmain.handles.view.cell(1) = uicontrol('Style','popupmenu',...
                'String',EPmain.view.theCells{EPmain.view.dataset(1)},'ForegroundColor','blue',...
                'Value',EPmain.view.cell(1),'Position',[20 460 150 20],...
                'Callback', ['global EPmain;','temp=get(EPmain.handles.view.cell(1),''value'');','if temp ~=0,EPmain.view.cell(1)=temp;end;','if isempty(temp),EPmain.view.cell(1)=temp;end;','EPmain.view.changeFlag(1)=1;','ep(''start'')']);
            
            EPmain.handles.view.subject(1) = uicontrol('Style','popupmenu',...
                'String',EPdataset.dataset(EPmain.view.dataset(1)).subNames,'ForegroundColor','blue',...
                'Value',EPmain.view.subject(1),'Position',[20 440 150 20],...
                'Callback', ['global EPmain;','temp=get(EPmain.handles.view.subject(1),''value'');','if temp ~=0,EPmain.view.subject(1)=temp;end;','if isempty(temp),EPmain.view.subject(1)=temp;end;','EPmain.view.changeFlag(1)=1;','ep(''start'')']);
            
            if ~isempty(EPdataset.dataset(EPmain.view.dataset(1)).trialNames)
                EPmain.handles.view.trial(1) = uicontrol('Style','popupmenu',...
                    'String',EPdataset.dataset(EPmain.view.dataset(1)).trialNames(strcmp(EPmain.view.theCells{EPmain.view.dataset(1)}(EPmain.view.cell(1)),EPdataset.dataset(EPmain.view.dataset(1)).cellNames)),'ForegroundColor','blue',...
                    'Value',EPmain.view.trial(1),'Position',[20 420 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.trial(1),''value'');','if temp ~=0,EPmain.view.trial(1)=temp;end;','if isempty(temp),EPmain.view.trial(1)=temp;end;','EPmain.view.changeFlag(1)=1;','ep(''start'')']);
            elseif strcmp(EPdataset.dataset(EPmain.view.dataset(1)).dataType,'continuous')
                theEpochs=cellstr(num2str([1:ceil(length(EPdataset.dataset(EPmain.view.dataset(1)).timeNames)/EPdataset.dataset(EPmain.view.dataset(1)).Fs)]'));
                for i=1:length(theEpochs)
                    theEpochs{i}=['Sec: ' theEpochs{i}];
                end;
                EPmain.handles.view.trial(1) = uicontrol('Style','popupmenu',...
                    'String',theEpochs,'ForegroundColor','blue',...
                    'Value',EPmain.view.trial(1),'Position',[20 420 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.trial(1),''value'');','if temp ~=0,EPmain.view.trial(1)=temp;end;','if isempty(temp),EPmain.view.trial(1)=temp;end;','EPmain.view.changeFlag(1)=1;','ep(''start'')']);
            else
                EPmain.handles.view.trial(1) = uicontrol('Style','text','HorizontalAlignment','left','String', '     No Trials',...
                    'ForegroundColor','blue','Position',[20 420 150 20]);
            end;
            
            if ~isempty(EPdataset.dataset(EPmain.view.dataset(1)).facNames)
                EPmain.handles.view.factor(1) = uicontrol('Style','popupmenu',...
                    'String',EPdataset.dataset(EPmain.view.dataset(1)).facNames,'ForegroundColor','blue',...
                    'Value',EPmain.view.factor(1),'Position',[20 400 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.factor(1),''value'');','if temp ~=0,EPmain.view.factor(1)=temp;end;','if isempty(temp),EPmain.view.factor(1)=temp;end;','EPmain.view.changeFlag(1)=1;','ep(''start'')']);
            else
                EPmain.handles.view.factor(1) = uicontrol('Style','text','HorizontalAlignment','left','String', '     No Factors',...
                    'ForegroundColor','blue','Position',[20 400 150 20]);
            end;
        end;
        
        %red
        EPmain.handles.view.dataset(2) = uicontrol('Style','popupmenu',...
            'String',[EPmain.view.theFileNames; {'none'}],'ForegroundColor','red',...
            'Value',EPmain.view.dataset(2),'Position',[20 380 150 20],...
            'Callback', ['pause(.2);','global EPmain;','temp=get(EPmain.handles.view.dataset(2),''value'');','if temp ~=0,EPmain.view.dataset(2)=temp;end;','if isempty(temp),EPmain.view.dataset(2)=temp;end;','EPmain.view.changeDatasetFlag(2)=1;','ep(''start'')']);
        
        if EPmain.view.dataset(2) <= length(EPdataset.dataset)
            EPmain.handles.view.cell(2) = uicontrol('Style','popupmenu',...
                'String',EPmain.view.theCells{EPmain.view.dataset(2)},'ForegroundColor','red',...
                'Value',EPmain.view.cell(2),'Position',[20 360 150 20],...
                'Callback', ['global EPmain;','temp=get(EPmain.handles.view.cell(2),''value'');','if temp ~=0,EPmain.view.cell(2)=temp;end;','if isempty(temp),EPmain.view.cell(2)=temp;end;','EPmain.view.changeFlag(2)=1;','ep(''start'')']);
            
            EPmain.handles.view.subject(2) = uicontrol('Style','popupmenu',...
                'String',EPdataset.dataset(EPmain.view.dataset(2)).subNames,'ForegroundColor','red',...
                'Value',EPmain.view.subject(2),'Position',[20 340 150 20],...
                'Callback', ['global EPmain;','temp=get(EPmain.handles.view.subject(2),''value'');','if temp ~=0,EPmain.view.subject(2)=temp;end;','if isempty(temp),EPmain.view.subject(2)=temp;end;','EPmain.view.changeFlag(2)=1;','ep(''start'')']);
            
            
            if ~isempty(EPdataset.dataset(EPmain.view.dataset(2)).trialNames)
                EPmain.handles.view.trial(2) = uicontrol('Style','popupmenu',...
                    'String',EPdataset.dataset(EPmain.view.dataset(2)).trialNames(strcmp(EPmain.view.theCells{EPmain.view.dataset(2)}(EPmain.view.cell(2)),EPdataset.dataset(EPmain.view.dataset(2)).cellNames)),'ForegroundColor','red',...
                    'Value',EPmain.view.trial(2),'Position',[20 320 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.trial(2),''value'');','if temp ~=0,EPmain.view.trial(2)=temp;end;','if isempty(temp),EPmain.view.trial(2)=temp;end;','EPmain.view.changeFlag(2)=1;','ep(''start'')']);
            elseif strcmp(EPdataset.dataset(EPmain.view.dataset(2)).dataType,'continuous')
                theEpochs=cellstr(num2str([1:ceil(length(EPdataset.dataset(EPmain.view.dataset(2)).timeNames)/EPdataset.dataset(EPmain.view.dataset(2)).Fs)]'));
                for i=1:length(theEpochs)
                    theEpochs{i}=['Sec: ' theEpochs{i}];
                end;
                EPmain.handles.view.trial(2) = uicontrol('Style','popupmenu',...
                    'String',theEpochs,'ForegroundColor','red',...
                    'Value',EPmain.view.trial(2),'Position',[20 320 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.trial(2),''value'');','if temp ~=0,EPmain.view.trial(2)=temp;end;','if isempty(temp),EPmain.view.trial(2)=temp;end;','EPmain.view.changeFlag(2)=1;','ep(''start'')']);
            else
                EPmain.handles.view.trial(2) = uicontrol('Style','text','HorizontalAlignment','left','String', '     No Trials',...
                    'ForegroundColor','red','Position',[20 320 150 20]);
            end;
            
            if ~isempty(EPdataset.dataset(EPmain.view.dataset(2)).facNames)
                EPmain.handles.view.factor(2) = uicontrol('Style','popupmenu',...
                    'String',EPdataset.dataset(EPmain.view.dataset(2)).facNames,'ForegroundColor','red',...
                    'Value',EPmain.view.factor(2),'Position',[20 300 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.factor(2),''value'');','if temp ~=0,EPmain.view.factor(2)=temp;end;','if isempty(temp),EPmain.view.factor(2)=temp;end;','EPmain.view.changeFlag(2)=1;','ep(''start'')']);
            else
                EPmain.handles.view.factor(2) = uicontrol('Style','text','HorizontalAlignment','left','String', '     No Factors',...
                    'ForegroundColor','red','Position',[20 300 150 20]);
            end;
        end;
        
        %green
        EPmain.handles.view.dataset(3) = uicontrol('Style','popupmenu',...
            'String',[EPmain.view.theFileNames; {'none'}],'ForegroundColor',[0 .5 0],...
            'Value',EPmain.view.dataset(3),'Position',[20 280 150 20],...
            'Callback', ['pause(.2);','global EPmain;','temp=get(EPmain.handles.view.dataset(3),''value'');','if temp ~=0,EPmain.view.dataset(3)=temp;end;','if isempty(temp),EPmain.view.dataset(3)=temp;end;','EPmain.view.changeDatasetFlag(3)=1;','ep(''start'')']);
        
        if EPmain.view.dataset(3) <= length(EPdataset.dataset)
            EPmain.handles.view.cell(3) = uicontrol('Style','popupmenu',...
                'String',EPmain.view.theCells{EPmain.view.dataset(3)},'ForegroundColor',[0 .5 0],...
                'Value',EPmain.view.cell(3),'Position',[20 260 150 20],...
                'Callback', ['global EPmain;','temp=get(EPmain.handles.view.cell(3),''value'');','if temp ~=0,EPmain.view.cell(3)=temp;end;','if isempty(temp),EPmain.view.cell(3)=temp;end;','EPmain.view.changeFlag(3)=1;','ep(''start'')']);
            
            EPmain.handles.view.subject(3) = uicontrol('Style','popupmenu',...
                'String',EPdataset.dataset(EPmain.view.dataset(3)).subNames,'ForegroundColor',[0 .5 0],...
                'Value',EPmain.view.subject(3),'Position',[20 240 150 20],...
                'Callback', ['global EPmain;','temp=get(EPmain.handles.view.subject(3),''value'');','if temp ~=0,EPmain.view.subject(3)=temp;end;','if isempty(temp),EPmain.view.subject(3)=temp;end;','EPmain.view.changeFlag(3)=1;','ep(''start'')']);
            if ~isempty(EPdataset.dataset(EPmain.view.dataset(3)).trialNames)
                EPmain.handles.view.trial(3) = uicontrol('Style','popupmenu',...
                    'String',EPdataset.dataset(EPmain.view.dataset(3)).trialNames(strcmp(EPmain.view.theCells{EPmain.view.dataset(3)}(EPmain.view.cell(3)),EPdataset.dataset(EPmain.view.dataset(3)).cellNames)),'ForegroundColor',[0 .5 0],...
                    'Value',EPmain.view.trial(3),'Position',[20 220 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.trial(3),''value'');','if temp ~=0,EPmain.view.trial(3)=temp;end;','if isempty(temp),EPmain.view.trial(3)=temp;end;','EPmain.view.changeFlag(3)=1;','ep(''start'')']);
            elseif strcmp(EPdataset.dataset(EPmain.view.dataset(3)).dataType,'continuous')
                theEpochs=cellstr(num2str([1:ceil(length(EPdataset.dataset(EPmain.view.dataset(3)).timeNames)/EPdataset.dataset(EPmain.view.dataset(3)).Fs)]'));
                for i=1:length(theEpochs)
                    theEpochs{i}=['Sec: ' theEpochs{i}];
                end;
                EPmain.handles.view.trial(3) = uicontrol('Style','popupmenu',...
                    'String',theEpochs,'ForegroundColor',[0 .5 0],...
                    'Value',EPmain.view.trial(3),'Position',[20 220 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.trial(3),''value'');','if temp ~=0,EPmain.view.trial(3)=temp;end;','if isempty(temp),EPmain.view.trial(3)=temp;end;','EPmain.view.changeFlag(3)=1;','ep(''start'')']);
            else
                EPmain.handles.view.trial(3) = uicontrol('Style','text','HorizontalAlignment','left','String', '     No Trials',...
                    'ForegroundColor',[0 .5 0],'Position',[20 220 150 20]);
            end;
            
            if ~isempty(EPdataset.dataset(EPmain.view.dataset(3)).facNames)
                EPmain.handles.view.factor(3) = uicontrol('Style','popupmenu',...
                    'String',EPdataset.dataset(EPmain.view.dataset(3)).facNames,'ForegroundColor',[0 .5 0],...
                    'Value',EPmain.view.factor(3),'Position',[20 200 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.factor(3),''value'');','if temp ~=0,EPmain.view.factor(3)=temp;end;','if isempty(temp),EPmain.view.factor(3)=temp;end;','EPmain.view.changeFlag(3)=1;','ep(''start'')']);
            else
                EPmain.handles.view.factor(3) = uicontrol('Style','text','HorizontalAlignment','left','String', '     No Factors',...
                    'ForegroundColor',[0 .5 0],'Position',[20 200 150 20]);
            end;
        end;
        
        %black
        EPmain.handles.view.dataset(4) = uicontrol('Style','popupmenu',...
            'String',[EPmain.view.theFileNames; {'none'}],'ForegroundColor','black',...
            'Value',EPmain.view.dataset(4),'Position',[20 180 150 20],...
            'Callback', ['pause(.2);','global EPmain;','temp=get(EPmain.handles.view.dataset(4),''value'');','if temp ~=0,EPmain.view.dataset(4)=temp;end;','if isempty(temp),EPmain.view.dataset(4)=temp;end;','EPmain.view.changeDatasetFlag(4)=1;','ep(''start'')']);
        
        if EPmain.view.dataset(4) <= length(EPdataset.dataset)
            EPmain.handles.view.cell(4) = uicontrol('Style','popupmenu',...
                'String',EPmain.view.theCells{EPmain.view.dataset(4)},'ForegroundColor','black',...
                'Value',EPmain.view.cell(4),'Position',[20 160 150 20],...
                'Callback', ['global EPmain;','temp=get(EPmain.handles.view.cell(4),''value'');','if temp ~=0,EPmain.view.cell(4)=temp;end;','if isempty(temp),EPmain.view.cell(4)=temp;end;','EPmain.view.changeFlag(4)=1;','ep(''start'')']);
            
            EPmain.handles.view.subject(4) = uicontrol('Style','popupmenu',...
                'String',EPdataset.dataset(EPmain.view.dataset(4)).subNames,'ForegroundColor','black',...
                'Value',EPmain.view.subject(4),'Position',[20 140 150 20],...
                'Callback', ['global EPmain;','temp=get(EPmain.handles.view.subject(4),''value'');','if temp ~=0,EPmain.view.subject(4)=temp;end;','if isempty(temp),EPmain.view.subject(4)=temp;end;','EPmain.view.changeFlag(4)=1;','ep(''start'')']);
            
            if ~isempty(EPdataset.dataset(EPmain.view.dataset(4)).trialNames)
                EPmain.handles.view.trial(4) = uicontrol('Style','popupmenu',...
                    'String',EPdataset.dataset(EPmain.view.dataset(4)).trialNames(strcmp(EPmain.view.theCells{EPmain.view.dataset(4)}(EPmain.view.cell(4)),EPdataset.dataset(EPmain.view.dataset(4)).cellNames)),'ForegroundColor','black',...
                    'Value',EPmain.view.trial(4),'Position',[20 120 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.trial(4),''value'');','if temp ~=0,EPmain.view.trial(4)=temp;end;','if isempty(temp),EPmain.view.trial(4)=temp;end;','EPmain.view.changeFlag(4)=1;','ep(''start'')']);
            elseif strcmp(EPdataset.dataset(EPmain.view.dataset(4)).dataType,'continuous')
                theEpochs=cellstr(num2str([1:ceil(length(EPdataset.dataset(EPmain.view.dataset(4)).timeNames)/EPdataset.dataset(EPmain.view.dataset(4)).Fs)]'));
                for i=1:length(theEpochs)
                    theEpochs{i}=['Sec: ' theEpochs{i}];
                end;
                EPmain.handles.view.trial(4) = uicontrol('Style','popupmenu',...
                    'String',theEpochs,'ForegroundColor','black',...
                    'Value',EPmain.view.trial(1),'Position',[20 120 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.trial(4),''value'');','if temp ~=0,EPmain.view.trial(4)=temp;end;','if isempty(temp),EPmain.view.trial(4)=temp;end;','EPmain.view.changeFlag(4)=1;','ep(''start'')']);
            else
                EPmain.handles.view.trial(4) = uicontrol('Style','text','HorizontalAlignment','left','String', '     No Trials',...
                    'ForegroundColor','black','Position',[20 120 150 20]);
            end;
            
            if ~isempty(EPdataset.dataset(EPmain.view.dataset(4)).facNames)
                EPmain.handles.view.factor(4) = uicontrol('Style','popupmenu',...
                    'String',EPdataset.dataset(EPmain.view.dataset(4)).facNames,'ForegroundColor','black',...
                    'Value',EPmain.view.factor(4),'Position',[20 100 150 20],...
                    'Callback', ['global EPmain;','temp=get(EPmain.handles.view.factor(4),''value'');','if temp ~=0,EPmain.view.factor(4)=temp;end;','if isempty(temp),EPmain.view.factor(4)=temp;end;','EPmain.view.changeFlag(4)=1;','ep(''start'')']);
            else
                EPmain.handles.view.factor(4) = uicontrol('Style','text','HorizontalAlignment','left','String', '     No Factors',...
                    'ForegroundColor','black','Position',[20 100 150 20]);
            end;
        end;
        
        uicontrol('Style','frame',...
            'Position',[15 36 180 62]);
        
        h = uicontrol('Style','text','HorizontalAlignment','left','String', 'Voltage',...
            'ForegroundColor','black','Position',[20 77 50 20]);
        EPmain.handles.view.topVolt = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%4.2f', plotMVmax),...
            'Position',[20 58 50 20],'Callback',@checkViewSettings);
        EPmain.handles.view.bottomVolt = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%4.2f', plotMVmin),...
            'Position',[20 38 50 20],'Callback',@checkViewSettings);
        
        h = uicontrol('Style','text','HorizontalAlignment','left','String', 'Epoch',...
            'ForegroundColor','black','Position',[80 77 50 20]);
        EPmain.handles.view.leftEpoch = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', startSamp),...
            'Position',[80 58 50 20],'Callback',@checkViewSettings);
        EPmain.handles.view.rightEpoch = uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', endSamp),...
            'Position',[80 38 50 20],'Callback',@checkViewSettings);
        
        h = uicontrol('Style','text','HorizontalAlignment','left','String', 'Markers',...
            'ForegroundColor','black','Position',[140 77 50 20]);
        
        EPmain.handles.view.marker1 = uicontrol('Style','edit','HorizontalAlignment','left','String', marker1,...
            'Position',[140 58 50 20],...
            'Callback',@checkViewSettings);
        
        EPmain.handles.view.marker2 = uicontrol('Style','edit','HorizontalAlignment','left','String', marker2,...
            'Position',[140 38 50 20],...
            'Callback',@checkViewSettings);
        
        EPmain.handles.view.waves= uicontrol('Style', 'pushbutton', 'String', 'Waves',...
            'Position', [20 0 50 35], 'Callback', 'ep(''viewWaves'')');
        
        EPmain.handles.view.topos= uicontrol('Style', 'pushbutton', 'String', 'Topos',...
            'Position', [70 0 50 35], 'Callback', 'ep(''viewTopos'')');
        
        if all(EPmain.view.dataset > length(EPdataset.dataset)) || ((plotMVmin == 0) && (plotMVmax == 0))
            set(EPmain.handles.view.waves ,'enable','off');
            set(EPmain.handles.view.topos ,'enable','off');
        end;
        
        EPmain.handles.view.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [120 0 50 35], 'Callback', ['global EPmain;','EPmain.mode=''main'';','EPmain.view=[];','ep(''start'');']);
        
    case 'viewWaves'
        set(EPmain.handles.view.waves,'enable','off');
        set(EPmain.handles.view.topos,'enable','off');        
        set(EPmain.handles.view.done,'enable','off');
        
        for color=1:4
            EPmain.view.dataset(color)=get(EPmain.handles.view.dataset(color),'value');
            if EPmain.view.dataset(color) <= length(EPdataset.dataset)
                EPmain.view.cell(color)=get(EPmain.handles.view.cell(color),'value');
                EPmain.view.subject(color)=get(EPmain.handles.view.subject(color),'value');
                if ~isempty(EPdataset.dataset(EPmain.view.dataset(color)).trialNames)
                    EPmain.view.trial(color)=get(EPmain.handles.view.trial(color),'value');
                end;
                if ~isempty(EPdataset.dataset(EPmain.view.dataset(color)).facNames)
                    EPmain.view.factor(color)=get(EPmain.handles.view.factor(color),'value');
                end;
            end;
        end;
        
        EPwaves.mode=[];
        EPwaves.direction=EPmain.preferences.view.positive;
        ep_showWaves(str2num(get(EPmain.handles.view.bottomVolt,'string')),str2num(get(EPmain.handles.view.topVolt,'string')),...
            str2num(get(EPmain.handles.view.leftEpoch,'string')),str2num(get(EPmain.handles.view.rightEpoch,'string')),...
            str2num(get(EPmain.handles.view.marker1,'string')),str2num(get(EPmain.handles.view.marker2,'string')));
        
        set(EPmain.handles.view.waves,'enable','on');
        set(EPmain.handles.view.topos,'enable','on');        
        set(EPmain.handles.view.done,'enable','on');
        
    case 'viewTopos'
        set(EPmain.handles.view.waves,'enable','off');
        set(EPmain.handles.view.topos,'enable','off');        
        set(EPmain.handles.view.done,'enable','off');
        set(EPmain.handles.view.marker1,'enable','off');
        set(EPmain.handles.view.marker2,'enable','off');
        set(EPmain.handles.view.leftEpoch,'enable','off');
        set(EPmain.handles.view.rightEpoch,'enable','off');
        set(EPmain.handles.view.topVolt,'enable','off');
        set(EPmain.handles.view.bottomVolt,'enable','off');
        for color=1:4
            set(EPmain.handles.view.dataset(color),'enable','off');
            if EPmain.view.dataset(color) <= length(EPdataset.dataset)
                set(EPmain.handles.view.cell(color),'enable','off');
                set(EPmain.handles.view.subject(color),'enable','off');
                set(EPmain.handles.view.trial(color),'enable','off');
                set(EPmain.handles.view.factor(color),'enable','off');
            end;
        end;
        
        for color=1:4
            EPmain.view.dataset(color)=get(EPmain.handles.view.dataset(color),'value');
            if EPmain.view.dataset(color) <= length(EPdataset.dataset)
                EPmain.view.cell(color)=get(EPmain.handles.view.cell(color),'value');
                EPmain.view.subject(color)=get(EPmain.handles.view.subject(color),'value');
                if ~isempty(EPdataset.dataset(EPmain.view.dataset(color)).trialNames)
                    EPmain.view.trial(color)=get(EPmain.handles.view.trial(color),'value');
                end;
                if ~isempty(EPdataset.dataset(EPmain.view.dataset(color)).facNames)
                    EPmain.view.factor(color)=get(EPmain.handles.view.factor(color),'value');
                end;
            end;
        end;
        
        EPtopos=[];
        EPtopos.page=0;
        EPtopos.direction=EPmain.preferences.view.positive;
        ep_showTopos(str2num(get(EPmain.handles.view.leftEpoch,'string')),str2num(get(EPmain.handles.view.rightEpoch,'string')));

    case 'startPCA'

        set(EPmain.handles.hMainWindow,'Name', 'PCA');

        h = uicontrol('Style','text',...
            'String','Mode','HorizontalAlignment','left',...
            'Position',[25 470 150 20]);

        EPmain.handles.pca.mode = uicontrol('Style','popupmenu',...
            'String',{'spatial','temporal'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.pca.mode,''Value'');','if temp ~=0,EPmain.pca.mode=temp;end;','if isempty(temp),EPmain.pca.mode=temp;end;','ep(''start'');'],...
            'Value',EPmain.pca.mode,'Position',[20 450 160 20]);

        h = uicontrol('Style','text',...
            'String','Rotation','HorizontalAlignment','left',...
            'Position',[25 430 150 20]);
        
        EPmain.handles.pca.rotation= uicontrol('Style','popupmenu',...
            'String',{'Varimax','Promax','Infomax','Quartimax','Quartimin','Oblimin','CRFE','MINE','IPSC','TIIC','Geomin','MMER','VOMN','unrotated'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.pca.rotation,''Value'');','if temp ~=0,EPmain.pca.rotation=temp;end;','if isempty(temp),EPmain.pca.rotation=temp;end;','ep(''start'');'],...
            'Value',EPmain.pca.rotation,'Position',[20 410 160 20]);
        
        h = uicontrol('Style','text',...
            'String','Rotation Parameter','HorizontalAlignment','left',...
            'Position',[25 390 150 20]);
        
        EPmain.handles.pca.rotopt= uicontrol('Style','edit','HorizontalAlignment','left',...
            'String',EPmain.pca.rotopt,...
            'CallBack',['global EPmain;','EPmain.pca.rotopt=get(EPmain.handles.pca.rotopt,''String'');','ep(''start'');'],...
            'Position',[25 370 150 20]);
        
        h = uicontrol('Style','text',...
            'String','Relationship matrix','HorizontalAlignment','left',...
            'Position',[25 350 150 20]);
        
        EPmain.handles.pca.rel = uicontrol('Style','popupmenu',...
            'String',{'correlation','covariance'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.pca.rel,''Value'');','if temp ~=0,EPmain.pca.rel=temp;end;','if isempty(temp),EPmain.pca.rel=temp;end;','ep(''start'');'],...
            'Value',EPmain.pca.rel,'Position',[20 330 160 20]);
        
        h = uicontrol('Style','text',...
            'String','Loading Weighting','HorizontalAlignment','left',...
            'Position',[25 310 150 20]);
        
        EPmain.handles.pca.loadings = uicontrol('Style','popupmenu',...
            'String',{'none','Kaiser','covariance','C-M'},...
            'CallBack',['global EPmain;','temp=get(EPmain.handles.pca.loadings,''Value'');','if temp ~=0,EPmain.pca.loadings=temp;end;','if isempty(temp),EPmain.pca.loadings=temp;end;','ep(''start'');'],...
            'Value',EPmain.pca.loadings,'Position',[20 290 160 20]);
        
        if get(EPmain.handles.pca.rotation,'Value') == 3 %infomax rotation
            set(EPmain.handles.pca.rotopt,'enable','off');
            set(EPmain.handles.pca.rel,'enable','off');
            set(EPmain.handles.pca.loadings,'enable','off');
        end;
        
        EPmain.handles.pca.parametric= uicontrol('Style','checkbox',...
            'String','Parametric Analysis',...
            'Value',EPmain.pca.parametric,'Position',[20 270 150 20]);
        
        h = uicontrol('Style','text',...
            'String','# Factors (0=scree)','HorizontalAlignment','left',...
            'Position',[25 250 150 20]);
        
        EPmain.handles.pca.facNum= uicontrol('Style','edit','HorizontalAlignment','left',...
            'String',EPmain.pca.facNum,...
            'Callback', ['global EPmain;','EPmain.pca.facNum=str2num(get(EPmain.handles.pca.facNum,''String''));'],...
            'Position',[25 230 150 20]);
        
        h = uicontrol('Style','text',...
            'String','Title of PCA','HorizontalAlignment','left',...
            'Position',[25 210 150 20]);
        
        EPmain.handles.pca.name= uicontrol('Style','edit','HorizontalAlignment','left',...
            'String',EPmain.pca.name,...
            'Callback', ['global EPmain;','EPmain.pca.name=get(EPmain.handles.pca.name,''String'');'],...
            'Position',[25 190 150 20]);
        
        if ~isempty(EPdataset.dataset)
            for i=1:length(EPdataset.dataset)
                fileName=EPdataset.dataset(i).dataName;
                if strcmp(EPdataset.dataset(i).saved,'no')
                    fileName=['*' fileName];
                end;
                tableData{i,1}=fileName;
            end;
        else
            tableData=[];
        end;
        
        tableNames{1}='data';
        columnEditable =  false;
        ColumnFormat{1}=[];
        
        if ~EPmain.noTable
            try
                EPmain.handles.pca.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'CellSelectionCallback',@pickPCAdata,...
                    'ColumnWidth',{135},'Position',[20 40 170 150]);
            catch
                EPmain.noTable=1;
            end;
        end;
        
        if EPmain.noTable
            EPmain.handles.pca.menu= uicontrol('Style','popupmenu',...
                'String',tableData,...
                'Position',[20 160 160 20]);
            
            EPmain.handles.pca.chooseMenu = uicontrol('Style', 'pushbutton', 'String', 'PCA',...
                'Position', [20 130 80 35], 'Callback', @pickPCAdata);
        end;
        
        EPmain.handles.pca.hQuitRead = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [20 0 100 40], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
    case 'startWindow'
        
        set(EPmain.handles.hMainWindow,'Name', 'Window Data');
        
        %initialize Window Data parameters if entering in from the Main Menu.
        if ~isfield(EPmain.window,'measure')
            
            EPmain.window.minFacVar=EPmain.preferences.window.minFacVar;
            EPmain.window.measure=1;
            EPmain.window.sampStart=1;
            EPmain.window.sampEnd=1;
            EPmain.window.chanGrp=1;
            EPmain.window.specSelect(1)=false;
            EPmain.window.factor=1;
            if ~isfield(EPmain.window,'dataset')
                EPmain.window.dataset=1;
            end;
            
            EPmain.window.dataNames=[];
            EPmain.window.factorData=[];
            EPmain.window.aveData=[];
            factorDataNum=0;
            if ~isempty(EPdataset.dataset)
                EPdata=ep_loadEPdataset(EPdataset,EPmain.window.dataset);
                numChans=length(EPdata.chanNames);
                
                for i=1:length(EPdataset.dataset)
                    EPdata=ep_loadEPdataset(EPdataset,i);
                    if ~strcmp(EPdata.dataType,'single_trial')
                        EPmain.window.aveData(end+1)=i; %keep track of which datasets are averaged data.
                        EPmain.window.dataNames{i,1}=EPdata.dataName;
                        if isfield(EPdata,'pca')
                            if isfield(EPdata.pca,'PCAmode2')
                                if strcmp(EPdata.pca.PCAmode2,'spat')
                                    if numChans == size(EPdata.pca.FacPatST,1)
                                        factorDataNum=factorDataNum+1;
                                        EPmain.window.factorData(factorDataNum).name=EPdata.dataName;
                                        EPmain.window.factorData(factorDataNum).FacPat=EPdata.pca.FacPatST;
                                        facNames=EPdata.facNames(find(strcmp('SGL',EPdata.facTypes)));
                                        if length(facNames)==size(EPmain.window.factorData(factorDataNum).FacPat,2)
                                            EPmain.window.factorData(factorDataNum).facNames=facNames;
                                        else
                                            for fac =1:size(EPmain.window.factorData(factorDataNum).FacPat,2)
                                                EPmain.window.factorData(factorDataNum).facNames{fac}=['fac' num2str(fac)];
                                            end;
                                        end;
                                    end;
                                end;
                            end;
                            if strcmp(EPdata.pca.PCAmode,'spat')
                                if numChans == size(EPdata.pca.FacPat,1)
                                    factorDataNum=factorDataNum+1;
                                    EPmain.window.factorData(factorDataNum).name=EPdata.dataName;
                                    EPmain.window.factorData(factorDataNum).FacPat=EPdata.pca.FacPat;
                                    facNames=EPdata.facNames(find(strcmp('SGL',EPdata.facTypes)));
                                    if length(facNames)==size(EPmain.window.factorData(factorDataNum).FacPat,2)
                                        EPmain.window.factorData(factorDataNum).facNames=facNames;
                                    else
                                        for fac =1:size(EPmain.window.factorData(factorDataNum).FacPat,2)
                                            EPmain.window.factorData(factorDataNum).facNames{fac}=['fac' num2str(fac)];
                                        end;
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
                EPmain.window.dataset=EPmain.window.aveData(1); %the active dataset
            else
                EPmain.window.dataNames='none';
            end;
            EPdata=ep_stripAdds(ep_loadEPdataset(EPdataset,EPmain.window.dataset));
            for i=1:length(EPdata.cellNames)
                EPmain.window.inCells{i}=EPdata.cellNames{i};
                EPmain.window.outCells{i}=EPdata.cellNames{i};
            end;
            
        else
            EPdata=ep_stripAdds(ep_loadEPdataset(EPdataset,EPmain.window.dataset));
        end;
        
        uicontrol('Style','text',...
            'String','Dataset','HorizontalAlignment','left',...
            'Position',[25 470 140 20]);
        
        EPmain.handles.window.dataset = uicontrol('Style','popupmenu',...
            'String',EPmain.window.dataNames(EPmain.window.aveData),...
            'Value',find(EPmain.window.dataset==EPmain.window.aveData),'Position',[20 450 150 20],...
            'Callback', @changeWindowDataset);
        
        if isempty(EPdataset.dataset)
            set(EPmain.handles.window.dataset,'enable','off');
        end;
        
        uicontrol('Style','text',...
            'String','Measure','HorizontalAlignment','left',...
            'Position',[25 430 140 20]);
        
        EPmain.handles.window.measure= uicontrol('Style','popupmenu',...
            'String',{'mean','minpeak','maxpeak','minlatency','maxlatency','mincentroid','maxcentroid'},...
            'Value',EPmain.window.measure,'Position',[20 410 150 20],...
            'Callback', ['global EPmain;','temp=get(EPmain.handles.window.measure,''Value'');','if temp ~=0,EPmain.window.measure=temp;end;','if isempty(temp),EPmain.window.measure=temp;end;','ep(''start'');']);
        
        uicontrol('Style','text',...
            'String','samples              ms','HorizontalAlignment','left',...
            'Position',[25 390 140 20]);
        
        EPmain.handles.window.sampStart= uicontrol('Style','edit',...
            'String',EPmain.window.sampStart,...
            'Position',[25 370 40 20],'Callback',@windowSampStart);
        
        EPmain.handles.window.sampEnd= uicontrol('Style','edit',...
            'String',EPmain.window.sampEnd,...
            'Position',[60 370 40 20],'Callback',@windowSampEnd);
        
        EPmain.handles.window.msStart= uicontrol('Style','text',...
            'String',(EPmain.window.sampStart-EPdata.baseline-1)*(1000/EPdata.Fs),...
            'Position',[100 370 40 20]);
        
        EPmain.handles.window.msEnd= uicontrol('Style','text',...
            'String',(EPmain.window.sampEnd-EPdata.baseline-1)*(1000/EPdata.Fs),...
            'Position',[140 370 40 20]);
        
        uicontrol('Style','text',...
            'String','Baseline(ms):','HorizontalAlignment','left',...
            'Position',[25 350 100 20]);
        
        EPmain.handles.window.baseline= uicontrol('Style','text',...
            'String',(EPdata.baseline)*(1000/EPdata.Fs),'HorizontalAlignment','left',...
            'Position',[105 350 40 20]);
        
        if isfield(EPchanGrp,'group')
            chanGrpNames={EPchanGrp.group.name};
        else
            chanGrpNames='none';
        end;
        
        uicontrol('Style','text',...
            'String','Channel Groups','HorizontalAlignment','left',...
            'Position',[25 330 140 20]);
        
        EPmain.handles.window.chanGrp = uicontrol('Style','popupmenu',...
            'String',chanGrpNames,...
            'Value',EPmain.window.chanGrp,'Position',[20 310 150 20],...
            'Callback', ['global EPmain;','temp=get(EPmain.handles.window.chanGrp,''Value'');','if temp ~=0,EPmain.window.chanGrp=temp;end;','if isempty(temp),EPmain.window.chanGrp=temp;end;','ep(''start'');']);
        
        EPmain.handles.window.channels = uicontrol('Style', 'pushbutton', 'String', 'Channels',...
            'Position', [100 275 80 35], 'Callback', ['global EPmain EPdataset;,ep_chanGrp(ep_loadEPdataset(EPdataset,EPmain.window.dataset),EPmain.window.factorData);']);
        
        if isempty(EPdataset.dataset)
            set(EPmain.handles.window.chanGrp,'enable','off');
        end;
        
        uicontrol('Style','text',...
            'String','Factor','HorizontalAlignment','left',...
            'Position',[20 255 50 20]);
        
        if ~isempty(EPdata.facNames)
            EPmain.handles.window.factor = uicontrol('Style', 'popupmenu', 'String', EPdata.facNames,...
                'Value',EPmain.window.factor,'Position', [70 240 100 35],...
                'Callback', ['global EPmain;','EPmain.window.factor=get(EPmain.handles.window.factor,''Value'');','ep(''start'');']);
        else
            EPmain.handles.window.factors = uicontrol('Style', 'popupmenu', 'String', 'none',...
                'Position', [70 240 100 35],'enable','off');
        end;
        
        %cell selection
        if ~isempty(EPdata.data)
            cellNames=EPdata.cellNames;
            cellNames{end+1}='none';
            
            for i=1:length(cellNames)-1
                tableData(i,1)=EPmain.window.outCells(i);
                tableData(i,2)=EPmain.window.inCells(i);
            end;
            
            tableNames{1}='outCells';
            tableNames{2}='inCells';
            columnEditable=[true true];
            ColumnFormat{1}='char';
            ColumnFormat{2}='char';
            
            if ~EPmain.noTable
                try
                    EPmain.handles.window.cellTable = uitable('Data',tableData,'ColumnName',tableNames,...
                        'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                        'ColumnWidth',{50 90},...
                        'Position',[10 125 190 130],...
                        'CellEditCallback', @windowCellTable);
                catch
                    EPmain.noTable=1;
                end;
            end
            if EPmain.noTable
                initializeEPtable=0;
                if isfield(EPtable,'window')
                    if ~isfield(EPtable.window,'cell')
                        initializeEPtable=1;
                    else
                        EPmain.window.outCells=EPtable.window.cell.tableData(:,1);
                        EPmain.window.inCells=EPtable.window.cell.tableData(:,2);
                    end
                else
                    initializeEPtable=1;
                end;
                if initializeEPtable
                    EPtable.window.cell.Xloc=10;
                    EPtable.window.cell.Yloc=235;
                    EPtable.window.cell.nRows=min(4,length(cellNames)-1);
                    EPtable.window.cell.nCols=2;
                    EPtable.window.cell.colWidth=60;
                    EPtable.window.cell.rowNames=[1:length(cellNames)-1]; %initialize rowNames field.
                    EPtable.window.cell.rowNames=cellstr(num2str(EPtable.window.cell.rowNames'));
                    EPtable.window.cell.tableNames=tableNames;
                    EPtable.window.cell.tableData=tableData;
                    EPtable.window.cell.columnEditable=columnEditable;
                    EPtable.window.cell.ColumnFormat=ColumnFormat;
                    EPtable.window.cell.selRows=[1:EPtable.window.cell.nRows];
                    EPtable.window.cell.selCols=[1:EPtable.window.cell.nCols];
                    EPtable.window.cell.endHeaderCallback='ep(''start'');';
                    EPtable.window.cell.endTableCallback='ep(''start'');';
                    if length(cellNames)-1 <= EPtable.window.cell.nRows
                        EPtable.window.cell.chgRows=repmat(false,1,EPtable.window.cell.nRows);
                    else
                        EPtable.window.cell.chgRows=repmat(true,1,EPtable.window.cell.nRows);
                    end;
                    EPtable.window.cell.chgCols=repmat(false,1,EPtable.window.cell.nCols);
                    EPtable.window.cell.name='window.cell';
                end
                
                EPtable.window.cell=ep_noTable(EPtable.window.cell);
            end;
        else
            uicontrol('Style','text','HorizontalAlignment','left','String', 'This average file cannot be windowed.',...
                'Position',[10 125 190 130]);
        end;
        
        %subject spec selection
        subjectSpecNames=EPdata.subjectSpecNames;
        if length(EPmain.window.specSelect) ~= length(subjectSpecNames)
            EPmain.window.specSelect=repmat(false,length(subjectSpecNames),1);
        end;
        if ~isempty(subjectSpecNames)
            tableData=[];
            for i=1:length(subjectSpecNames)
                tableData{i,1}=EPmain.window.specSelect(i);
                tableData{i,2}=subjectSpecNames{i};
            end;
        else
            tableData=[];
        end;
        
        tableNames{1}='Select';
        tableNames{2}='Specs';
        columnEditable=[true false];
        ColumnFormat{1}='logical';
        ColumnFormat{2}='char';
        
        if ~EPmain.noTable
            try
                EPmain.handles.window.specsTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'ColumnWidth',{40 100},...
                    'Position',[10 40 190 80],...
                    'CellEditCallback', @windowSpecsTable);
            catch
                EPmain.noTable=1;
            end;
        end
        if EPmain.noTable
            initializeEPtable=0;
            if isfield(EPtable,'window')
                if ~isfield(EPtable.window,'spec')
                    initializeEPtable=1;
                else
                    for i=1:size(EPtable.window.spec.tableData,1)
                        EPmain.window.specSelect(i)=EPtable.window.spec.tableData{i,1};
                    end;
                end
            else
                initializeEPtable=1;
            end;
            if initializeEPtable
                EPtable.window.spec.Xloc=10;
                EPtable.window.spec.Yloc=135;
                EPtable.window.spec.nRows=min(5,length(subjectSpecNames));
                EPtable.window.spec.nCols=2;
                EPtable.window.spec.colWidth=60;
                EPtable.window.spec.rowNames=[1:length(subjectSpecNames)];
                EPtable.window.spec.rowNames=cellstr(num2str(EPtable.window.spec.rowNames'));
                EPtable.window.spec.tableNames=tableNames;
                EPtable.window.spec.tableData=tableData;
                EPtable.window.spec.columnEditable=columnEditable;
                EPtable.window.spec.ColumnFormat=ColumnFormat;
                EPtable.window.spec.selRows=[1:EPtable.window.spec.nRows];
                EPtable.window.spec.selCols=[1:EPtable.window.spec.nCols];
                EPtable.window.spec.chgRows=repmat(true,1,EPtable.window.spec.nRows);
                EPtable.window.spec.chgCols=repmat(false,1,EPtable.window.spec.nCols);
                EPtable.window.spec.name='window.spec';
                EPtable.window.spec.endHeaderCallback='ep(''start'');';
                EPtable.window.spec.endTableCallback='ep(''start'');';
            end
            
            EPtable.window.spec=ep_noTable(EPtable.window.spec);
        end;
        
        EPmain.handles.window.window = uicontrol('Style', 'pushbutton', 'String', 'Window',...
            'Position', [10 0 60 35], 'Callback', 'ep(''WindowData'')');
        
        if isempty(EPdata.data)
            set(EPmain.handles.window.window,'enable','off')
        end;
        
        EPmain.handles.window.automatic = uicontrol('Style', 'pushbutton', 'String', 'AutoPCA',...
            'Position', [70 0 60 35], 'Callback', 'ep(''AutoPCA'')');
        
        if ~isfield(EPdata,'pca')
            set(EPmain.handles.window.automatic,'enable','off');
        end;
        
        EPmain.handles.window.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [130 0 60 35], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
    case 'WindowData'
        %output the windowed measure to a file for later ANOVA
        
        set(EPmain.handles.window.window,'enable','off');
        set(EPmain.handles.window.done,'enable','off');
        set(EPmain.handles.window.channels,'enable','off');
        set(EPmain.handles.window.automatic,'enable','off');
        drawnow
        
        EPdata=ep_stripAdds(ep_loadEPdataset(EPdataset,EPmain.window.dataset));
        sampStart = EPmain.window.sampStart;
        sampEnd = EPmain.window.sampEnd;
        factor = EPmain.window.factor;
        theChanGrp = EPmain.window.chanGrp;
        
        if isempty(EPchanGrp)
            warndlg('No channel groups have been specified yet.');
            set(EPmain.handles.window.window,'enable','on');
            set(EPmain.handles.window.done,'enable','on');
            set(EPmain.handles.window.channels,'enable','on');
            set(EPmain.handles.window.automatic,'enable','on');
            drawnow
            return
        end;
        
        areaList=setdiff(unique(EPchanGrp.group(theChanGrp).channel),length(EPchanGrp.group(theChanGrp).areaName));
        if length(areaList) > 1 && ~isempty(EPdata.facVecS)
            warndlg('Each spatial PCA factor is a virtual electrode covering the entire head.  The electrode regions would be 100% correlated so the ANOVA would fail.');
            set(EPmain.handles.window.window,'enable','on');
            set(EPmain.handles.window.done,'enable','on');
            set(EPmain.handles.window.channels,'enable','on');
            set(EPmain.handles.window.automatic,'enable','on');
            drawnow
            return
        end;
        
        measureNum = EPmain.window.measure;
        switch measureNum
            case 1
                measure='mean';
            case 2
                measure='minpeak';
            case 3
                measure='maxpeak';
            case 4
                measure='minlatency';
            case 5
                measure='maxlatency';
            case 6
                measure='mincentroid';
            case 7
                measure='maxcentroid';
        end;
        
        inputcells=[];
        outCellNames=[];
        for theCell=1:length(EPmain.window.inCells)
            if ~strcmp(EPmain.window.inCells(theCell),'none')
                if ~any(strcmp(EPmain.window.outCells(theCell),outCellNames))
                    outCellNames{end+1}=EPmain.window.outCells{theCell};
                    inputcells{end+1}=find(strcmp(EPmain.window.inCells(theCell),EPdata.cellNames));
                else
                    inputcells{find(strcmp(EPmain.window.outCells(theCell),outCellNames))}(end+1)=find(strcmp(EPmain.window.inCells(theCell),EPdata.cellNames));
                end;
            end;
        end;
        
        subjectSpecs =find(EPmain.window.specSelect);
        
        [FileName,PathName,FilterIndex] = uiputfile('*.txt','Windowed Measures File');
        
        if isnumeric(FileName)
            if FileName == 0
                msg{1}='No file name specified.';
                [msg]=ep_errorMsg(msg);
                set(EPmain.handles.window.window,'enable','on');
                set(EPmain.handles.window.done,'enable','on');
                set(EPmain.handles.window.channels,'enable','on');
                set(EPmain.handles.window.automatic,'enable','on');
                drawnow
                return
            end;
        end;
        
        [pathstr, name, ext] = fileparts(FileName);
        
        if ~strcmp(ext,'.txt')
            ext=[ext '.txt'];
        end;
        
        if exist([PathName FileName ext],'file')
            msg{1}=[PathName FileName ext ' already exists!'];
            [msg]=ep_errorMsg(msg);
        else
            ep_windowData(EPdata, EPchanGrp.group(theChanGrp), inputcells, outCellNames, subjectSpecs, sampStart, sampEnd, measure, [PathName FileName ext], factor);
            
            if EPmain.preferences.window.adds
                windowAdds(inputcells, outCellNames, false);
            end;
            disp('done');
        end;
        
        set(EPmain.handles.window.window,'enable','on');
        set(EPmain.handles.window.done,'enable','on');
        set(EPmain.handles.window.channels,'enable','on');
        set(EPmain.handles.window.automatic,'enable','on');
        drawnow
        
    case 'AutoPCA'
        %output the windowed measure to a file for later ANOVA for an entire PCA dataset, setting channels and windows to
        %the peaks.
        
        set(EPmain.handles.window.window,'enable','off');
        set(EPmain.handles.window.done,'enable','off');
        set(EPmain.handles.window.channels,'enable','off');
        set(EPmain.handles.window.automatic,'enable','off');
        drawnow
        
        EPdata=ep_stripAdds(ep_loadEPdataset(EPdataset,EPmain.window.dataset));
        
        numChans=length(EPdata.chanNames);
        numPoints=length(EPdata.timeNames);
        numFacs=length(EPdata.facNames);
        
        if isfield(EPdata,'facVar')
            factorList=find(EPdata.facVar >= EPmain.window.minFacVar);
            disp(['There are ' num2str(length(factorList)) ' factors that meet the minimum variance criterion of: ' num2str(EPmain.window.minFacVar) '.']);
        else
            factorList=[1:length(EPdata.facNames)];
        end;
        
        if isempty(factorList)
            disp('Procedure aborted as there are no factors to process.');
        else
            [FileName,PathName,FilterIndex] = uiputfile('','PCA ANOVA file root name');
            
            if isnumeric(FileName)
                if FileName == 0
                    msg{1}='No file name specified.';
                    [msg]=ep_errorMsg(msg);
                    set(EPmain.handles.window.window,'enable','on');
                    set(EPmain.handles.window.done,'enable','on');
                    set(EPmain.handles.window.channels,'enable','on');
                    set(EPmain.handles.window.automatic,'enable','on');
                    drawnow
                    return
                end;
            end;
            
            [pathstr, name, ext] = fileparts(FileName);
            
            if ~strcmp(ext,'.txt')
                ext=[ext '.txt'];
            end;
            
            for theFactor=factorList
                gave=ep_expandFacs(EPdata,find(ismember(EPdata.chanTypes,{'EEG','REF'})),[],[],[],theFactor);
                if ~isempty(EPdata.facVecT)
                    [C peakLatency]=max(abs(EPdata.facVecT(:,theFactor)));
                else
                    [C peakLatency]=max(max(reshape(shiftdim(abs(mean(gave,4)),1),numPoints,[])'));
                end;
                
                if ~isempty(EPdata.facVecS)
                    [C peakChan]=max(abs(EPdata.facVecS(:,theFactor)));
                else
                    [C peakChan]=max(max(reshape(abs(mean(gave,4)),numChans,[])'));
                end;
                
                measureNum = get(EPmain.handles.window.measure,'Value');
                switch measureNum
                    case 1
                        measure='mean';
                    case 2
                        measure='minpeak';
                    case 3
                        measure='maxpeak';
                    case 4
                        measure='minlatency';
                    case 5
                        measure='maxlatency';
                    case 6
                        measure='mincentroid';
                    case 7
                        measure='maxcentroid';
                end;
                
                inputcells=[];
                outCellNames=[];
                for theCell=1:length(EPmain.window.inCells)
                    if ~strcmp(EPmain.window.inCells(theCell),'none')
                        if ~any(strcmp(EPmain.window.outCells(theCell),outCellNames))
                            outCellNames{end+1}=EPmain.window.outCells{theCell};
                            inputcells{end+1}=find(strcmp(EPmain.window.inCells(theCell),EPdata.cellNames));
                        else
                            inputcells{strcmp(EPmain.window.outCells(theCell),outCellNames)}(end+1)=find(strcmp(EPmain.window.inCells(theCell),EPdata.cellNames));
                        end;
                    end;
                end;
                
                subjectSpecs =find(EPmain.window.specSelect);
                
                chanGrp=[];
                chanGrp.areaName(1)=EPdata.chanNames(peakChan);
                chanGrp.areaName{2}='none';
                chanGrp.name='autoPCA';
                chanGrp.channel=zeros(length(EPdata.chanNames),1);
                chanGrp.channel(peakChan)=1;
                
                outputdata=ep_windowData(EPdata, chanGrp, inputcells, outCellNames, subjectSpecs, peakLatency, peakLatency, measure, [PathName FileName '-' EPdata.facNames{theFactor} ext], theFactor);
            end;
            
            if EPmain.preferences.window.adds
                windowAdds(inputcells, outCellNames, true);
            end;
            
            disp('done');
        end;
        set(EPmain.handles.window.window,'enable','on');
        set(EPmain.handles.window.done,'enable','on');
        set(EPmain.handles.window.channels,'enable','on');
        set(EPmain.handles.window.automatic,'enable','on');
        drawnow
        
    case 'startANOVA'
        %Set up ANOVA pane of main window.
        
        set(EPmain.handles.hMainWindow,'Name', 'Robust ANOVA');
        
        if isempty(EPmain.anova.data)
            EPmain.anova.data.leftColumn=1;
            EPmain.anova.data.rightColumn=1;
            EPmain.anova.data.columnNames{1}='none';
            EPmain.anova.data.totalLevels=0;
            EPmain.anova.numComps=0;
            
            for i=1:6
                EPmain.anova.data.between(i)=2;
                EPmain.anova.data.factor{i}='';
                EPmain.anova.data.levels{i}='';
                EPmain.anova.data.betweenName{i}='';
            end;
        end;
        
        betweenNames=EPmain.anova.data.columnNames;
        betweenNames{end+1}='none';
        
        uicontrol('Style','text',...
            'String','Data Columns','HorizontalAlignment','left',...
            'Position',[25 470 100 20]);
        
        EPmain.handles.anova.leftColumn = uicontrol('Style','popupmenu',...
            'String',EPmain.anova.data.columnNames,...
            'Value',EPmain.anova.data.leftColumn,'Position',[20 450 150 20],...
            'Callback', ['global EPmain;','temp=get(EPmain.handles.anova.leftColumn,''Value'');','if temp ~=0,EPmain.anova.data.leftColumn=temp;end;','if isempty(temp),EPmain.anova.data.leftColumn=temp;end;','ep(''start'');']);
        
        if ~isfield(EPmain.anova.data,'name')
            set(EPmain.handles.anova.leftColumn,'enable','off');
        end;
        
        EPmain.handles.anova.rightColumn = uicontrol('Style','popupmenu',...
            'String',EPmain.anova.data.columnNames,...
            'Value',EPmain.anova.data.rightColumn,'Position',[20 430 150 20],...
            'Callback', ['global EPmain;','temp=get(EPmain.handles.anova.rightColumn,''Value'');','if temp ~=0,EPmain.anova.data.rightColumn=temp;end;','if isempty(temp),EPmain.anova.data.rightColumn=temp;end;','ep(''start'');']);
        
        if ~isfield(EPmain.anova.data,'name')
            set(EPmain.handles.anova.rightColumn,'enable','off');
        end;
        
        uicontrol('Style','text',...
            'String',sprintf('have: %d',EPmain.anova.data.totalLevels),'HorizontalAlignment','left',...
            'Position',[25 410 50 20]);
        
        if isfield(EPmain.anova.data,'name')
            needLevels=EPmain.anova.data.rightColumn-EPmain.anova.data.leftColumn+1;
            if needLevels == 1
                needLevels=0; %no within factors
            end;
        else
            needLevels=0;
        end;
        
        uicontrol('Style','text',...
            'String',sprintf('need: %d',needLevels),'HorizontalAlignment','left',...
            'Position',[100 410 50 20]);
        
        uicontrol('Style','text',...
            'String','Between Group','HorizontalAlignment','left',...
            'Position',[25 380 100 20]);
        
        for i=1:6
            
            betweenRange=[setdiff([1:length(betweenNames)],[1:EPmain.anova.data.rightColumn,EPmain.anova.data.between(setdiff(1:6,i))]) length(betweenNames)];
            %range of values betweenGroup can take.  Ones already taken are not an option.  "none" is always an option.
            
            if ~ismember(EPmain.anova.data.between(i),betweenRange)
                EPmain.anova.data.between(i)=length(betweenNames);
            end;
            
            EPmain.handles.anova.between(i) = uicontrol('Style','popupmenu',...
                'String',betweenNames(betweenRange),...
                'Value',find(EPmain.anova.data.between(i)==betweenRange),'Position',[10 380-i*20 130 20],...
                'Callback', @betweenGroup);
            
            if ~isfield(EPmain.anova.data,'name')
                set(EPmain.handles.anova.between(i),'enable','off');
            end;
            
            if (EPmain.anova.data.between(i) == length(betweenNames))
                EPmain.anova.data.betweenName{i}='';
            end;
            
            EPmain.handles.anova.betweenName(i)= uicontrol('Style','edit',...
                'String',EPmain.anova.data.betweenName{i},...
                'Position',[130 380-i*20 50 20],...
                'Callback', @betweenName);
            
            if ~isfield(EPmain.anova.data,'name') || (EPmain.anova.data.between(i) == length(betweenNames))
                set(EPmain.handles.anova.betweenName(i),'enable','off');
            end;
        end;
        
        uicontrol('Style','text',...
            'String','Factor','HorizontalAlignment','left',...
            'Position',[20 230 50 20]);
        
        uicontrol('Style','text',...
            'String','Levels','HorizontalAlignment','left',...
            'Position',[80 230 50 20]);
        
        uicontrol('Style','frame',...
            'Position',[15 105 180 130]);
        
        for i=1:6
            
            EPmain.handles.anova.factor(i)= uicontrol('Style','edit',...
                'String',EPmain.anova.data.factor{i},...
                'Position',[20 230-i*20 50 20],...
                'Callback', @ANOVATable);
            
            if ~isfield(EPmain.anova.data,'name')
                set(EPmain.handles.anova.factor(i),'enable','off');
            end;
            
            EPmain.handles.anova.levels(i)= uicontrol('Style','edit',...
                'String',EPmain.anova.data.levels{i},...
                'Position',[80 230-i*20 100 20],...
                'Callback', @ANOVATable);
            
            if ~isfield(EPmain.anova.data,'name')
                set(EPmain.handles.anova.levels(i),'enable','off');
            end;
        end;
        
        uicontrol('Style','text',...
            'String','Bonferroni','HorizontalAlignment','left',...
            'Position',[25 80 70 20]);
        
        EPmain.handles.anova.numComps= uicontrol('Style','edit',...
            'String',EPmain.anova.numComps,...
            'Position',[25 60 50 20],...
            'Callback', ['global EPmain;','EPmain.anova.numComps=str2num(get(EPmain.handles.anova.numComps,''String''));']);
        
        EPmain.handles.anova.contrast = uicontrol('Style', 'pushbutton', 'String', 'Contrast',...
            'Position', [100 60 80 30], 'Callback', 'ep_contrastANOVA');
        
        EPmain.handles.anova.load = uicontrol('Style', 'pushbutton', 'String', 'Load',...
            'Position', [20 30 80 30], 'Callback', 'ep(''loadANOVA'')');
        
        EPmain.handles.anova.view = uicontrol('Style', 'pushbutton', 'String', 'View',...
            'Position', [100 30 80 30], 'Callback', 'ep_viewANOVA');
        
        EPmain.handles.anova.run = uicontrol('Style', 'pushbutton', 'String', 'Run',...
            'Position', [20 0 80 30], 'Callback', 'ep(''runANOVA'')');
        
        if (EPmain.anova.data.totalLevels ~= (EPmain.anova.data.rightColumn-EPmain.anova.data.leftColumn+1)) && ~(EPmain.anova.data.totalLevels ==0 && length(char([EPmain.anova.data.betweenName])) > 0)
            set(EPmain.handles.anova.run,'enable','off');
            set(EPmain.handles.anova.view,'enable','off');
        end;
        
        EPmain.handles.anova.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [100 0 80 30], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
    case 'loadANOVA'
        %Load in an initial ANOVA file in order to set up the structure of the analysis.
        
        [ANOVAfile, pathname] = uigetfile('*.txt','ANOVA File');
        
        if ANOVAfile == 0
            return;
        end;
        
        infid=fopen([pathname ANOVAfile],'r');
        ANOVA = textscan(infid, '%s','delimiter','\n');
        ANOVAdata.name=ANOVA{1}{1};
        ANOVAdata.window=ANOVA{1}{2};
        ANOVAdata.measure=ANOVA{1}{3};
        ANOVAdata.changrp=ANOVA{1}{4};
        ANOVAdata.factor=ANOVA{1}{5};
        
        remain = ANOVA{1}{6};
        numCols=length(strfind(remain,sprintf('\t')))+1;
        if strcmp(remain(end),sprintf('\t'))
            numCols=numCols-1;
        end;
        for i = 1:numCols
            [ANOVAdata.cellNames{i}, remain] = strtok(remain,sprintf('\t'));
        end
        remain = ANOVA{1}{7};
        for i = 1:numCols
            [ANOVAdata.areaNames{i}, remain] = strtok(remain,sprintf('\t'));
        end
        
        numSpecs=length(find(strcmp('spec',ANOVAdata.areaNames)));
        numDataCols=numCols-numSpecs;
        ANOVAdata.betweenLvl=[];
        
        for i=1:size(ANOVA{1},1)-7
            remain = ANOVA{1}{i+7};
            for k = 1:numDataCols
                [theNum, remain] = strtok(remain,sprintf('\t'));
                ANOVAdata.data(i,k)=str2num(theNum);
            end
            for k = 1:numSpecs
                [temp, remain] = strtok(remain,sprintf('\t'));
                ANOVAdata.betweenLvl{i,k}=temp;
            end
        end;
        
        EPmain.anova.data=[];
        
        if ~strcmp(ANOVAdata.name,'behavioral')
            for column=1:numCols
                EPmain.anova.data.columnNames{column}=[ANOVAdata.cellNames{column} '-' ANOVAdata.areaNames{column}];
            end;
        else
            for column=1:numCols
                EPmain.anova.data.columnNames{column}=[ANOVAdata.cellNames{column}];
            end;
        end;
        
        for i=1:6
            EPmain.anova.data.between(i)=numCols+1;
            EPmain.anova.data.factor{i}='';
            EPmain.anova.data.levels{i}='';
            EPmain.anova.data.betweenName{i}='';
        end;
        
        EPmain.anova.data.leftColumn=1;
        
        EPmain.anova.data.rightColumn=numDataCols;
        
        EPmain.anova.data.data=ANOVAdata.data;
        
        EPmain.anova.data.betweenLvl=ANOVAdata.betweenLvl;
        
        EPmain.anova.data.areaNames=ANOVAdata.areaNames;
        
        EPmain.anova.data.name=ANOVAdata.name;
        
        EPmain.anova.data.totalLevels=0;
        
        ep('start');
        
    case 'runANOVA'
        %run the ANOVAs on the data
        
        set(EPmain.handles.anova.contrast,'enable','off');
        set(EPmain.handles.anova.load,'enable','off');
        set(EPmain.handles.anova.view,'enable','off');
        set(EPmain.handles.anova.run,'enable','off');
        set(EPmain.handles.anova.done,'enable','off');
        drawnow
        
        %initial error checking of settings
        for i=1:6
            if xor(isempty(EPmain.anova.data.betweenName{i}),(EPmain.anova.data.between(i)>length(EPmain.anova.data.columnNames)))
                msg{1}='All between-group independent variables need to have a three letter label specified.';
                [msg]=ep_errorMsg(msg);
                set(EPmain.handles.anova.contrast,'enable','on');
                set(EPmain.handles.anova.load,'enable','on');
                set(EPmain.handles.anova.view,'enable','on');
                set(EPmain.handles.anova.run,'enable','on');
                set(EPmain.handles.anova.done,'enable','on');
                return
            end
            
            if ~isempty(EPmain.anova.data.factor{i}) && isempty(EPmain.anova.data.levels{i})
                msg{1}='There is a within-group factor with a name but no levels.';
                [msg]=ep_errorMsg(msg);
                set(EPmain.handles.anova.contrast,'enable','on');
                set(EPmain.handles.anova.load,'enable','on');
                set(EPmain.handles.anova.view,'enable','on');
                set(EPmain.handles.anova.run,'enable','on');
                set(EPmain.handles.anova.done,'enable','on');
                return
            end
        end;
        
        %determine the within group factors and which are electrode factors
        factorNames=cell(0);
        levelNames=cell(0);
        for i=1:6
            if ~isempty(EPmain.anova.data.factor{i})
                factorNames{end+1}=EPmain.anova.data.factor{i};
                levelNames{end+1}=EPmain.anova.data.levels{i};
            end;
        end;
        
        repFactor=1;
        repCount=1;
        lvlCount=1;
        
        if ~strcmp(EPmain.anova.data.name,'behavioral')
            elecFactors=ones(length(factorNames),1);
            for theFactor =length(factorNames):-1:1
                factorArea=cell(length(levelNames{theFactor}),1);
                for theCol=EPmain.anova.data.leftColumn:EPmain.anova.data.rightColumn
                    repCount=repCount+1;
                    if repCount > repFactor
                        repCount=1;
                        lvlCount=lvlCount+1;
                        if lvlCount > length(levelNames{theFactor})
                            lvlCount=1;
                        end;
                    end;
                    factorArea{lvlCount}{end+1}=EPmain.anova.data.areaNames{theCol};
                end;
                repFactor=repFactor*length(levelNames{theFactor});
                for theLevel=1:length(levelNames{theFactor});
                    if length(unique(factorArea{theLevel})) > 1
                        elecFactors(theFactor)=0; %not an electrode factor
                    end;
                end;
            end;
        else
            elecFactors=zeros(length(factorNames),1);
        end;
        
        %Select input and output files
        
        [outFileName, pathname] = uiputfile('*.*','ANOVA output:');
        if outFileName == 0
            ep('start');
            return %user hit cancel on file requestor
        end;
        outFileName=[pathname outFileName];
        if exist(outFileName,'file')
            delete(outFileName); %user must have clicked "yes" to whether to replace existing file
        end;
        [outPathstr, outFileName, outExt] = fileparts(outFileName);
        if ~strcmp(outExt,'.html')
            outExt='.html';
        end;
        
        outfid=fopen([outPathstr filesep outFileName outExt],'w');
        
        [ANOVAfiles, pathname] = uigetfile('*.txt','Open:','MultiSelect','on');
        activeDirectory=pathname;
        if ~iscell(ANOVAfiles)
            temp=ANOVAfiles;
            ANOVAfiles=[];
            ANOVAfiles{1}=temp;
        end;
        if ANOVAfiles{1}==0
            msg{1}='No filenames selected. You have to click on a name';
            [msg]=ep_errorMsg(msg);
            set(EPmain.handles.anova.contrast,'enable','on');
            set(EPmain.handles.anova.load,'enable','on');
            set(EPmain.handles.anova.view,'enable','on');
            set(EPmain.handles.anova.run,'enable','on');
            set(EPmain.handles.anova.done,'enable','on');
            return
        end
        if ~iscell(ANOVAfiles)
            temp=ANOVAfiles;
            ANOVAfiles=[];
            ANOVAfiles{1}=temp;
        end;
        for theFile=1:size(ANOVAfiles,2)
            ANOVAfiles{theFile}=[activeDirectory ANOVAfiles{theFile}];
        end;
        
        %start processing each ANOVA file
        
        fprintf(outfid,'<font face="Courier">');
        fprintf(outfid,'<font size="2">');
        
        for theFile=1:length(ANOVAfiles)
            [pathstr, fileName, ext] = fileparts(ANOVAfiles{theFile});
            infid=fopen([pathname fileName ext],'r');
            if infid == -1
                msg{1}=['There was an error trying to open ' fileName '.'];
                [msg]=ep_errorMsg(msg);
                set(EPmain.handles.anova.contrast,'enable','on');
                set(EPmain.handles.anova.load,'enable','on');
                set(EPmain.handles.anova.view,'enable','on');
                set(EPmain.handles.anova.run,'enable','on');
                set(EPmain.handles.anova.done,'enable','on');
                return
            end
            
            fprintf(outfid,'%s </BR>',fileName);
            
            ANOVA = textscan(infid, '%s','delimiter','\r');
            ANOVAdata.name=deblank(ANOVA{1}{1});
            ANOVAdata.window=deblank(ANOVA{1}{2});
            ANOVAdata.measure=deblank(ANOVA{1}{3});
            ANOVAdata.changrp=deblank(ANOVA{1}{4});
            ANOVAdata.factor=deblank(ANOVA{1}{5});
            ANOVAdata.betweenLvl=cell(0);
            remain = ANOVA{1}{6};
            numCols=length(strfind(remain,sprintf('\t')))+1;
            if strcmp(remain(end),sprintf('\t'))
                numCols=numCols-1;
            end;
            for i = 1:numCols
                [ANOVAdata.cellNames{i}, remain] = strtok(remain,sprintf('\t'));
            end
            remain = ANOVA{1}{7};
            for i = 1:numCols
                [ANOVAdata.areaNames{i}, remain] = strtok(remain,sprintf('\t'));
            end
            
            numSpecs=length(find(strcmp('spec',ANOVAdata.areaNames)));
            numDataCols=numCols-numSpecs;
            
            for i=1:size(ANOVA{1},1)-7
                remain = ANOVA{1}{i+7};
                for k = 1:numDataCols
                    [theNum, remain] = strtok(remain,sprintf('\t'));
                    ANOVAdata.data(i,k)=str2num(theNum);
                end
                for k = 1:numSpecs
                    [ANOVAdata.betweenLvl{i,k}, remain] = strtok(remain,sprintf('\t'));
                end
            end;
            
            if ~strcmp(EPmain.anova.data.name,'behavioral')
                for column=1:numCols
                    ANOVAcellNames{column}=[ANOVAdata.cellNames{column} '-' ANOVAdata.areaNames{column}];
                end;
            else
                for column=1:numCols
                    ANOVAcellNames{column}=[ANOVAdata.cellNames{column}];
                end;
            end;
            
            if (any(~ismember(EPmain.anova.data.columnNames,ANOVAcellNames)) && ~strcmp('autoPCA',ANOVAdata.changrp)) || (length(EPmain.anova.data.columnNames) ~= length(ANOVAcellNames))
                msg{1}=['The column names for ' fileName ' were different from the ANOVA structure that was set up.'];
                [msg]=ep_errorMsg(msg);
                set(EPmain.handles.anova.contrast,'enable','on');
                set(EPmain.handles.anova.load,'enable','on');
                set(EPmain.handles.anova.view,'enable','on');
                set(EPmain.handles.anova.run,'enable','on');
                set(EPmain.handles.anova.done,'enable','on');
                return;
            end;
            
            %check each file to make sure the between group levels are fully crossed
            betweenFactors=[];
            totBetweenLevels=1;
            for theFactor=1:6
                if ~isempty(EPmain.anova.data.betweenName{theFactor})
                    theLevels={ANOVAdata.betweenLvl{:,EPmain.anova.data.between(theFactor)-numDataCols}};
                    for i=1:length(theLevels)
                        theLevelsFirstLetter{i}=theLevels{i}(1);
                    end;
                    
                    if length(unique(theLevels)) ~= length(unique(theLevelsFirstLetter))
                        msg{1}=['The between group variable ' EPmain.anova.data.betweenName{theFactor} ' in ' fileName ' has different level labels with the same first letter.'];
                        [msg]=ep_errorMsg(msg);
                        set(EPmain.handles.anova.contrast,'enable','on');
                        set(EPmain.handles.anova.load,'enable','on');
                        set(EPmain.handles.anova.view,'enable','on');
                        set(EPmain.handles.anova.run,'enable','on');
                        set(EPmain.handles.anova.done,'enable','on');
                        return;
                    end;
                    
                    if length(unique({ANOVAdata.betweenLvl{:,EPmain.anova.data.between(theFactor)-numDataCols}})) ==1
                        msg{1}=['The between group variable ' EPmain.anova.data.betweenName{theFactor} ' in ' fileName ' has only a single level.'];
                        [msg]=ep_errorMsg(msg);
                        set(EPmain.handles.anova.contrast,'enable','on');
                        set(EPmain.handles.anova.load,'enable','on');
                        set(EPmain.handles.anova.view,'enable','on');
                        set(EPmain.handles.anova.run,'enable','on');
                        set(EPmain.handles.anova.done,'enable','on');
                        return;
                    end;
                    
                    betweenFactors(end+1)=EPmain.anova.data.between(theFactor);
                    totBetweenLevels=totBetweenLevels*length(unique(theLevels));
                end;
            end;
            
            ANOVAdata.betweenLvl=ANOVAdata.betweenLvl;
            
            if totBetweenLevels> 1
                %only use first letter of between levels
                for i=1:size(ANOVAdata.betweenLvl,1)
                    for j=1:size(ANOVAdata.betweenLvl,2)
                        ANOVAdata.betweenLvl{i,j}=ANOVAdata.betweenLvl{i,j}(1);
                    end;
                end;
                
                [sortedBetween,index] = sortrows(ANOVAdata.betweenLvl(:,betweenFactors-numDataCols));
                ANOVAdata.data(index,:)=ANOVAdata.data;
                ANOVAdata.index=index;
                betweenCombos=cellstr(cell2mat(sortedBetween));
                if length(unique(betweenCombos)) ~= totBetweenLevels
                    msg{1}=['The between group factors are incompletely crossed.  Nested designs are not currently supported.'];
                    [msg]=ep_errorMsg(msg);
                    set(EPmain.handles.anova.contrast,'enable','on');
                    set(EPmain.handles.anova.load,'enable','on');
                    set(EPmain.handles.anova.view,'enable','on');
                    set(EPmain.handles.anova.run,'enable','on');
                    set(EPmain.handles.anova.done,'enable','on');
                    return;
                end;
                
                uniqueCombos=unique(betweenCombos);
                subjects=zeros(totBetweenLevels,1);
                for i=1:totBetweenLevels
                    subjects(i)=length(find(strcmp(uniqueCombos{i},betweenCombos)));
                end;
            else
                subjects=size(ANOVAdata.data,1);
                ANOVAdata.index=[1:subjects];
                betweenCombos=cellstr(repmat('gave',subjects,1));
            end;
            
            if any(subjects < 2)
                msg{1}=['Each between group cell needs at least two subjects to be able to estimate error variance.'];
                [msg]=ep_errorMsg(msg);
                set(EPmain.handles.anova.contrast,'enable','on');
                set(EPmain.handles.anova.load,'enable','on');
                set(EPmain.handles.anova.view,'enable','on');
                set(EPmain.handles.anova.run,'enable','on');
                set(EPmain.handles.anova.done,'enable','on');
                return;
            end;
            
            factorGroupNames=cell(0);
            levelGroupNames=cell(0);
            for i=1:6
                if ~isempty(EPmain.anova.data.betweenName{i})
                    factorGroupNames{end+1}=EPmain.anova.data.betweenName{i};
                    levelGroupNames{end+1}=cell2mat(unique(sortedBetween(:,i)'));
                end;
            end;
            
            numComps=EPmain.anova.numComps;
            if numComps==0
                numComps=1;
            end;
            alpha.uncorrected=.05; %threshold for declaring statistical significance.
            alpha.corrected=alpha.uncorrected/numComps; %with Bonferroni correction
            
            Y=ANOVAdata.data(:,EPmain.anova.data.leftColumn:EPmain.anova.data.rightColumn);
            ep_ADF(Y, subjects, 1, EPmain.preferences.anova.trimming, 1, EPmain.preferences.anova.bootstrap, EPmain.preferences.anova.seed, EPmain.preferences.anova.missing, factorNames, levelNames, factorGroupNames, levelGroupNames, elecFactors, alpha, 0, outfid);
            
            if EPmain.preferences.anova.adds
                ANOVAAdds(ANOVAdata, Y, subjects, betweenCombos);
            end;
            fclose(infid);
        end;
        fclose(outfid);
        
        ep('start');
        
    case 'startSave'
        %Set up save pane of main window.
        
        set(EPmain.handles.hMainWindow,'Name', 'Save Data');
        
        uicontrol('Style','text',...
            'String','File Format','HorizontalAlignment','left',...
            'Position',[25 450 100 20]);
        
        EPmain.handles.save.hFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatSaveList,...
            'Value',EPmain.save.format,'Position',[20 420 150 20],...
            'Callback', ['global EPmain;','temp=get(EPmain.handles.save.hFormat,''Value'');','if temp ~=0,EPmain.save.format=temp;end;','if isempty(temp),EPmain.save.format=temp;end;','ep(''start'');']);
        
        EPmain.handles.save.SGLchan= uicontrol('Style','checkbox',...
            'String','Single Channels',...
            'Value',EPmain.save.SGLchan,'Position',[20 390 150 20],...
            'Callback', ['global EPmain;','EPmain.save.SGLchan=get(EPmain.handles.save.SGLchan,''Value'');']);
        
        EPmain.handles.save.REGchan= uicontrol('Style','checkbox',...
            'String','Regional Channels',...
            'Value',EPmain.save.REGchan,'Position',[20 370 150 20],...
            'Callback', ['global EPmain;','EPmain.save.REGchan=get(EPmain.handles.save.REGchan,''Value'');']);
        
        EPmain.handles.save.SGLcell= uicontrol('Style','checkbox',...
            'String','Single Cells',...
            'Value',EPmain.save.SGLcell,'Position',[20 350 150 20],...
            'Callback', ['global EPmain;','EPmain.save.SGLcell=get(EPmain.handles.save.SGLcell,''Value'');']);
        
        EPmain.handles.save.CMBcell= uicontrol('Style','checkbox',...
            'String','Combination Cells',...
            'Value',EPmain.save.CMBcell,'Position',[20 330 150 20],...
            'Callback', ['global EPmain;','EPmain.save.CMBcell=get(EPmain.handles.save.CMBcell,''Value'');']);
        
        EPmain.handles.save.RAW= uicontrol('Style','checkbox',...
            'String','Single Trial Data',...
            'Value',EPmain.save.RAW,'Position',[20 310 150 20],...
            'Callback', ['global EPmain;','EPmain.save.RAW=get(EPmain.handles.save.RAW,''Value'');']);
        
        EPmain.handles.save.AVG= uicontrol('Style','checkbox',...
            'String','Averaged Data',...
            'Value',EPmain.save.AVG,'Position',[20 290 150 20],...
            'Callback', ['global EPmain;','EPmain.save.AVG=get(EPmain.handles.save.AVG,''Value'');']);
        
        EPmain.handles.save.GAV= uicontrol('Style','checkbox',...
            'String','Grand Average Data',...
            'Value',EPmain.save.GAV,'Position',[20 270 150 20],...
            'Callback', ['global EPmain;','EPmain.save.GAV=get(EPmain.handles.save.GAV,''Value'');']);
        
        EPmain.handles.save.SGLfac= uicontrol('Style','checkbox',...
            'String','Single Factors',...
            'Value',EPmain.save.SGLfac,'Position',[20 250 150 20],...
            'Callback', ['global EPmain;','EPmain.save.SGLfac=get(EPmain.handles.save.SGLfac,''Value'');']);
        
        EPmain.handles.save.CMBfac= uicontrol('Style','checkbox',...
            'String','Combination Factors',...
            'Value',EPmain.save.CMBfac,'Position',[20 230 150 20],...
            'Callback', ['global EPmain;','EPmain.save.CMBfac=get(EPmain.handles.save.CMBfac,''Value'');']);
        
        if ~isempty(EPdataset.dataset)
            for i=1:length(EPdataset.dataset)
                fileName=EPdataset.dataset(i).dataName;
                if strcmp(EPdataset.dataset(i).saved,'no')
                    fileName=['*' fileName];
                end;
                tableData{i,1}=fileName;
            end;
        else
            tableData=[];
        end;
        
        tableNames{1}='data';
        columnEditable =  false;
        ColumnFormat{1}=[];
        
        if ~EPmain.noTable
            try
                EPmain.handles.save.hTable = uitable('Data',tableData,'ColumnName',tableNames,...
                    'ColumnEditable', columnEditable, 'ColumnFormat', ColumnFormat,...
                    'CellSelectionCallback',@saveData,...
                    'ColumnWidth',{135},'Position',[20 60 170 150]);
            catch
                EPmain.noTable=1;
            end;
        end;
        
        if EPmain.noTable
            EPmain.handles.save.menu= uicontrol('Style','popupmenu',...
                'String',tableData,...
                'Position',[20 160 160 20]);
            
            EPmain.handles.save.chooseMenu = uicontrol('Style', 'pushbutton', 'String', 'Save',...
                'Position', [20 130 80 35], 'Callback', @saveData);
        end;
        
        EPmain.handles.save.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [20 0 100 40], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
    case 'startPreferenceMain'
        
        set(EPmain.handles.hMainWindow,'Name', 'Preferences');
        
        EPmain.handles.preferences.hGeneral = uicontrol('Style', 'pushbutton', 'String', 'Files',...
            'Position', [20 450 100 30], 'Callback', ['global EPmain;','EPmain.mode=''preferenceGeneral'';','ep(''start'');']);
        
        EPmain.handles.preferences.hPreprocess = uicontrol('Style', 'pushbutton', 'String', 'Preprocess',...
            'Position', [20 410 100 30], 'Callback', ['global EPmain;','EPmain.mode=''preferencePreprocess'';','ep(''start'');']);
        
        EPmain.handles.preferences.hAverage = uicontrol('Style', 'pushbutton', 'String', 'Average',...
            'Position', [20 370 100 30], 'Callback', ['global EPmain;','EPmain.mode=''preferenceAverage'';','ep(''start'');']);
        
        EPmain.handles.preferences.hPostprocess = uicontrol('Style', 'pushbutton', 'String', 'Postprocess',...
            'Position', [20 330 100 30], 'Callback', ['global EPmain;','EPmain.mode=''preferencePostprocess'';','ep(''start'');']);
        
        EPmain.handles.preferences.hView = uicontrol('Style', 'pushbutton', 'String', 'View',...
            'Position', [20 290 100 30], 'Callback', ['global EPmain;','EPmain.mode=''preferenceView'';','ep(''start'');']);
        
        EPmain.handles.preferences.hPCA = uicontrol('Style', 'pushbutton', 'String', 'PCA',...
            'Position', [20 250 100 30], 'Callback', ['global EPmain;','EPmain.mode=''preferencePCA'';','ep(''start'');']);
        
        EPmain.handles.preferences.hWindow = uicontrol('Style', 'pushbutton', 'String', 'Window',...
            'Position', [20 210 100 30], 'Callback', ['global EPmain;','EPmain.mode=''preferenceWindow'';','ep(''start'');']);
        
        EPmain.handles.preferences.hANOVA = uicontrol('Style', 'pushbutton', 'String', 'ANOVA',...
            'Position', [20 170 100 30], 'Callback', ['global EPmain;','EPmain.mode=''preferenceANOVA'';','ep(''start'');']);
        
        EPmain.handles.preferences.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [20 100 100 40], 'Callback', ['global EPmain;','EPmain.mode=''main'';','ep(''start'');']);
        
        EPmain.handles.preferences.reset = uicontrol('Style', 'pushbutton', 'String', 'Reset',...
            'Position', [20 50 100 40], 'Callback', @resetPrefs);
        
        EPmain.handles.preferences.save = uicontrol('Style', 'pushbutton', 'String', 'Save',...
            'Position', [20 0 100 40], 'Callback', 'ep(''savePrefs'');');
        
    case 'startPreferenceGeneral'
        
        set(EPmain.handles.hMainWindow,'Name', 'General Pref');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Session Import Format',...
            'Position',[20 470 150 20]);
        
        EPmain.handles.preferences.general.sessionImportFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatReadList,...
            'Value',EPmain.preferences.general.sessionImportFormat,'Position',[20 450 150 20],'TooltipString','Default file format for reading session files.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Session Ouput Format',...
            'Position',[20 430 150 20]);
        
        EPmain.handles.preferences.general.sessionOutputFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatSaveList,...
            'Value',EPmain.preferences.general.sessionOutputFormat,'Position',[20 410 150 20],'TooltipString','Default file format for saving session files.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'File Import Format',...
            'Position',[20 390 150 20]);
        
        EPmain.handles.preferences.general.importFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatReadList,...
            'Value',EPmain.preferences.general.importFormat,'Position',[20 370 150 20],'TooltipString','Default file format for reading average files.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'File Ouput Format',...
            'Position',[20 350 150 20]);
        
        EPmain.handles.preferences.general.outputFormat = uicontrol('Style','popupmenu',...
            'String',fileFormatSaveList,...
            'Value',EPmain.preferences.general.outputFormat,'Position',[20 330 150 20],'TooltipString','Default file format for saving average files.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'First Text Row',...
            'Position',[20 310 150 20]);
        
        EPmain.handles.preferences.general.firstRow= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.general.firstRow),...
            'Position',[20 290 70 20],'TooltipString','First row of data (as opposed to header rows) when reading in text files.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'First Text Column',...
            'Position',[20 270 150 20]);
        
        EPmain.handles.preferences.general.firstCol= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.general.firstCol),...
            'Position',[20 250 70 20],'TooltipString','First column of data when reading in text files.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Last Text Col (0=last)',...
            'Position',[20 230 150 20]);
        
        EPmain.handles.preferences.general.lastCol= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.general.lastCol),...
            'Position',[20 210 70 20],'TooltipString','Last column of data when reading in text files (0=last).');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Text File Orientation',...
            'Position',[20 190 150 20]);
        
        EPmain.handles.preferences.general.orientation = uicontrol('Style','popupmenu',...
            'String',{'Chan Cols','Chan Rows'},...
            'Value',EPmain.preferences.general.orientation,'Position',[20 170 150 20],'TooltipString','When reading in text files, are channels the rows or the columns?');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Montage for EGIS files',...
            'Position',[20 150 150 20]);
        
        EPmain.handles.preferences.general.montage = uicontrol('Style','checkbox',...
            'Value',EPmain.preferences.general.montage,'Position',[20 130 150 20],'TooltipString','When saving files in EGIS file format, add montage information to it?');
        
        EPmain.handles.preferences.general.cancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
            'Position', [20 50 100 40], 'Callback', ['global EPmain;','EPmain.mode=''preferenceMain'';','ep(''start'');']);
        
        EPmain.handles.preferences.general.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [20 0 100 40], 'Callback', 'ep(''getGeneralPrefs'');');
        
    case 'getGeneralPrefs' %retrieve preference settings from the general preference input fields
        
        temp=EPmain.preferences.general;
        
        EPmain.preferences.general.sessionImportFormat=get(EPmain.handles.preferences.general.sessionImportFormat,'Value');
        
        EPmain.preferences.general.sessionOutputFormat=get(EPmain.handles.preferences.general.sessionOutputFormat,'Value');
        
        EPmain.preferences.general.importFormat=get(EPmain.handles.preferences.general.importFormat,'Value');
        
        EPmain.preferences.general.outputFormat=get(EPmain.handles.preferences.general.outputFormat,'Value');
        
        EPmain.preferences.general.firstRow=str2num(get(EPmain.handles.preferences.general.firstRow,'String'));
        
        EPmain.preferences.general.firstCol=str2num(get(EPmain.handles.preferences.general.firstCol,'String'));
        
        EPmain.preferences.general.lastCol=str2num(get(EPmain.handles.preferences.general.lastCol,'String'));
        
        EPmain.preferences.general.orientation=get(EPmain.handles.preferences.general.orientation,'Value');
        
        EPmain.preferences.general.montage=get(EPmain.handles.preferences.general.montage,'Value');
        
        err=checkPrefs;
        
        if err
            msg{1}=['New preferences are incorrect.  Either correct or cancel.'];
            [msg]=ep_errorMsg(msg);
            EPmain.preferences.general=temp;
        else
            EPmain.mode='preferenceMain';
            ep('start');
        end;
        
    case 'startPreferencePreprocess'
        
        set(EPmain.handles.hMainWindow,'Name', 'Preprocess Pref');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Saturation',...
            'Position',[20 470 100 20]);
        
        EPmain.handles.preferences.preprocess.saturation= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.saturation),...
            'Position',[130 470 70 20],'TooltipString','Saturation is the voltage level at which the amplifier reaches the maximum number that it is capable of reporting.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Moving Window',...
            'Position',[20 450 100 20]);
        
        EPmain.handles.preferences.preprocess.window= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.window),...
            'Position',[130 450 70 20],'TooltipString','Moving Window is the number of milliseconds over which the artifact correction routines average the data in a form of low pass filtering.  The larger the number, the less sensitive it is to high frequency spikes.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Chan Min-Max',...
            'Position',[20 430 100 20]);
        
        EPmain.handles.preferences.preprocess.minmax= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.minmax),...
            'Position',[130 430 70 20],'TooltipString','Chan Min-Max v is the maximum allowed change in voltage levels for a channel during a trial before it is deemed to be a bad channel for that trial.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', '% Bad Channel',...
            'Position',[20 410 100 20]);
        
        EPmain.handles.preferences.preprocess.badnum= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.badnum),...
            'Position',[130 410 70 20],'TooltipString','% Bad Channel is the maximum number of channels allowed to be bad in a trial before it is deemed to be a bad trial.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', '# Neigh Chans',...
            'Position',[20 390 100 20]);
        
        EPmain.handles.preferences.preprocess.neighbors= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.neighbors),...
            'Position',[130 390 70 20],'TooltipString','# Neigh Chans is the number of channels considered to be a neighbor for purposes of the artifact correction algorithms.  The electrode coordinates are then used to figure out which channels are to be used.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Neigh Diff v',...
            'Position',[20 370 100 20]);
        
        EPmain.handles.preferences.preprocess.maxneighbor= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.maxneighbor),...
            'Position',[130 370 70 20],'TooltipString','Neigh Diff v is the maximum voltage difference allowed between a channel and its neighbors before it is deemed to be a bad channel.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Bad Chan Corr',...
            'Position',[20 350 100 20]);
        
        EPmain.handles.preferences.preprocess.badchan= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%.3f', EPmain.preferences.preprocess.badchan),...
            'Position',[130 350 70 20],'TooltipString','Bad Chan Corr is the correlation criterion for determining whether a channel is a globally bad channel over the entire session.  If its best correlation with any neighbor is lower than this criterion then it is judged to be globally bad.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Blink Corr',...
            'Position',[20 330 100 20]);
        
        EPmain.handles.preferences.preprocess.blink= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%.3f', EPmain.preferences.preprocess.blink),...
            'Position',[130 330 70 20],'TooltipString','Blink Corr is the correlation criterion for determining whether an ICA factor matches the blink template and should therefore be subtracted from the data.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', '% Bad Trial',...
            'Position',[20 310 100 20]);
        
        EPmain.handles.preferences.preprocess.badtrials= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.badtrials),...
            'Position',[130 310 70 20],'TooltipString','% Bad Trial Chan is the maximum number of trials a channel is allowed to be judged bad before it is deemed to be globally bad.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Size of Chunks',...
            'Position',[20 290 100 20]);
        
        EPmain.handles.preferences.preprocess.chunkSize= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.chunkSize),...
            'Position',[130 290 70 20],'TooltipString','Size of Chunks is the number of time points that are read into each chunk (about 100,000 per GB of available RAM seems to generally work).');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Warn Trials/Cell',...
            'Position',[20 270 100 20]);
        
        EPmain.handles.preferences.preprocess.minTrialsPerCell= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.minTrialsPerCell),...
            'Position',[130 270 70 20],'TooltipString','Warn Trials/Cell is the minimum number of good trials that is considered to be sufficient for a cell.  Any cells dropping below this number will trigger a warning in the artifact correction log.  There is no other effect of this setting.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Bad Neighbors',...
            'Position',[20 250 100 20]);
        
        EPmain.handles.preferences.preprocess.noadjacent= uicontrol('Style','checkbox','HorizontalAlignment','left','value', EPmain.preferences.preprocess.noadjacent,...
            'Position',[130 250 70 20],'TooltipString','Bad Neighbors is an option where if two neighboring channels are marked as being locally bad then the trial is also marked bad (because this typically means that a movement artifact of some sort is present, as opposed to isolated bad channels).');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'v Move Fac',...
            'Position',[20 230 100 20]);
        
        EPmain.handles.preferences.preprocess.trialminmax= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.trialminmax),...
            'Position',[130 230 70 20],'TooltipString','v Move Fac is the maximum voltage difference (maximum-minimum) allowed by a factor by the movement artifact correction step.  Factors exceeding this limit are deeemed to reflect artifacts and are subtracted from the data.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Move Corr Facs',...
            'Position',[20 210 100 20]);
        
        EPmain.handles.preferences.preprocess.movefacs= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%d', EPmain.preferences.preprocess.movefacs),...
            'Position',[130 210 70 20],'TooltipString','Move Corr Facs is the number of factors to be retained by the movement artifact correction routine.  A larger number results in a more accurate but slower process.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'No Figure',...
            'Position',[20 190 100 20]);
        
        EPmain.handles.preferences.preprocess.noFigure= uicontrol('Style','checkbox','HorizontalAlignment','left','value', EPmain.preferences.preprocess.noFigure,...
            'Position',[130 190 70 20],'TooltipString','No Figure is an option to not provide a summary figure for the artifact correction process.  While a very useful figure, it requires substantial memory and so dropping it can be helpful when encountering recalcitrant memory problems. ');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Saccade Corr',...
            'Position',[20 170 100 20]);
        
        EPmain.handles.preferences.preprocess.saccade= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%.3f', EPmain.preferences.preprocess.saccade),...
            'Position',[130 170 70 20],'TooltipString','Saccade Corr is the correlation criterion for determining whether a factor matches the saccade template and should therefore be subtracted from the data.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'v Saccade Fac',...
            'Position',[20 150 100 20]);
        
        EPmain.handles.preferences.preprocess.saccademin= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%.3f', EPmain.preferences.preprocess.saccademin),...
            'Position',[130 150 70 20],'TooltipString','v Saccade Fac is the minimum HEOG voltage difference required to constitute a possible saccade in the saccade correction step.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'EOG channels',...
            'Position',[20 130 100 20]);
        
        EPmain.handles.preferences.preprocess.EOGchans= uicontrol('Style','edit','HorizontalAlignment','left','String', sprintf('%.3f', EPmain.preferences.preprocess.EOGchans),...
            'Position',[130 130 70 20],'TooltipString','EOG chans if automatic identification is not working [LUV RUV LLV RLV LH RH].  Enter in -1 for a missing electrode.');
        
        EPmain.handles.preferences.preprocess.cancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
            'Position', [20 50 100 40], 'Callback', ['global EPmain;','EPmain.mode=''preferenceMain'';','ep(''start'');']);
        
        EPmain.handles.preferences.preprocess.done = uicontrol('Style', 'pushbutton', 'String', 'Main',...
            'Position', [20 0 100 40], 'Callback', 'ep(''getPreprocessPrefs'');');
        
    case 'getPreprocessPrefs' %retrieve preference settings from the preprocessing preference input fields
        
        temp=EPmain.preferences.preprocess;
        
        EPmain.preferences.preprocess.saturation=str2num(get(EPmain.handles.preferences.preprocess.saturation,'String'));
        
        EPmain.preferences.preprocess.window=str2num(get(EPmain.handles.preferences.preprocess.window,'String'));
        
        EPmain.preferences.preprocess.minmax=str2num(get(EPmain.handles.preferences.preprocess.minmax,'String'));
        
        EPmain.preferences.preprocess.badnum=str2num(get(EPmain.handles.preferences.preprocess.badnum,'String'));
        
        EPmain.preferences.preprocess.neighbors=str2num(get(EPmain.handles.preferences.preprocess.neighbors,'String'));
        
        EPmain.preferences.preprocess.maxneighbor=str2num(get(EPmain.handles.preferences.preprocess.maxneighbor,'String'));
        
        EPmain.preferences.preprocess.badchan=str2num(get(EPmain.handles.preferences.preprocess.badchan,'String'));
        
        EPmain.preferences.preprocess.blink=str2num(get(EPmain.handles.preferences.preprocess.blink,'String'));
        
        EPmain.preferences.preprocess.chunkSize=str2num(get(EPmain.handles.preferences.preprocess.chunkSize,'String'));
        
        EPmain.preferences.preprocess.minTrialsPerCell=str2num(get(EPmain.handles.preferences.preprocess.minTrialsPerCell,'String'));
        
        EPmain.preferences.preprocess.noadjacent=get(EPmain.handles.preferences.preprocess.noadjacent,'Value');
        
        EPmain.preferences.preprocess.trialminmax=str2num(get(EPmain.handles.preferences.preprocess.trialminmax,'String'));
        
        EPmain.preferences.preprocess.movefacs=str2num(get(EPmain.handles.preferences.preprocess.movefacs,'String'));
        
        EPmain.preferences.preprocess.noFigure=get(EPmain.handles.preferences.preprocess.noFigure,'Value');
        
        EPmain.preferences.preprocess.saccade=str2num(get(EPmain.handles.preferences.preprocess.saccade,'String'));
        
        EPmain.preferences.preprocess.saccademin=str2num(get(EPmain.handles.preferences.preprocess.saccademin,'String'));
        
        EPmain.preferences.preprocess.EOGchans=str2num(get(EPmain.handles.preferences.preprocess.EOGchans,'String'));
        
        err=checkPrefs;
        
        if err
            msg{1}=['New preferences are incorrect.  Either correct or cancel.'];
            [msg]=ep_errorMsg(msg);
            EPmain.preferences.preprocess=temp;
        else
            EPmain.mode='preferenceMain';
            ep('start');
        end;
        
    case 'startPreferenceAverage'
        
        set(EPmain.handles.hMainWindow,'Name', 'Average Pref');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Averaging Method',...
            'Position',[20 470 150 20]);
        
        EPmain.handles.preferences.average.method = uicontrol('Style','popupmenu',...
            'String',{'Mean','Median','Trimmed Mean'},...
            'Value',EPmain.preferences.average.method,'Position',[20 450 150 20],'TooltipString','Central tendency estimator used for averaging procedure.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Trimming Level',...
            'Position',[20 430 150 20]);
        
        EPmain.handles.preferences.average.trimLevel = uicontrol('Style','edit',...
            'String',EPmain.preferences.average.trimLevel,'Position',[20 410 150 20],'TooltipString','If trimmed mean chosen, proportion of trials trimmed from each tail of the distribution.');
        
        EPmain.handles.preferences.average.cancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
            'Position', [20 50 100 40], 'Callback', ['global EPmain;','EPmain.mode=''preferenceMain'';','ep(''start'');']);
        
        EPmain.handles.preferences.average.done = uicontrol('Style', 'pushbutton', 'String', 'Done',...
            'Position', [20 0 100 40], 'Callback', 'ep(''getAveragePrefs'');');
        
    case 'getAveragePrefs' %retrieve preference settings from the general preference input fields
        
        temp=EPmain.preferences.average;
        
        EPmain.preferences.average.method=get(EPmain.handles.preferences.average.method,'Value');
        
        EPmain.preferences.average.trimLevel=str2num(get(EPmain.handles.preferences.average.trimLevel,'String'));
        
        err=checkPrefs;
        
        if err
            msg{1}=['New preferences are incorrect.  Either correct or cancel.'];
            [msg]=ep_errorMsg(msg);
            EPmain.preferences.average=temp;
        else
            EPmain.mode='preferenceMain';
            ep('start');
        end;
        
    case 'startPreferencePostprocess'
        
        set(EPmain.handles.hMainWindow,'Name', 'Postprocess Pref');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Reference',...
            'Position',[20 470 150 20]);
        
        EPmain.handles.preferences.postprocess.reference = uicontrol('Style','popupmenu',...
            'String',{'Average','Traditional','none'},...
            'Value',EPmain.preferences.postprocess.reference,'Position',[20 450 150 20],'TooltipString','Default type of referencing scheme to apply when postprocessing data.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Reference Channel 1',...
            'Position',[20 430 150 20]);
        
        EPmain.handles.preferences.postprocess.refChan1 = uicontrol('Style','edit',...
            'String',EPmain.preferences.postprocess.refChan1,'Position',[20 410 150 20],'TooltipString','If traditional reference chosen, default channel to use.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Reference Channel 2',...
            'Position',[20 390 150 20]);
        
        EPmain.handles.preferences.postprocess.refChan2 = uicontrol('Style','edit',...
            'String',EPmain.preferences.postprocess.refChan2,'Position',[20 370 150 20],'TooltipString','If traditional reference chosen, default second channel to use, if any.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Baseline',...
            'Position',[20 350 150 20]);
        
        EPmain.handles.preferences.postprocess.baseline = uicontrol('Style','edit',...
            'String',EPmain.preferences.postprocess.baseline,'Position',[20 330 150 20],'TooltipString','Default time points to use for baseline correction when postprocessing data.');
        
        EPmain.handles.preferences.postprocess.cancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
            'Position', [20 50 100 40], 'Callback', ['global EPmain;','EPmain.mode=''preferenceMain'';','ep(''start'');']);
        
        EPmain.handles.preferences.postprocess.done = uicontrol('Style', 'pushbutton', 'String', 'Done',...
            'Position', [20 0 100 40], 'Callback', 'ep(''getPostprocessPrefs'');');
        
    case 'getPostprocessPrefs' %retrieve preference settings from the postprocess preference input fields
        
        temp=EPmain.preferences.postprocess;
        
        EPmain.preferences.postprocess.reference=get(EPmain.handles.preferences.postprocess.reference,'Value');
        
        EPmain.preferences.postprocess.refChan1=str2num(get(EPmain.handles.preferences.postprocess.refChan1,'String'));
        
        EPmain.preferences.postprocess.refChan2=str2num(get(EPmain.handles.preferences.postprocess.refChan2,'String'));
        
        EPmain.preferences.postprocess.baseline=get(EPmain.handles.preferences.postprocess.baseline,'String');
        
        err=checkPrefs;
        
        if err
            msg{1}=['New preferences are incorrect.  Either correct or cancel.'];
            [msg]=ep_errorMsg(msg);
            EPmain.preferences.postprocess=temp;
        else
            EPmain.mode='preferenceMain';
            ep('start');
        end;
        
    case 'startPreferencePCA'
        
        set(EPmain.handles.hMainWindow,'Name', 'PCA Pref');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Mode',...
            'Position',[20 470 150 20]);
        
        EPmain.handles.preferences.pca.mode = uicontrol('Style','popupmenu',...
            'String',{'spatial','temporal'},...
            'Value',EPmain.preferences.pca.mode,'Position',[20 450 150 20],'TooltipString','When conducting PCA, whether default mode is temporal or spatial.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Rotation',...
            'Position',[20 430 150 20]);
        
        EPmain.handles.preferences.pca.rotation = uicontrol('Style','popupmenu',...
            'String',{'Varimax','Promax','Infomax','Quartimax','Quartimin','Oblimin','CRFE','MINE','IPSC','TIIC','Geomin','MMER','VOMN'},...
            'Value',EPmain.preferences.pca.rotation,'Position',[20 410 150 20],'TooltipString','When conducting PCA, default rotation.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Parameter',...
            'Position',[20 390 150 20]);
        
        EPmain.handles.preferences.pca.rotopt = uicontrol('Style','edit','HorizontalAlignment','left',...
            'String',EPmain.preferences.pca.rotopt,'Position',[20 370 150 20],'TooltipString','When conducting PCA, default parameter for rotations that have a parameter.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Rel matrix',...
            'Position',[20 350 150 20]);
        
        EPmain.handles.preferences.pca.rel = uicontrol('Style','popupmenu',...
            'String',{'correlation','covariance'},...
            'Value',EPmain.preferences.pca.rel,'Position',[20 330 150 20],'TooltipString','When conducting PCA, the default relationship matrix.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Loadings',...
            'Position',[20 310 150 20]);
        
        EPmain.handles.preferences.pca.loadings = uicontrol('Style','popupmenu',...
            'String',{'none','Kaiser','covariance','C-M'},...
            'Value',EPmain.preferences.pca.loadings,'Position',[20 290 150 20],'TooltipString','When conducting PCA, the default factor loading weighting scheme.');
        
        EPmain.handles.preferences.pca.cancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
            'Position', [20 50 100 40], 'Callback', ['global EPmain;','EPmain.mode=''preferenceMain'';','ep(''start'');']);
        
        EPmain.handles.preferences.pca.done = uicontrol('Style', 'pushbutton', 'String', 'Done',...
            'Position', [20 0 100 40], 'Callback', 'ep(''getPCAPrefs'');');
        
    case 'getPCAPrefs' %retrieve preference settings from the PCA preference input fields
        
        temp=EPmain.preferences.pca;
        
        EPmain.preferences.pca.mode=get(EPmain.handles.preferences.pca.mode,'Value');
        
        EPmain.preferences.pca.rotation=get(EPmain.handles.preferences.pca.rotation,'Value');
        
        EPmain.preferences.pca.rotopt=str2num(get(EPmain.handles.preferences.pca.rotopt,'String'));
        
        EPmain.preferences.pca.rel=get(EPmain.handles.preferences.pca.rel,'Value');
        
        EPmain.preferences.pca.loadings=get(EPmain.handles.preferences.pca.loadings,'Value');
        
        err=checkPrefs;
        
        if err
            msg{1}=['New preferences are incorrect.  Either correct or cancel.'];
            [msg]=ep_errorMsg(msg);
            EPmain.preferences.pca=temp;
        else
            EPmain.mode='preferenceMain';
            ep('start');
        end;
        
    case 'startPreferenceView'
        
        set(EPmain.handles.hMainWindow,'Name', 'View Pref');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Plot positive...',...
            'Position',[20 470 150 20]);
        
        EPmain.handles.preferences.view.positive = uicontrol('Style','popupmenu',...
            'String',{'up','down'},...
            'Value',EPmain.preferences.view.positive,'Position',[20 450 150 20],'TooltipString','When plotting waveforms, whether positive is up or down.');
        
        EPmain.handles.preferences.view.cancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
            'Position', [20 50 100 40], 'Callback', ['global EPmain;','EPmain.mode=''preferenceMain'';','ep(''start'');']);
        
        EPmain.handles.preferences.view.done = uicontrol('Style', 'pushbutton', 'String', 'Done',...
            'Position', [20 0 100 40], 'Callback', 'ep(''getViewPrefs'');');
        
    case 'getViewPrefs' %retrieve preference settings from the view preference input fields
        
        temp=EPmain.preferences.view;
        
        EPmain.preferences.view.positive=get(EPmain.handles.preferences.view.positive,'Value');
        
        err=checkPrefs;
        
        if err
            msg{1}=['New preferences are incorrect.  Either correct or cancel.'];
            [msg]=ep_errorMsg(msg);
            EPmain.preferences.view=temp;
        else
            EPmain.mode='preferenceMain';
            ep('start');
        end;
        
    case 'startPreferenceWindow'
        
        set(EPmain.handles.hMainWindow,'Name', 'Window Pref');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Min Factor Variance',...
            'Position',[20 470 150 20]);
        
        EPmain.handles.preferences.window.minFacVar = uicontrol('Style','edit','HorizontalAlignment','left',...
            'String',EPmain.preferences.window.minFacVar,'Position',[20 450 150 20],'TooltipString','When using autoPCA option, the minimum percent of variance for a factor to be included.');
        
        EPmain.handles.preferences.window.adds= uicontrol('Style','checkbox',...
            'String','Adds',...
            'Value',EPmain.preferences.window.adds,'Position',[20 430 150 20],'TooltipString','When windowing data, whether to add summary cell and channel waveforms corresponding to the analysis to the dataset.');
        
        EPmain.handles.preferences.window.cancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
            'Position', [20 50 100 40], 'Callback', ['global EPmain;','EPmain.mode=''preferenceMain'';','ep(''start'');']);
        
        EPmain.handles.preferences.window.done = uicontrol('Style', 'pushbutton', 'String', 'Done',...
            'Position', [20 0 100 40], 'Callback', 'ep(''getWindowPrefs'');');
        
    case 'getWindowPrefs' %retrieve preference settings from the window preference input fields
        
        temp=EPmain.preferences.window;
        
        EPmain.preferences.window.minFacVar=str2num(get(EPmain.handles.preferences.window.minFacVar,'String'));
        
        EPmain.preferences.window.adds=get(EPmain.handles.preferences.window.adds,'Value');
        
        err=checkPrefs;
        
        if err
            msg{1}=['New preferences are incorrect.  Either correct or cancel.'];
            [msg]=ep_errorMsg(msg);
            EPmain.preferences.window=temp;
        else
            EPmain.mode='preferenceMain';
            ep('start');
        end;
        
    case 'startPreferenceANOVA'
        
        set(EPmain.handles.hMainWindow,'Name', 'ANOVA Pref');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Trimming',...
            'Position',[20 470 150 20]);
        
        EPmain.handles.preferences.anova.trimming = uicontrol('Style','edit','HorizontalAlignment','left',...
            'String',EPmain.preferences.anova.trimming,'Position',[20 450 150 20],'TooltipString','Proportion of distribution of each cell''s tail to trim during ANOVA.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Bootstrap Samples',...
            'Position',[20 430 150 20]);
        
        EPmain.handles.preferences.anova.bootstrap = uicontrol('Style','edit','HorizontalAlignment','left',...
            'String',EPmain.preferences.anova.bootstrap,'Position',[20 410 150 20],'TooltipString','Number of times to run bootstrap to generate ANOVA distributions.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Seed',...
            'Position',[20 390 150 20]);
        
        EPmain.handles.preferences.anova.seed = uicontrol('Style','edit','HorizontalAlignment','left',...
            'String',EPmain.preferences.anova.seed,'Position',[20 370 150 20],'TooltipString','The starting seed number to use for random number generator for bootstrapping.');
        
        uicontrol('Style','text','HorizontalAlignment','left','String', 'Missing',...
            'Position',[20 350 150 20]);
        
        EPmain.handles.preferences.anova.missing = uicontrol('Style','edit','HorizontalAlignment','left',...
            'String',EPmain.preferences.anova.missing,'Position',[20 330 150 20],'TooltipString','Value used to indicate a missing number.');
        
        EPmain.handles.preferences.anova.adds= uicontrol('Style','checkbox',...
            'String','Adds',...
            'Value',EPmain.preferences.anova.adds,'Position',[20 310 150 20],'TooltipString','When conducting ANOVA, whether to add summary subject waveforms corresponding to the analysis to the dataset.');
        
        EPmain.handles.preferences.anova.cancel = uicontrol('Style', 'pushbutton', 'String', 'Cancel',...
            'Position', [20 50 100 40], 'Callback', ['global EPmain;','EPmain.mode=''preferenceMain'';','ep(''start'');']);
        
        EPmain.handles.preferences.anova.done = uicontrol('Style', 'pushbutton', 'String', 'Done',...
            'Position', [20 0 100 40], 'Callback', 'ep(''getANOVAPrefs'');');
        
    case 'getANOVAPrefs' %retrieve preference settings from the view preference input fields
        
        temp=EPmain.preferences.anova;
        
        EPmain.preferences.anova.trimming=str2num(get(EPmain.handles.preferences.anova.trimming,'String'));
        
        EPmain.preferences.anova.bootstrap=str2num(get(EPmain.handles.preferences.anova.bootstrap,'String'));
        
        EPmain.preferences.anova.seed=str2num(get(EPmain.handles.preferences.anova.seed,'String'));
        
        EPmain.preferences.anova.missing=str2num(get(EPmain.handles.preferences.anova.missing,'String'));
        
        EPmain.preferences.anova.adds=get(EPmain.handles.preferences.anova.adds,'Value');
        
        err=checkPrefs;
        
        if err
            msg{1}=['New preferences are incorrect.  Either correct or cancel.'];
            [msg]=ep_errorMsg(msg);
            EPmain.preferences.anova=temp;
        else
            EPmain.mode='preferenceMain';
            ep('start');
        end;
        
    case 'savePrefs' %save current preference settings
        
        PathName=userpath;
        if any(strcmp(PathName(end),{';',':'}))
            PathName=PathName(1:end-1);
        end;
        
        prefs=EPmain.preferences;
        if exist('~/Library/Preferences/EPprefs.mat','file')
            eval('save ~/Library/Preferences/EPprefs.mat prefs');
        elseif exist('EPprefs.mat','file')
            location = which('EPprefs.mat');
            eval(['save ''' location ''' prefs']);
        elseif exist('~/Library/Preferences','dir')
            resetPrefs;
            prefs=EPmain.preferences;
            eval('save ~/Library/Preferences/EPprefs.mat prefs');
        elseif exist([PathName filesep 'EPwork' filesep 'EPprefs.mat'],'file')
            save([PathName filesep 'EPwork' filesep 'EPprefs.mat']);
        else
            [FileName,PathName,FilterIndex] = uiputfile('','Save Preferences File','EPprefs');
            eval(['save ''' PathName 'EPprefs'' prefs']);
            if ~strcmp(PathName,path)
                addpath(PathName,'-end'); %adds directory with preferences file to Matlab's path
            end;
        end;
        
        EPmain.mode='main';
        ep('start');
        
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function deleteReadData(src,eventdata)
global EPdataset EPmain

if ~EPmain.noTable
    if isempty(eventdata.Indices) %if just deselecting
        return;
    end;
    theDataset=eventdata.Indices(1);
else
    theDataset=get(EPmain.handles.read.menu,'Value');
end;

delete([EPdataset.EPwork filesep 'EPwork' filesep EPdataset.dataset(theDataset).dataName '.mat']);
EPdataset.dataset=EPdataset.dataset(setdiff([1:length(EPdataset.dataset)],theDataset));
eval(['save ''' EPdataset.EPwork filesep 'EPwork' filesep 'EPdataset''']);

ep('start');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function pickEditData(src,eventdata)
global EPoverview EPmain

if ~EPmain.noTable
    if isempty(eventdata.Indices) %if just deselecting
        return;
    end;
end;

if EPmain.noTable
    EPoverview.dataset=get(EPmain.handles.edit.menu,'Value');
else
    EPoverview.dataset=eventdata.Indices(1);
end;

EPoverview.mode=[];
ep_editData


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function pickPCAdata(src,eventdata)
global EPmain EPdataset EPscree

if ~EPmain.noTable
    if isempty(eventdata.Indices) %if just deselecting
        return;
    end;
end;

EPscree=[];
screeFigure=findobj('Name', 'ScreeWindow');
if ~isempty(screeFigure)
    close(screeFigure)
end;

if EPmain.noTable
    theDataset=get(EPmain.handles.pca.menu,'Value');
else
    theDataset=eventdata.Indices(1);
end;
theData=ep_stripAdds(ep_loadEPdataset(EPdataset,theDataset));

if isempty(theData.data)
    warndlg(['Error: The file had no data left after additions were removed.']);
    return;
end;

modeNum = get(EPmain.handles.pca.mode,'value');
switch modeNum
    case 1
        PCAmode='spat';
    case 2
        PCAmode='temp';
end;
EPmain.pca.mode=modeNum;

rotationNum = get(EPmain.handles.pca.rotation,'value');
switch rotationNum
    case 1
        PCArotation='VMAX';
    case 2
        PCArotation='PMAX';
    case 3
        PCArotation='IMAX';
    case 4
        PCArotation='QMAX';
    case 5
        PCArotation='QMIN';
    case 6
        PCArotation='OMIN';
    case 7
        PCArotation='CRFE';
    case 8
        PCArotation='MINE';
    case 9
        PCArotation='IPSC';
    case 10
        PCArotation='TIIC';
    case 11
        PCArotation='GMIN';
    case 12
        PCArotation='MMER';
    case 13
        PCArotation='VOMN';
    case 14
        PCArotation='UNRT';
end;

EPmain.pca.rotation=rotationNum;

EPmain.pca.rotopt=str2num(get(EPmain.handles.pca.rotopt,'String'));

relNum = get(EPmain.handles.pca.rel,'value');
switch relNum
    case 1
        PCArel='COR';
    case 2
        PCArel='COV';
end;
EPmain.pca.rel=relNum;

loadingNum = get(EPmain.handles.pca.loadings,'value');
switch loadingNum
    case 1
        PCAloading='N';
    case 2
        PCAloading='K';
    case 3
        PCAloading='C';
    case 4
        PCAloading='W';
end;
EPmain.pca.loading=loadingNum;

parametric = get(EPmain.handles.pca.parametric,'value');
EPmain.pca.parametric=parametric;

EPmain.pca.facNum=str2num(get(EPmain.handles.pca.facNum,'String'));

PCAname=get(EPmain.handles.pca.name,'String');

if parametric
    [parametricFile, pathname] = uigetfile('*.*','Parametric File');
    parametricData=importdata([pathname parametricFile],sprintf('\t'),1);
    if isempty(parametricData)
        msg{1}=['Was unable to retrieve the parametric data.  It needs to be a tab-delimited text file in which the first row are the labels.'];
        [msg]=ep_errorMsg(msg);
        return
    end;
else
    parametricData.data=[];
    parametricData.colheaders=[];
end;

if isempty(PCAname)
    PCAname='PCA';
end;
sameName=1;
suffix=0;
PCAnameSuffix=PCAname;
while sameName
    sameName=0;
    for i=1:length(EPdataset.dataset)
        if strcmp(EPdataset.dataset(i).dataName,PCAnameSuffix)
            sameName=1;
        end;
    end;
    if sameName
        suffix=suffix+1;
        PCAnameSuffix=[PCAname '-' num2str(suffix)];
    end;
end;

if ~isfield(theData,'pca')
    disp('Starting the PCA.');
    if EPmain.pca.facNum==0 %scree test only needs unrotated solution
        disp('Scree: The PCA of the data.');
        FactorResults = ep_doPCA(PCAmode, 'UNRT', EPmain.pca.rotopt, PCArel, 1, theData, PCAloading);
        if isempty(FactorResults)
            msg{1}='Error: PCA was not successful.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        if isfield(theData,'noise')
            disp('Scree: The PCA of the noise data.');
            noiseResults = ep_doPCA(PCAmode, 'UNRT', EPmain.pca.rotopt, PCArel, 1, theData.noise, PCAloading);
            if isempty(noiseResults)
                msg{1}='Error: PCA was not successful.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        else
            noiseResults = [];
        end;
        disp('Scree: The PCA of the random data.');
        randResults = ep_doPCA(PCAmode, 'UNRT', EPmain.pca.rotopt, PCArel, 1, randn(size(theData.data)), PCAloading);
        if isempty(randResults)
            msg{1}='Error: PCA was not successful.';
            [msg]=ep_errorMsg(msg);
            return
        end;
    else
        FactorResults = ep_doPCA(PCAmode, PCArotation, EPmain.pca.rotopt, PCArel, EPmain.pca.facNum, theData, PCAloading);
        if isempty(FactorResults)
            msg{1}='Error: PCA was not successful.';
            [msg]=ep_errorMsg(msg);
            return
        end;
        
        %jack-knife analysis
        numSubs=length(theData.subNames);
        if numSubs > 1
            disp('Computing Jack-Knife PCA loadings.');
            fprintf('%60s\n',' ' );
            jackLoadings=zeros(size(FactorResults.FacPat,1),size(FactorResults.FacPat,2),numSubs);
            jackSD=zeros(length(FactorResults.varSD),numSubs);
            for theSubject=1:numSubs
                fprintf('%s%-60s',repmat(sprintf('\b'),1,60),sprintf('%s%4d%s%4d.','Working on subject# ', theSubject, ' of ', numSubs))
                theDataJK=theData;
                subjectList=setdiff([1:numSubs],theSubject);
                theDataJK.data=theDataJK.data(:,:,:,subjectList);
                theDataJK.subNames=theDataJK.subNames(subjectList);
                theDataJK.subTypes=theDataJK.subTypes(subjectList);
                if ~isempty(theDataJK.subjectSpecs)
                    theDataJK.subjectSpecs=theDataJK.subjectSpecs(subjectList,:);
                end;
                theDataJK.avgNum=theDataJK.avgNum(subjectList,:);
                theDataJK.subNum=theDataJK.subNum(subjectList,:);
                theDataJK.events=theDataJK.events(subjectList,:);
                theDataJK.analysis.badChans=theDataJK.analysis.badChans(subjectList,:,:);
                theDataJK.analysis.moveTrial=theDataJK.analysis.moveTrial(subjectList,:);
                theDataJK.analysis.blinkTrial=theDataJK.analysis.blinkTrial(subjectList,:);
                theDataJK.analysis.saccadeTrial=theDataJK.analysis.saccadeTrial(subjectList,:);
                theDataJK.analysis.saccadeOnset=theDataJK.analysis.saccadeOnset(subjectList,:);
                theDataJK.analysis.badTrials=theDataJK.analysis.badTrials(subjectList,:);
                FactorResultsJK = ep_doPCA(PCAmode, PCArotation, EPmain.pca.rotopt, PCArel, EPmain.pca.facNum, theDataJK, PCAloading);
                if ~isempty(FactorResultsJK)
                    jackLoadings(:,:,theSubject)=FactorResultsJK.FacPat;
                    jackSD(:,theSubject)=FactorResultsJK.varSD;
                end;
            end;
            FactorResults.jack.FacPat=jackLoadings;
            FactorResults.jack.varSD=jackSD;
            fprintf('%60s\n',' ' );
        end;
    end;
elseif isfield(theData.pca,'secondPCAmode')
    msg{1}='These data have already undergone a two-step PCA process.';
    [msg]=ep_errorMsg(msg);
    return
elseif isfield(theData.pca,'PCAmode')
    if strcmp(PCAmode,theData.pca.PCAmode)
        if strcmp(PCAmode,'temp')
            msg{1}='These data have already undergone the temporal PCA step.';
        elseif strcmp(PCAmode,'spat')
            msg{1}='These data have already undergone the spatial PCA step.';
        else
            msg{1}='Error: PCA mode not recognized.';
        end;
        [msg]=ep_errorMsg(msg);
        return
    else
        if EPmain.pca.facNum==0
            disp('Scree: The PCA of the data.');
            FactorResults = ep_doPCAst(theData.pca, 'UNRT', EPmain.pca.rotopt, PCArel, 1, PCAloading); %scree test only needs unrotated solution
            if isempty(FactorResults)
                msg{1}='Error: PCA was not successful.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            if isfield(theData,'noise')
                %perform two-step PCA of noise data.  Since it is meant to show results when the signal is removed,
                %use the first step scoring coefficients so that they are within the same "window" as the regular data.
                
                disp('Scree: The PCA of the noise data.');
                noiseResults=theData.pca;
                
                numchan=size(noiseResults.noise,1);
                timePoints=size(noiseResults.noise,2);
                numCells=size(noiseResults.noise,3);
                numSubs=size(noiseResults.noise,4);
                
                if strcmp(noiseResults.PCAmode,'temp')
                    data2=zeros(numchan*numCells*numSubs,timePoints);
                    for cell = 1:numCells
                        for sub = 1:numSubs
                            data2((sub-1)*numchan+(cell-1)*numSubs*numchan+1:sub*numchan+(cell-1)*numSubs*numchan,:)=squeeze(noiseResults.noise(:,:,cell,sub));
                        end
                    end
                elseif strcmp(noiseResults.PCAmode,'spat')
                    data2=zeros(timePoints*numCells*numSubs,numchan);
                    for cell = 1:numCells
                        for sub = 1:numSubs
                            data2((sub-1)*timePoints+(cell-1)*numSubs*timePoints+1:sub*timePoints+(cell-1)*numSubs*timePoints,:)=squeeze(noiseResults.noise(:,:,cell,sub))';
                        end
                    end
                elseif strcmp(noiseResults.PCAmode,'asis')
                    data2=noiseResults.noise;
                else
                    error('PCAmode must be set to either temp or spat or asis');
                end;
                
                noiseResults.FacScr=data2*noiseResults.FacCof;
                noiseResults.FacScr=(noiseResults.FacScr)*inv(diag(std(noiseResults.FacScr))); %Standardize factor scores, not mean corrected.
                noiseResults = ep_doPCAst(noiseResults, 'UNRT', EPmain.pca.rotopt, PCArel, 1, PCAloading);
                if isempty(noiseResults)
                    msg{1}='Error: PCA was not successful.';
                    [msg]=ep_errorMsg(msg);
                    return
                end;
            else
                noiseResults = [];
            end;
            disp('Scree: The PCA of the random data.');
            randResults = ep_doPCA(theData.pca.PCAmode, theData.pca.ROTATION, theData.pca.RotOpt, theData.pca.MAT_TYPE, theData.pca.numFacs, randn(theData.pca.numchan,theData.pca.timepoints,theData.pca.numCells,theData.pca.numSubs), theData.pca.LOADING);
            randResults = ep_doPCAst(randResults, 'UNRT', EPmain.pca.rotopt, PCArel, 1, PCAloading);
            if isempty(randResults)
                msg{1}='Error: PCA was not successful.';
                [msg]=ep_errorMsg(msg);
                return
            end;
        else
            [FactorResults] = ep_doPCAst(theData.pca, PCArotation, EPmain.pca.rotopt, PCArel, EPmain.pca.facNum, PCAloading);
            if isempty(FactorResults)
                msg{1}='Error: PCA was not successful.';
                [msg]=ep_errorMsg(msg);
                return
            end;
            
            %jack-knife analysis
            numSubs=length(theData.subNames);
            numCells=length(theData.cellNames);
            numVars=size(FactorResults.FacScr,1)/(numSubs*numCells);
            if numSubs > 1
                disp('Computing Jack-Knife PCA loadings.');
                fprintf('%60s\n',' ' );
                jackLoadingsST=zeros(size(FactorResults.FacPatST,1),size(FactorResults.FacPatST,2),numSubs);
                jackSDST=zeros(size(FactorResults.varSDST,1),size(FactorResults.varSDST,2),numSubs);
                for theSubject=1:numSubs
                    fprintf('%s%-60s',repmat(sprintf('\b'),1,60),sprintf('%s%4d%s%4d.','Working on subject# ', theSubject, ' of ', numSubs))
                    theDataJK=theData.pca;
                    subjectVec=ones(numSubs,1);
                    subjectVec(theSubject)=0;
                    subjectList=kron(ones(numCells,1),subjectVec);
                    subjectList=logical(kron(subjectList,ones(numVars,1)));
                    theDataJK.FacScr=theDataJK.FacScr(subjectList,:);
                    [FactorResultsJK] = ep_doPCAst(theDataJK, PCArotation, EPmain.pca.rotopt, PCArel, EPmain.pca.facNum, PCAloading);
                    if ~isempty(FactorResultsJK)
                        jackLoadingsST(:,:,theSubject)=FactorResultsJK.FacPatST;
                        jackSDST(:,:,theSubject)=FactorResultsJK.varSDST;
                    end;
                end;
                FactorResults.jack.FacPatST=jackLoadingsST;
                FactorResults.jack.varSDST=jackSDST;
                fprintf('%60s\n',' ' );
            end;
        end;
    end;
else
    msg{1}='These data have already undergone a PCA process but is corrupted in some manner.';
    [msg]=ep_errorMsg(msg);
    return
end;

if EPmain.pca.facNum==0
    ep_scree(FactorResults,noiseResults,randResults);
else
    if parametric
        if size(parametricData.data,1) ~= size(FactorResults.FacScr,1)
            msg{1}='The number of observations in the parameters file does not match the data.  It needs to be a tab-delimited text file in which the first row are the labels.';
            [msg]=ep_errorMsg(msg);
            return
        end;
    end;
    
    cnames = theData.cellNames;
    for i=1:length(cnames)
        cellcoll{i} = i;
    end;
    exclChan=[];
    
    [PCAoutput, peakLatency, peakSamp, peakChan] = ep_PCAoutput(FactorResults, cnames, cellcoll, exclChan, parametricData.data, parametricData.colheaders);
    if isempty(PCAoutput)
        msg{1}='PCA data reconstruction aborted.';
        [msg]=ep_errorMsg(msg);
        return
    end;
    
    PCAoutput.dataName=PCAnameSuffix;
    EPdataset=ep_saveEPdataset(EPdataset,PCAoutput,length(EPdataset.dataset)+1,'no');
end;


ep('start');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function resetPrefs(src,eventdata)
%reset the permanent preference settings

global EPmain

EPmain.preferences.general.sessionImportFormat=7; %EP format
EPmain.preferences.general.sessionOutputFormat=5; %EP format
EPmain.preferences.general.importFormat=7; %EP format
EPmain.preferences.general.outputFormat=5; %EP format
EPmain.preferences.general.firstRow=1;
EPmain.preferences.general.firstCol=1;
EPmain.preferences.general.lastCol=0;
EPmain.preferences.general.orientation=1;
EPmain.preferences.general.montage=1;

EPmain.preferences.preprocess.saturation=inf;
EPmain.preferences.preprocess.window=80;
EPmain.preferences.preprocess.minmax=100;
EPmain.preferences.preprocess.badnum=10;
EPmain.preferences.preprocess.neighbors=6;
EPmain.preferences.preprocess.maxneighbor=30;
EPmain.preferences.preprocess.badchan=.4;
EPmain.preferences.preprocess.blink=.9;
EPmain.preferences.preprocess.badtrials=20;
EPmain.preferences.preprocess.chunkSize=200000;
EPmain.preferences.preprocess.minTrialsPerCell=15;
EPmain.preferences.preprocess.noadjacent=0;
EPmain.preferences.preprocess.trialminmax=200;
EPmain.preferences.preprocess.movefacs=20;
EPmain.preferences.preprocess.noFigure=0;
EPmain.preferences.preprocess.saccade=.8;
EPmain.preferences.preprocess.saccademin=20;
EPmain.preferences.preprocess.EOGchans=[];

EPmain.preferences.average.method=1;
EPmain.preferences.average.trimLevel=.25;

EPmain.preferences.postprocess.reference=1; %average reference
EPmain.preferences.postprocess.refChan1=57;
EPmain.preferences.postprocess.refChan2=101;
EPmain.preferences.postprocess.baseline='1:50';

EPmain.preferences.view.positive=1;

EPmain.preferences.pca.mode=2;
EPmain.preferences.pca.rotation=2;
EPmain.preferences.pca.rotopt=3;
EPmain.preferences.pca.rel=2;
EPmain.preferences.pca.loadings=2;

EPmain.preferences.window.minFacVar=.005;
EPmain.preferences.window.adds=1;

EPmain.preferences.anova.trimming=.05;
EPmain.preferences.anova.missing=-999;
EPmain.preferences.anova.seed=1000;
EPmain.preferences.anova.bootstrap=50000;
EPmain.preferences.anova.adds=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function err=checkPrefs(src,eventdata) %check the preference settings
global EPmain

err=[];

if isfield(EPmain.preferences,'general')
    if ~isfield(EPmain.preferences.general,'sessionImportFormat')
        err = [err ;14];
        disp('The EPmain.preferences.general.sessionImportFormat field is missing.');
    elseif isempty(EPmain.preferences.general.sessionImportFormat)
        err = [err ;14];
        disp('The EPmain.preferences.general.sessionImportFormat field is empty.');
    elseif ~isnumeric(EPmain.preferences.general.sessionImportFormat)
        err = [err ;14];
        disp('The EPmain.preferences.general.sessionImportFormat field is not a number.');
    elseif EPmain.preferences.general.sessionImportFormat < 1 || EPmain.preferences.general.sessionImportFormat > 7
        err = [err ;14];
        disp('The EPmain.preferences.general.sessionImportFormat field is not within the range of 1 to 7.');
    end;
    
    if ~isfield(EPmain.preferences.general,'sessionOutputFormat')
        err = [err ;15];
        disp('The EPmain.preferences.general.sessionOutputFormat field is missing.');
    elseif isempty(EPmain.preferences.general.sessionOutputFormat)
        err = [err ;15];
        disp('The EPmain.preferences.general.sessionOutputFormat field is empty.');
    elseif ~isnumeric(EPmain.preferences.general.sessionOutputFormat)
        err = [err ;15];
        disp('The EPmain.preferences.general.sessionOutputFormat field is not a number.');
    elseif EPmain.preferences.general.sessionOutputFormat < 1 || EPmain.preferences.general.sessionOutputFormat > 5
        err = [err ;15];
        disp('The EPmain.preferences.general.sessionOutputFormat field is not within the range of 1 to 5.');
    end;
    if ~isfield(EPmain.preferences.general,'importFormat')
        err = [err ;19];
        disp('The EPmain.preferences.general.importFormat field is missing.');
    elseif isempty(EPmain.preferences.general.importFormat)
        err = [err ;19];
        disp('The EPmain.preferences.general.importFormat field is empty.');
    elseif ~isnumeric(EPmain.preferences.general.importFormat)
        err = [err ;19];
        disp('The EPmain.preferences.general.importFormat field is not a number.');
    elseif EPmain.preferences.general.importFormat < 1 || EPmain.preferences.general.importFormat > 7
        err = [err ;19];
        disp('The EPmain.preferences.general.importFormat field is not within the range of 1 to 7.');
    end;
    
    if ~isfield(EPmain.preferences.general,'outputFormat')
        err = [err ;20];
        disp('The EPmain.preferences.general.outputFormat field is missing.');
    elseif isempty(EPmain.preferences.general.outputFormat)
        err = [err ;20];
        disp('The EPmain.preferences.general.outputFormat field is empty.');
    elseif ~isnumeric(EPmain.preferences.general.outputFormat)
        err = [err ;20];
        disp('The EPmain.preferences.general.outputFormat field is not a number.');
    elseif EPmain.preferences.general.outputFormat < 1 || EPmain.preferences.general.outputFormat > 5
        err = [err ;20];
        disp('The EPmain.preferences.general.outputFormat field is not within the range of 1 to 5.');
    end;
    
    if ~isfield(EPmain.preferences.general,'firstRow')
        err = [err ;47];
        disp('The EPmain.preferences.general.firstRow field is missing.');
    elseif isempty(EPmain.preferences.general.firstRow)
        err = [err ;47];
        disp('The EPmain.preferences.general.firstRow field is empty.');
    elseif ~isnumeric(EPmain.preferences.general.firstRow)
        err = [err ;47];
        disp('The EPmain.preferences.general.firstRow field is not a number.');
    elseif EPmain.preferences.general.firstRow < 1
        err = [err ;47];
        disp('The EPmain.preferences.general.firstRow field is less than 1.');
    end;
    
    if ~isfield(EPmain.preferences.general,'firstCol')
        err = [err ;48];
        disp('The EPmain.preferences.general.firstCol field is missing.');
    elseif isempty(EPmain.preferences.general.firstCol)
        err = [err ;48];
        disp('The EPmain.preferences.general.firstCol field is empty.');
    elseif ~isnumeric(EPmain.preferences.general.firstCol)
        err = [err ;48];
        disp('The EPmain.preferences.general.firstCol field is not a number.');
    elseif EPmain.preferences.general.firstCol < 1
        err = [err ;48];
        disp('The EPmain.preferences.general.firstCol field is less than 1.');
    end;
    
    if ~isfield(EPmain.preferences.general,'lastCol')
        err = [err ;49];
        disp('The EPmain.preferences.general.lastCol field is missing.');
    elseif isempty(EPmain.preferences.general.lastCol)
        err = [err ;49];
        disp('The EPmain.preferences.general.lastCol field is empty.');
    elseif ~isnumeric(EPmain.preferences.general.lastCol)
        err = [err ;49];
        disp('The EPmain.preferences.general.lastCol field is not a number.');
    elseif EPmain.preferences.general.lastCol < 0
        err = [err ;49];
        disp('The EPmain.preferences.general.lastCol field is less than zero.');
    end;
    
    if ~isfield(EPmain.preferences.general,'orientation')
        err = [err ;50];
        disp('The EPmain.preferences.general.orientation field is missing.');
    elseif isempty(EPmain.preferences.general.orientation)
        err = [err ;50];
        disp('The EPmain.preferences.general.orientation field is empty.');
    elseif ~isnumeric(EPmain.preferences.general.orientation)
        err = [err ;50];
        disp('The EPmain.preferences.general.orientation field is not a number.');
    elseif EPmain.preferences.general.orientation < 1 || EPmain.preferences.general.orientation > 2
        err = [err ;50];
        disp('The EPmain.preferences.general.orientation field is not within the range of 1 to 2.');
    end;
    
    if ~isfield(EPmain.preferences.general,'montage')
        err = [err ;52];
        disp('The EPmain.preferences.general.montage field is missing.');
    elseif isempty(EPmain.preferences.general.montage)
        err = [err ;52];
        disp('The EPmain.preferences.general.montage field is empty.');
    elseif ~isnumeric(EPmain.preferences.general.montage)
        err = [err ;52];
        disp('The EPmain.preferences.general.montage field is not a number.');
    elseif EPmain.preferences.general.montage ~= 0 && EPmain.preferences.general.montage ~= 1
        err = [err ;52];
        disp('The EPmain.preferences.general.montage field does not equal zero or one.');
    end;
else
    err = [err ;17];
    disp('The EPmain.preferences.general field is missing.');
end

if isfield(EPmain.preferences,'preprocess')
    if ~isfield(EPmain.preferences.preprocess,'saturation')
        err = [err ;1];
        disp('The EPmain.preferences.preprocess.saturation field is missing.');
    elseif isempty(EPmain.preferences.preprocess.saturation)
        err = [err ;1];
        disp('The EPmain.preferences.preprocess.saturation field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.saturation)
        err = [err ;1];
        disp('The EPmain.preferences.preprocess.saturation field is not a number.');
    elseif EPmain.preferences.preprocess.saturation == 0
        err = [err ;1];
        disp('The EPmain.preferences.preprocess.saturation field is zero.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'window')
        err = [err ;2];
        disp('The EPmain.preferences.preprocess.window field is missing.');
    elseif isempty(EPmain.preferences.preprocess.window)
        err = [err ;2];
        disp('The EPmain.preferences.preprocess.window field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.window)
        err = [err ;2];
        disp('The EPmain.preferences.preprocess.window field is not a number.');
    elseif EPmain.preferences.preprocess.window < 1
        err = [err ;2];
        disp('The EPmain.preferences.preprocess.window field is less than one.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'minmax')
        err = [err ;3];
        disp('The EPmain.preferences.preprocess.minmax field is missing.');
    elseif isempty(EPmain.preferences.preprocess.minmax)
        err = [err ;3];
        disp('The EPmain.preferences.preprocess.minmax field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.minmax)
        err = [err ;3];
        disp('The EPmain.preferences.preprocess.minmax field is not a number.');
    elseif EPmain.preferences.preprocess.saturation < 1
        err = [err ;3];
        disp('The EPmain.preferences.preprocess.minmax field is less than one.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'badnum')
        err = [err ;4];
        disp('The EPmain.preferences.preprocess.badnum field is missing.');
    elseif isempty(EPmain.preferences.preprocess.badnum)
        err = [err ;4];
        disp('The EPmain.preferences.preprocess.badnum field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.badnum)
        err = [err ;4];
        disp('The EPmain.preferences.preprocess.badnum field is not a number.');
    elseif EPmain.preferences.preprocess.badnum < 0 || EPmain.preferences.preprocess.badnum > 100
        err = [err ;4];
        disp('The EPmain.preferences.preprocess.badnum field is not between 0 and 100.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'neighbors')
        err = [err ;6];
        disp('The EPmain.preferences.preprocess.neighbors field is missing.');
    elseif isempty(EPmain.preferences.preprocess.neighbors)
        err = [err ;6];
        disp('The EPmain.preferences.preprocess.neighbors field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.neighbors)
        err = [err ;6];
        disp('The EPmain.preferences.preprocess.neighbors field is not a number.');
    elseif EPmain.preferences.preprocess.neighbors < 0
        err = [err ;6];
        disp('The EPmain.preferences.preprocess.neighbors field is less than zero.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'maxneighbor')
        err = [err ;7];
        disp('The EPmain.preferences.preprocess.maxneighbor field is missing.');
    elseif isempty(EPmain.preferences.preprocess.maxneighbor)
        err = [err ;7];
        disp('The EPmain.preferences.preprocess.maxneighbor field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.maxneighbor)
        err = [err ;7];
        disp('The EPmain.preferences.preprocess.maxneighbor field is not a number.');
    elseif EPmain.preferences.preprocess.maxneighbor < 0
        err = [err ;7];
        disp('The EPmain.preferences.preprocess.maxneighbor field is less than zero.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'badchan')
        err = [err ;8];
        disp('The EPmain.preferences.preprocess.badchan field is missing.');
    elseif isempty(EPmain.preferences.preprocess.badchan)
        err = [err ;8];
        disp('The EPmain.preferences.preprocess.badchan field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.badchan)
        err = [err ;8];
        disp('The EPmain.preferences.preprocess.badchan field is not a number.');
    elseif EPmain.preferences.preprocess.badchan < 0 || EPmain.preferences.preprocess.badchan > 1
        err = [err ;8];
        disp('The EPmain.preferences.preprocess.badchan field is not between 0 and 1.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'blink')
        err = [err ;9];
        disp('The EPmain.preferences.preprocess.blink field is missing.');
    elseif isempty(EPmain.preferences.preprocess.blink)
        err = [err ;9];
        disp('The EPmain.preferences.preprocess.blink field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.blink)
        err = [err ;9];
        disp('The EPmain.preferences.preprocess.blink field is not a number.');
    elseif EPmain.preferences.preprocess.blink < 0 || EPmain.preferences.preprocess.blink > 1
        err = [err ;9];
        disp('The EPmain.preferences.preprocess.blink field is not between 0 and 1.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'badtrials')
        err = [err ;10];
        disp('The EPmain.preferences.preprocess.badtrials field is missing.');
    elseif isempty(EPmain.preferences.preprocess.badtrials)
        err = [err ;10];
        disp('The EPmain.preferences.preprocess.badtrials field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.badtrials)
        err = [err ;10];
        disp('The EPmain.preferences.preprocess.badtrials field is not a number.');
    elseif EPmain.preferences.preprocess.badtrials < 0 || EPmain.preferences.preprocess.badtrials > 100
        err = [err ;10];
        disp('The EPmain.preferences.preprocess.badtrials field is not between 0 and 100.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'chunkSize')
        err = [err ;11];
        disp('The EPmain.preferences.preprocess.chunkSize field is missing.');
    elseif isempty(EPmain.preferences.preprocess.chunkSize)
        err = [err ;11];
        disp('The EPmain.preferences.preprocess.chunkSize field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.chunkSize)
        err = [err ;11];
        disp('The EPmain.preferences.preprocess.chunkSize field is not a number.');
    elseif EPmain.preferences.preprocess.chunkSize < 1
        err = [err ;11];
        disp('The EPmain.preferences.preprocess.chunkSize field is less than one.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'minTrialsPerCell')
        err = [err ;12];
        disp('The EPmain.preferences.preprocess.minTrialsPerCell field is missing.');
    elseif isempty(EPmain.preferences.preprocess.minTrialsPerCell)
        err = [err ;12];
        disp('The EPmain.preferences.preprocess.minTrialsPerCell field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.minTrialsPerCell)
        err = [err ;12];
        disp('The EPmain.preferences.preprocess.minTrialsPerCell field is not a number.');
    elseif EPmain.preferences.preprocess.minTrialsPerCell < 0
        err = [err ;12];
        disp('The EPmain.preferences.preprocess.minTrialsPerCell field is less than zero.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'noadjacent')
        err = [err ;13];
        disp('The EPmain.preferences.preprocess.noadjacent field is missing.');
    elseif isempty(EPmain.preferences.preprocess.noadjacent)
        err = [err ;13];
        disp('The EPmain.preferences.preprocess.noadjacent field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.noadjacent)
        err = [err ;13];
        disp('The EPmain.preferences.preprocess.noadjacent field is not a number.');
    elseif EPmain.preferences.preprocess.noadjacent ~= 0 && EPmain.preferences.preprocess.noadjacent ~= 1
        err = [err ;13];
        disp('The EPmain.preferences.preprocess.noadjacent field does not equal zero or one.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'trialminmax')
        err = [err ;18];
        disp('The EPmain.preferences.preprocess.trialminmax field is missing.');
    elseif isempty(EPmain.preferences.preprocess.trialminmax)
        err = [err ;18];
        disp('The EPmain.preferences.preprocess.trialminmax field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.trialminmax)
        err = [err ;18];
        disp('The EPmain.preferences.preprocess.trialminmax field is not a number.');
    elseif EPmain.preferences.preprocess.trialminmax < 0
        err = [err ;18];
        disp('The EPmain.preferences.preprocess.trialminmax field is less than zero.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'movefacs')
        err = [err ;21];
        disp('The EPmain.preferences.preprocess.movefacs field is missing.');
    elseif isempty(EPmain.preferences.preprocess.movefacs)
        err = [err ;21];
        disp('The EPmain.preferences.preprocess.movefacs field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.movefacs)
        err = [err ;21];
        disp('The EPmain.preferences.preprocess.movefacs field is not a number.');
    elseif EPmain.preferences.preprocess.movefacs < 0
        err = [err ;21];
        disp('The EPmain.preferences.preprocess.movefacs field is less than zero.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'noFigure')
        err = [err ;51];
        disp('The EPmain.preferences.preprocess.noFigure field is missing.');
    elseif isempty(EPmain.preferences.preprocess.noFigure)
        err = [err ;51];
        disp('The EPmain.preferences.preprocess.noFigure field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.noFigure)
        err = [err ;51];
        disp('The EPmain.preferences.preprocess.noFigure field is not a number.');
    elseif EPmain.preferences.preprocess.noFigure ~= 0 && EPmain.preferences.preprocess.noFigure ~= 1
        err = [err ;51];
        disp('The EPmain.preferences.preprocess.noFigure field does not equal zero or one.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'saccade')
        err = [err ;53];
        disp('The EPmain.preferences.preprocess.saccade field is missing.');
    elseif isempty(EPmain.preferences.preprocess.saccade)
        err = [err ;53];
        disp('The EPmain.preferences.preprocess.saccade field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.saccade)
        err = [err ;53];
        disp('The EPmain.preferences.preprocess.saccade field is not a number.');
    elseif EPmain.preferences.preprocess.saccade < 0 || EPmain.preferences.preprocess.saccade > 1
        err = [err ;53];
        disp('The EPmain.preferences.preprocess.saccade field is not between 0 and 1.');
    end;
    
    if ~isfield(EPmain.preferences.preprocess,'saccademin')
        err = [err ;54];
        disp('The EPmain.preferences.preprocess.saccademin field is missing.');
    elseif isempty(EPmain.preferences.preprocess.saccademin)
        err = [err ;54];
        disp('The EPmain.preferences.preprocess.saccademin field is empty.');
    elseif ~isnumeric(EPmain.preferences.preprocess.saccademin)
        err = [err ;54];
        disp('The EPmain.preferences.preprocess.saccademin field is not a number.');
    elseif EPmain.preferences.preprocess.saccademin < 0
        err = [err ;54];
        disp('The EPmain.preferences.preprocess.saccademin field is less than zero.');
    end;

    if ~isfield(EPmain.preferences.preprocess,'EOGchans')
        err = [err ;55];
        disp('The EPmain.preferences.preprocess.EOGchans field is missing.');
    elseif ~isnumeric(EPmain.preferences.preprocess.EOGchans)
        err = [err ;55];
        disp('The EPmain.preferences.preprocess.EOGchans field is not a number.');
    elseif length(EPmain.preferences.preprocess.EOGchans) ~= 6 && ~isempty(EPmain.preferences.preprocess.EOGchans)
        err = [err ;55];
        disp('The EPmain.preferences.preprocess.EOGchans field does not have six numbers in it.');
    end;

else
    err = [err ;16];
    disp('The EPmain.preferences.preprocess field is missing.');
end

if isfield(EPmain.preferences,'average')
    if ~isfield(EPmain.preferences.average,'method')
        err = [err ;23];
        disp('The EPmain.preferences.average.method field is missing.');
    elseif isempty(EPmain.preferences.average.method)
        err = [err ;23];
        disp('The EPmain.preferences.average.method field is empty.');
    elseif ~isnumeric(EPmain.preferences.average.method)
        err = [err ;23];
        disp('The EPmain.preferences.average.method field is not a number.');
    elseif EPmain.preferences.average.method < 1 && EPmain.preferences.average.method > 3
        err = [err ;23];
        disp('The EPmain.preferences.average.method field is not within the range of 1 to 3.');
    end;
    if ~isfield(EPmain.preferences.average,'trimLevel')
        err = [err ;24];
        disp('The EPmain.preferences.average.trimLevel field is missing.');
    elseif isempty(EPmain.preferences.average.trimLevel)
        err = [err ;24];
        disp('The EPmain.preferences.average.trimLevel field is empty.');
    elseif ~isnumeric(EPmain.preferences.average.trimLevel)
        err = [err ;24];
        disp('The EPmain.preferences.average.trimLevel field is not a number.');
    elseif ~(EPmain.preferences.average.trimLevel < .5 && EPmain.preferences.average.trimLevel > 0)
        err = [err ;24];
        disp('The EPmain.preferences.average.trimLevel field is not between 0 and .5.');
    end;
else
    err = [err ;22];
    disp('The EPmain.preferences.average field is missing.');
end

if isfield(EPmain.preferences,'postprocess')
    if ~isfield(EPmain.preferences.postprocess,'reference')
        err = [err ;26];
        disp('The EPmain.preferences.postprocess.reference field is missing.');
    elseif isempty(EPmain.preferences.postprocess.reference)
        err = [err ;26];
        disp('The EPmain.preferences.postprocess.reference field is empty.');
    elseif ~isnumeric(EPmain.preferences.postprocess.reference)
        err = [err ;26];
        disp('The EPmain.preferences.postprocess.reference field is not a number.');
    elseif EPmain.preferences.postprocess.reference < 1 && EPmain.postprocess.postprocess.reference > 3
        err = [err ;26];
        disp('The EPmain.preferences.postprocess.reference field is not within the range of 1 to 3.');
    end;
    if ~isfield(EPmain.preferences.postprocess,'refChan1')
        err = [err ;27];
        disp('The EPmain.preferences.postprocess.refChan1 field is missing.');
    elseif isempty(EPmain.preferences.postprocess.refChan1)
        err = [err ;27];
        disp('The EPmain.preferences.postprocess.refChan1 field is empty.');
    elseif ~isnumeric(EPmain.preferences.postprocess.refChan1)
        err = [err ;27];
        disp('The EPmain.preferences.postprocess.refChan1 field is not a number.');
    end;
    if ~isfield(EPmain.preferences.postprocess,'refChan2')
        err = [err ;28];
        disp('The EPmain.preferences.postprocess.refChan2 field is missing.');
    elseif isempty(EPmain.preferences.postprocess.refChan2)
        err = [err ;28];
        disp('The EPmain.preferences.postprocess.refChan2 field is empty.');
    elseif ~isnumeric(EPmain.preferences.postprocess.refChan2)
        err = [err ;28];
        disp('The EPmain.preferences.postprocess.refChan2 field is not a number.');
    end;
    if ~isfield(EPmain.preferences.postprocess,'baseline')
        err = [err ;29];
        disp('The EPmain.preferences.postprocess.baseline field is missing.');
    elseif isempty(EPmain.preferences.postprocess.baseline)
        err = [err ;29];
        disp('The EPmain.preferences.postprocess.baseline field is empty.');
    end;
else
    err = [err ;25];
    disp('The EPmain.preferences.postprocess field is missing.');
end

if isfield(EPmain.preferences,'view')
    if ~isfield(EPmain.preferences.view,'positive')
        err = [err ;37];
        disp('The EPmain.preferences.view.positive field is missing.');
    elseif isempty(EPmain.preferences.view.positive)
        err = [err ;37];
        disp('The EPmain.preferences.view.positive field is empty.');
    elseif ~isnumeric(EPmain.preferences.view.positive)
        err = [err ;37];
        disp('The EPmain.preferences.view.positive field is not a number.');
    elseif EPmain.preferences.view.positive < 1 && EPmain.preferences.view.positive > 2
        err = [err ;37];
        disp('The EPmain.preferences.view.positive field is not within the range of 1 to 2.');
    end;
else
    err = [err ;36];
    disp('The EPmain.preferences.view field is missing.');
end

if isfield(EPmain.preferences,'pca')
    if ~isfield(EPmain.preferences.pca,'mode')
        err = [err ;31];
        disp('The EPmain.preferences.pca.mode field is missing.');
    elseif isempty(EPmain.preferences.pca.mode)
        err = [err ;31];
        disp('The EPmain.preferences.pca.mode field is empty.');
    elseif ~isnumeric(EPmain.preferences.pca.mode)
        err = [err ;31];
        disp('The EPmain.preferences.pca.mode field is not a number.');
    elseif EPmain.preferences.pca.mode < 1 && EPmain.preferences.pca.mode > 2
        err = [err ;31];
        disp('The EPmain.preferences.pca.mode field is not within the range of 1 to 2.');
    end;
    if ~isfield(EPmain.preferences.pca,'rotation')
        err = [err ;32];
        disp('The EPmain.preferences.pca.rotation field is missing.');
    elseif isempty(EPmain.preferences.pca.rotation)
        err = [err ;32];
        disp('The EPmain.preferences.pca.rotation field is empty.');
    elseif ~isnumeric(EPmain.preferences.pca.rotation)
        err = [err ;32];
        disp('The EPmain.preferences.pca.rotation field is not a number.');
    elseif EPmain.preferences.pca.rotation < 1 && EPmain.preferences.pca.rotation > 13
        err = [err ;32];
        disp('The EPmain.preferences.pca.rotation field is not within the range of 1 to 13.');
    end;
    if ~isfield(EPmain.preferences.pca,'rotopt')
        err = [err ;33];
        disp('The EPmain.preferences.pca.rotopt field is missing.');
    elseif isempty(EPmain.preferences.pca.rotopt)
        err = [err ;33];
        disp('The EPmain.preferences.pca.rotopt field is empty.');
    elseif ~isnumeric(EPmain.preferences.pca.rotopt)
        err = [err ;33];
        disp('The EPmain.preferences.pca.rotopt field is not a number.');
    end;
    if ~isfield(EPmain.preferences.pca,'rel')
        err = [err ;34];
        disp('The EPmain.preferences.pca.rel field is missing.');
    elseif isempty(EPmain.preferences.pca.rel)
        err = [err ;34];
        disp('The EPmain.preferences.pca.rel field is empty.');
    elseif ~isnumeric(EPmain.preferences.pca.rel)
        err = [err ;34];
        disp('The EPmain.preferences.pca.rel field is not a number.');
    elseif EPmain.preferences.pca.rel < 1 && EPmain.preferences.pca.rel > 2
        err = [err ;34];
        disp('The EPmain.preferences.pca.rel field is not within the range of 1 to 2.');
    end;
    if ~isfield(EPmain.preferences.pca,'loadings')
        err = [err ;35];
        disp('The EPmain.preferences.pca.loadings field is missing.');
    elseif isempty(EPmain.preferences.pca.loadings)
        err = [err ;35];
        disp('The EPmain.preferences.pca.loadings field is empty.');
    elseif ~isnumeric(EPmain.preferences.pca.loadings)
        err = [err ;35];
        disp('The EPmain.preferences.pca.loadings field is not a number.');
    elseif EPmain.preferences.pca.loadings < 1 && EPmain.preferences.pca.loadings > 4
        err = [err ;35];
        disp('The EPmain.preferences.pca.loadings field is not within the range of 1 to 4.');
    end;
else
    err = [err ;30];
    disp('The EPmain.preferences.pca field is missing.');
end

if isfield(EPmain.preferences,'window')
    if ~isfield(EPmain.preferences.window,'minFacVar')
        err = [err ;39];
        disp('The EPmain.preferences.window.minFacVar field is missing.');
    elseif isempty(EPmain.preferences.window.minFacVar)
        err = [err ;39];
        disp('The EPmain.preferences.window.minFacVar field is empty.');
    elseif ~isnumeric(EPmain.preferences.window.minFacVar)
        err = [err ;39];
        disp('The EPmain.preferences.window.minFacVar field is not a number.');
    elseif EPmain.preferences.window.minFacVar < 0 && EPmain.preferences.window.minFacVar > 1
        err = [err ;39];
        disp('The EPmain.preferences.window.minFacVar field is not within the range of 0 to 1.');
    end;
    if ~isfield(EPmain.preferences.window,'adds')
        err = [err ;46];
        disp('The EPmain.preferences.window.adds field is missing.');
    elseif isempty(EPmain.preferences.window.adds)
        err = [err ;46];
        disp('The EPmain.preferences.window.adds field is empty.');
    elseif ~isnumeric(EPmain.preferences.window.adds)
        err = [err ;46];
        disp('The EPmain.preferences.window.adds field is not a number.');
    elseif EPmain.preferences.window.adds ~= 0 && EPmain.preferences.window.adds ~=1
        err = [err ;46];
        disp('The EPmain.preferences.window.adds field must be either 0 or 1.');
    end;
else
    err = [err ;38];
    disp('The EPmain.preferences.window field is missing.');
end

if isfield(EPmain.preferences,'anova')
    if ~isfield(EPmain.preferences.anova,'trimming')
        err = [err ;41];
        disp('The EPmain.preferences.anova.trimming field is missing.');
    elseif isempty(EPmain.preferences.anova.trimming)
        err = [err ;41];
        disp('The EPmain.preferences.anova.trimming field is empty.');
    elseif ~isnumeric(EPmain.preferences.anova.trimming)
        err = [err ;41];
        disp('The EPmain.preferences.anova.trimming field is not a number.');
    elseif EPmain.preferences.anova.trimming < 0 && EPmain.preferences.anova.trimming > .5
        err = [err ;41];
        disp('The EPmain.preferences.anova.trimming field is not within the range of 0 to .5.');
    end;
    if ~isfield(EPmain.preferences.anova,'bootstrap')
        err = [err ;42];
        disp('The EPmain.preferences.anova.bootstrap field is missing.');
    elseif isempty(EPmain.preferences.anova.bootstrap)
        err = [err ;42];
        disp('The EPmain.preferences.anova.bootstrap field is empty.');
    elseif ~isnumeric(EPmain.preferences.anova.bootstrap)
        err = [err ;42];
        disp('The EPmain.preferences.anova.bootstrap field is not a number.');
    end;
    if ~isfield(EPmain.preferences.anova,'seed')
        err = [err ;43];
        disp('The EPmain.preferences.anova.seed field is missing.');
    elseif isempty(EPmain.preferences.anova.seed)
        err = [err ;43];
        disp('The EPmain.preferences.anova.seed field is empty.');
    elseif ~isnumeric(EPmain.preferences.anova.seed)
        err = [err ;43];
        disp('The EPmain.preferences.anova.seed field is not a number.');
    end;
    if ~isfield(EPmain.preferences.anova,'missing')
        err = [err ;44];
        disp('The EPmain.preferences.anova.missing field is missing.');
    elseif isempty(EPmain.preferences.anova.missing)
        err = [err ;44];
        disp('The EPmain.preferences.anova.missing field is empty.');
    elseif ~isnumeric(EPmain.preferences.anova.missing)
        err = [err ;44];
        disp('The EPmain.preferences.anova.missing field is not a number.');
    end;
    if ~isfield(EPmain.preferences.anova,'adds')
        err = [err ;45];
        disp('The EPmain.preferences.anova.adds field is missing.');
    elseif isempty(EPmain.preferences.anova.adds)
        err = [err ;45];
        disp('The EPmain.preferences.anova.adds field is empty.');
    elseif ~isnumeric(EPmain.preferences.anova.adds)
        err = [err ;45];
        disp('The EPmain.preferences.anova.adds field is not a number.');
    elseif EPmain.preferences.anova.adds ~= 0 && EPmain.preferences.anova.adds ~=1
        err = [err ;45];
        disp('The EPmain.preferences.anova.adds field must be either 0 or 1.');
    end;
else
    err = [err ;40];
    disp('The EPmain.preferences.anova field is missing.');
end

if err
    beep
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function splash(src,eventdata) %present splash page

hSPlash=figure('Name', 'About EP Toolkit', 'NumberTitle', 'off', 'MenuBar', 'none');

try
    EPver=ver('EP_Toolkit');
catch
    EPver.Version='unavailable'; %workaround for bug in earlier version of Matlab
end;

copyright=['ERP PCA Toolkit version: ' EPver.Version ' '...
    'Copyright (C) 1999-2011  Joseph Dien. ' ...
    sprintf('\n\n')...
    'This program is free software: you can redistribute it and/or modify '...
    'it under the terms of the GNU General Public License as published by '...
    'the Free Software Foundation, either version 3 of the License, or '...
    '(at your option) any later version. '...
    'This program is distributed in the hope that it will be useful, '...
    'but WITHOUT ANY WARRANTY; without even the implied warranty of '...
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE (especially medical).  See the '...
    'GNU General Public License for more details. '...
    'You should have received a copy of the GNU General Public License '...
    'along with this program.  If not, see <http://www.gnu.org/licenses/>. '...
    sprintf('\n\n')...
    'Note also that Dr. Dien does not have grant funding for this software '...
    'project and cannot therefore guarantee the same level of support '...
    'available for software projects that have full-time staff allocated '...
    'to them.  Nonetheless, Dr. Dien will endeavor to help users with '...
    'bug reports and questions to the best of his ability, given his time constraints. '...
    'He will be happy to help with implementing support for new file types. '...
    'Others are welcome to contribute code to this software project. '...
    'See files in the documentation directory for more information.'...
    sprintf('\n\n')...
    '***Again, this software is intended solely for scientific research and should never be used for medical purposes.***'];

uicontrol('Style','text',...
    'String',copyright,...
    'HorizontalAlignment','left','Position',[20 50 400 350]);

EPmain.handles.splash.agree = uicontrol('Style', 'pushbutton', 'String', 'Agree',...
    'Position', [20 0 100 40], 'Callback', ['global EPmain;','EPmain.mode=''agree'';','ep']);

EPmain.handles.splash.disagree = uicontrol('Style', 'pushbutton', 'String', 'Disagree',...
    'Position', [120 0 100 40], 'Callback', ['global EPmain;','EPmain.mode=''disagree'';','ep']);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function windowSampStart(src,eventdata) %change start of window

global EPmain EPdataset

EPdata=ep_loadEPdataset(EPdataset,EPmain.window.dataset);
sampStart=str2num(get(EPmain.handles.window.sampStart,'String'));
sampEnd=str2num(get(EPmain.handles.window.sampEnd,'String'));
numPoints=length(EPdata.timeNames);
Fs=EPdata.Fs;
baseline=EPdata.baseline;

if sampStart > numPoints
    sampStart=numPoints;
end;

if sampStart < 1
    sampStart=1;
end;

if sampStart > sampEnd
    sampEnd = sampStart;
end;

set(EPmain.handles.window.sampStart,'String',sampStart);
set(EPmain.handles.window.sampEnd,'String',sampEnd);
set(EPmain.handles.window.msStart,'String',(sampStart-baseline)*(1000/Fs));
set(EPmain.handles.window.msEnd,'String',(sampEnd-baseline)*(1000/Fs));

EPmain.window.sampStart=sampStart;
EPmain.window.sampEnd=sampEnd;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function windowSampEnd(src,eventdata) %change end of window

global EPmain EPdataset

EPdata=ep_loadEPdataset(EPdataset,EPmain.window.dataset);
sampStart=str2num(get(EPmain.handles.window.sampStart,'String'));
sampEnd=str2num(get(EPmain.handles.window.sampEnd,'String'));
numPoints=length(EPdata.timeNames);
Fs=EPdata.Fs;
baseline=EPdata.baseline;

if sampEnd > numPoints
    sampEnd=numPoints;
end;

if sampEnd < 1
    sampEnd=1;
end;

if sampStart > sampEnd
    sampStart = sampEnd;
end;

set(EPmain.handles.window.sampStart,'String',sampStart);
set(EPmain.handles.window.sampEnd,'String',sampEnd);
set(EPmain.handles.window.msStart,'String',(sampStart-baseline)*(1000/Fs));
set(EPmain.handles.window.msEnd,'String',(sampEnd-baseline)*(1000/Fs));

EPmain.window.sampStart=sampStart;
EPmain.window.sampEnd=sampEnd;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function windowSpecsTable(src,eventdata) %change specs table data in Window pane

global EPmain

specTable=get(EPmain.handles.window.specsTable,'Data');
EPmain.window.specSelect=cell2mat(specTable(:,1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function windowCellTable(src,eventdata) %change cell table data in Window pane

global EPmain

cellTable=get(EPmain.handles.window.cellTable,'Data');
EPmain.window.outCells=cellTable(:,1);
EPmain.window.inCells=cellTable(:,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function saveData(src,eventdata) %save the data

global EPmain EPdataset

if ~EPmain.noTable
    if isempty(eventdata.Indices) %if just deselecting
        return;
    end;
end;

if EPmain.noTable
    whichData=get(EPmain.handles.save.menu,'Value');
else
    whichData=eventdata.Indices(1);
end;

formatNum = get(EPmain.handles.save.hFormat,'value');
switch formatNum
    case 1
        format='egi_egis';
    case 2
        format='egi_sbin';
    case 3
        format='eeglab_set';      
    case 4
        format='text';
    case 5
        format='ep_mat';
end;
EPdata=ep_loadEPdataset(EPdataset,whichData);
if strcmp(format,'egi_egis') && ~strcmp(EPdata.dataType,'single_trial')
    format='egi_egia';
end;

if strcmp(format,'egi_egia') && EPdata.baseline ==0
    choice = questdlg('EGIS average file with a zero baseline will cause problems for NetStation.  Save anyway?', ...
        'Options', ...
        'Yes','Cancel','Yes');
    % Handle response
    switch choice
        case 'Yes'
        case 'Cancel'
            return
    end
end;

EPmain.save.format=formatNum;

if ~EPmain.save.SGLchan && ~EPmain.save.REGchan
    EPmain.save.SGLchan=1; %at least some type of channel needs to be kept
end;

if ~EPmain.save.SGLcell && ~EPmain.save.CMBcell
    EPmain.save.SGLcell=1; %at least some type of cell needs to be kept
end;

if ~EPmain.save.RAW && ~EPmain.save.AVG && ~EPmain.save.GAV
    EPmain.save.AVG=1; %at least some type of subject needs to be kept
end;

if ~EPmain.save.SGLfac && ~EPmain.save.CMBfac
    EPmain.save.SGLfac=1; %at least some type of factor needs to be kept
end;

adds=[];
if EPmain.save.SGLchan
    adds{end+1}='SGLchan';
end;
if EPmain.save.REGchan
    adds{end+1}='REGchan';
end;
if EPmain.save.SGLcell
    adds{end+1}='SGLcell';
end;
if EPmain.save.CMBcell
    adds{end+1}='CMBcell';
end;
if EPmain.save.RAW
    adds{end+1}='RAW';
end;
if EPmain.save.AVG
    if any(strcmp(format,{'egi_egis','egi_egia','egi_sbin','eeglab_set','text'})) && ~isempty(EPdata.facNames)
        choice = questdlg('Averaged Data option will result in an additional file for each subject.  Do anyway?', ...
            'Options', ...
            'Yes','No','Cancel','Yes');
        % Handle response
        switch choice
            case 'Yes'
                adds{end+1}='AVG';
            case 'No'
            case 'Cancel'
                return
        end
    else
        adds{end+1}='AVG';
    end;
end;
if EPmain.save.GAV
    adds{end+1}='GAV';
end;
if EPmain.save.SGLfac
    adds{end+1}='SGLfac';
end;
if EPmain.save.CMBfac
    adds{end+1}='CMBfac';
end;

prefs={};
if ~EPmain.preferences.general.montage
    prefs{end+1}='no montage';
end;

[outFileName, pathname] = uiputfile('*.*','Save:',EPdataset.dataset(whichData).dataName);
if outFileName == 0
    return %user hit cancel on file requestor
end;
outFileName=[pathname outFileName];
if exist(outFileName,'file')
    delete(outFileName); %user must have clicked "yes" to whether to replace existing file
end;

set(EPmain.handles.save.done,'enable','off');
drawnow
ep_writeData(EPdata,outFileName,format,adds, prefs);
EPdataset.dataset(whichData).saved='yes';
set(EPmain.handles.save.done,'enable','on');
drawnow

ep('start');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function windowAdds(inputcells, outCellNames, autoPCA)
%add regional channels and combined cells to the active dataset that correspond to the windowing procedure.

global EPmain EPdataset EPchanGrp

EPdata=ep_loadEPdataset(EPdataset,EPmain.window.dataset);
numChans=length(EPdata.chanNames);
numPoints=length(EPdata.timeNames);
numCells=length(EPdata.cellNames);
numSubs=length(EPdata.subNames);
numFacs=length(EPdata.facNames);
if numFacs==0
    numFacs=1;
end;
if ~isempty(EPdata.facData)
    numCMBfacs=size(EPdata.facData,5);
else
    numCMBfacs=0;
end;
numSGLfacs=numFacs-numCMBfacs;

if ~autoPCA
    %add regional channels if any have new names
    areaList = setdiff(unique(EPchanGrp.group(EPmain.window.chanGrp).channel),EPchanGrp.numAreas+1); %identify area numbers that were used
    newAreas=find(~ismember(EPchanGrp.group(EPmain.window.chanGrp).areaName(areaList),EPdata.chanNames));
    if ~isempty(newAreas)
        for theArea=1:length(newAreas)
            whichArea=areaList(newAreas(theArea));
            chanList=find(EPchanGrp.group(EPchanGrp.activeGrp).channel == whichArea);
            
            newChanData=zeros(1,numPoints,numCells,numSubs,numSGLfacs);
            
            if ~isempty(EPdata.facVecT)
                newChanData=newChanData(:,1,:,:,:);
            end;
            newFacVecSData=zeros(1,numSGLfacs);
            newBadChanData=zeros(numSubs,numCells,1);
            newFacData=zeros(1,numPoints,numCells,numSubs,numCMBfacs);
            for theChan=1:length(chanList)
                chan=chanList(theChan);
                if ~isempty(EPdata.facVecS)
                    newFacVecSData=newFacVecSData+(1/length(chanList))*EPdata.facVecS(chan,:);
                else
                    newChanData=newChanData+(1/length(chanList))*EPdata.data(chan,:,:,:,:);
                end;
                if ~isempty(EPdata.facData)
                    newFacData=newFacData+(1/length(chanList))*EPdata.facData(chan,:,:,:,:);
                end;
                newBadChanData(:,:,1)=newBadChanData(:,:,1)+(1/length(chanList))*EPdata.analysis.badChans(:,:,chan);
            end;
            
            if ~isempty(EPdata.facVecS)
                EPdata.facVecS(end+1,:)=newFacVecSData;
            else
                EPdata.data(end+1,:,:,:,:)=newChanData;
            end;
            if ~isempty(EPdata.facData)
                EPdata.facData(end+1,:,:,:,:)=newFacData;
            end;
            EPdata.analysis.badChans(:,:,end+1)=newBadChanData;
            
            if isfield(EPdata,'noise')
                newChanData=zeros(1,numPoints,numCells,numSubs,numSGLfacs);
                if ~isempty(EPdata.facVecT)
                    newChanData=newChanData(:,1,:,:,:);
                end;
                for theChan=1:length(chanList)
                    chan=chanList(theChan);
                    newChanData=newChanData+(1/length(chanList))*EPdata.noise(chan,:,:,:);
                end;
                EPdata.noise(end+1,:,:,:,:)=newChanData;
            end;
            if isfield(EPdata,'std')
                newChanData=zeros(1,numPoints,numCells,numSubs,numSGLfacs);
                if ~isempty(EPdata.facVecT)
                    newChanData=newChanData(:,1,:,:,:);
                end;
                for theChan=1:length(chanList)
                    chan=chanList(theChan);
                    newChanData=newChanData+(1/length(chanList))*EPdata.std(chan,:,:,:);
                end;
                EPdata.std(end+1,:,:,:,:)=newChanData;
            end;
            
            EPdata.chanNames{end+1}=EPchanGrp.group(EPmain.window.chanGrp).areaName{whichArea};
            EPdata.chanTypes{end+1}='REG';
            if ~isempty(EPdata.eloc)
                EPdata.eloc(end+1)=cell2struct(cell(1,length(fieldnames(EPdata.eloc))),fieldnames(EPdata.eloc),2);
            end;
            numChans=numChans+1;
        end;
    end;
end;

%add combined output cells if any have new names

newCells=find(~ismember(outCellNames,EPdata.cellNames));
if ~isempty(newCells)
    for theOutCell=1:length(newCells)
        cellList=inputcells{newCells(theOutCell)};
        newCellData=zeros(numChans,numPoints,1,numSubs,numSGLfacs);
        newFacData=zeros(numChans,numPoints,1,numSubs,numCMBfacs);
        if ~isempty(EPdata.facVecT)
            newCellData=newCellData(:,1,:,:,:);
        end;
        if ~isempty(EPdata.facVecS)
            newCellData=newCellData(1,:,:,:,:);
        end;
        for i=1:length(cellList)
            theCell=cellList(i);
            newCellData=newCellData+(1/length(cellList))*EPdata.data(:,:,theCell,:,:);
            if ~isempty(EPdata.facData)
                newFacData=newFacData+(1/length(cellList))*EPdata.facData(:,:,theCell,:,:);
            end;
        end;
        EPdata.data(:,:,end+1,:,:)=newCellData;
        if ~isempty(EPdata.facData)
            EPdata.facData(:,:,end+1,:,:)=newFacData;
        end;
        if isfield(EPdata,'noise')
            newCellData=zeros(numChans,numPoints,1,numSubs,numSGLfacs);
            if ~isempty(EPdata.facVecT)
                newCellData=newCellData(:,1,:,:,:);
            end;
            if ~isempty(EPdata.facVecS)
                newCellData=newCellData(1,:,:,:,:);
            end;
            
            for i=1:length(cellList)
                theCell=cellList(i);
                newCellData=newCellData+(1/length(cellList))*EPdata.noise(:,:,theCell,:,:);
            end;
            EPdata.noise(:,:,end+1,:,:)=newCellData;
        end;
        if isfield(EPdata,'std')
            newCellData=zeros(numChans,numPoints,1,numSubs,numSGLfacs);
            if ~isempty(EPdata.facVecT)
                newCellData=newCellData(:,1,:,:,:);
            end;
            if ~isempty(EPdata.facVecS)
                newCellData=newCellData(1,:,:,:,:);
            end;
            
            for i=1:length(cellList)
                theCell=cellList(i);
                newCellData=newCellData+(1/length(cellList))*EPdata.std(:,:,theCell,:,:);
            end;
            EPdata.std(:,:,end+1,:,:)=newCellData;
        end;
        
        for i=1:numSubs
            if ~isempty(find(EPdata.avgNum(i,cellList) == -1))
                EPdata.avgNum(i,numCells+1) = -1; %bad number
            elseif ~isempty(find(EPdata.avgNum(i,cellList) == 0))
                EPdata.avgNum(i,numCells+1) = 0; %unknown number
            else
                EPdata.avgNum(i,numCells+1)=sum(EPdata.avgNum(i,cellList));
            end
            
            if ~isempty(find(EPdata.subNum(i,cellList) == -1))
                EPdata.subNum(i,numCells+1) = -1; %bad number
            elseif ~isempty(find(EPdata.subNum(i,cellList) == 0))
                EPdata.subNum(i,numCells+1) = 0; %unknown number
            elseif length(unique(EPdata.subNum(i,cellList))) > 1
                EPdata.subNum(i,numCells+1) = 0; %unknown number
            else
                EPdata.subNum(i,numCells+1)=sum(EPdata.subNum(i,cellList(1))); %since all the same, the first one is fine
            end
            
            EPdata.analysis.blinkTrial(i,numCells+1)= sum(EPdata.analysis.blinkTrial(i,cellList));
            EPdata.analysis.saccadeTrial(i,numCells+1)= sum(EPdata.analysis.saccadeTrial(i,cellList));
            EPdata.analysis.saccadeOnset(i,numCells+1)= sum(EPdata.analysis.saccadeOnset(i,cellList));
            EPdata.analysis.moveTrial(i,numCells+1)= sum(EPdata.analysis.moveTrial(i,cellList));
            EPdata.analysis.badTrials(i,numCells+1)= sum(EPdata.analysis.badTrials(i,cellList));
            EPdata.analysis.badChans(i,numCells+1,:)= sum(EPdata.analysis.badChans(i,cellList,:));
        end;
        
        if length(cellList) > 1
            EPdata.cellTypes{end+1}='CMB';
        else
            EPdata.cellTypes{end+1}=EPdata.cellTypes(cellList);
        end;
        
        if ~isempty(EPdata.trialNames)
            EPdata.trialNames{end+1}=1;
        end;
        
        numCells=numCells+1;
        EPdata.cellNames{end+1}=outCellNames{newCells(theOutCell)};
        
        EPdata.events(:,end+1)=cell(size(EPdata.events,1),1);
    end;
end;

[err]=ep_checkEPfile(EPdata);

if ~exist([EPdataset.EPwork filesep 'EPwork'],'dir')
    warndlg('The work directory cannot be found.')
    return
end;

delete([EPdataset.EPwork filesep 'EPwork' filesep EPdataset.dataset(EPmain.window.dataset).dataName '.mat']);
EPdataset.dataset=EPdataset.dataset(setdiff([1:length(EPdataset.dataset)],EPmain.window.dataset));
EPdataset=ep_saveEPdataset(EPdataset,EPdata,length(EPdataset.dataset)+1,'no');
EPdataset.dataset=[EPdataset.dataset(1:EPmain.window.dataset-1) EPdataset.dataset(end) EPdataset.dataset(EPmain.window.dataset:end-1)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function ANOVAAdds(ANOVAdata, Y, subjects, betweenCombos);
%add new grand averages for each between group, such that the subjects that are trimmed correspond to the ones that will be dropped in the ANOVA.
%These will only be computed for channel areas and cells that are present in the ANOVA output.

global EPmain EPdataset

%search through active datasets to see if one has a name matching that in the ANOVA file
whichData=find(strcmp(ANOVAdata.name,{EPdataset.dataset.dataName}));
if isempty(whichData)
    return
end;

EPdata=ep_loadEPdataset(EPdataset,whichData);

if ~any(ismember(ANOVAdata.cellNames,EPdata.cellNames))
    disp('No cells in the ANOVA are present in the dataset so corresponding grand averages will not be added.');
    return
end;

if ~any(ismember(ANOVAdata.areaNames,EPdata.chanNames))
    disp('No channel regions in the ANOVA are present in the dataset so corresponding grand averages will not be added.');
    return
end;

numChans=length(EPdata.chanNames);
numPoints=length(EPdata.timeNames);
numCells=length(EPdata.cellNames);
numSubs=length(EPdata.subNames);
numFacs=length(EPdata.facNames);
if numFacs==0
    numFacs=1;
end;
if ~isempty(EPdata.facData)
    numCMBfacs=size(EPdata.facData,5);
else
    numCMBfacs=0;
end;
numSGLfacs=numFacs-numCMBfacs;

theFactor=str2num(strtok(ANOVAdata.factor,'Factor:'));
if isempty(theFactor)
    theFactor=1;
end;

if strcmp(ANOVAdata.changrp,'autoPCA')
    windowName='autoPCA';
else
    windowName=ANOVAdata.window;
end;

subCounter=0;
for theBetween = 1:length(subjects)
    subList=[1+subCounter:subCounter+subjects(theBetween)];
    betweenName=[betweenCombos{1+subCounter} '|' windowName  '|' ANOVAdata.measure];
    
    if ~any(find(strcmp(betweenName,EPdata.subNames))) %add between group if not already present
        EPdata.subNames{end+1}=betweenName;
        EPdata.subTypes{end+1}='GAV';
        EPdata.subjectSpecs(end+1,:)=cell(1,size(EPdata.subjectSpecs,2));
        EPdata.events(end+1,:)=cell(1,size(EPdata.events,2));
        EPdata.avgNum(end+1,:)=zeros(1,numCells);
        EPdata.subNum(end+1,:)=zeros(1,numCells);
        EPdata.analysis.blinkTrial(end+1,:)= zeros(1,numCells);
        EPdata.analysis.saccadeTrial(end+1,:)= zeros(1,numCells);
        EPdata.analysis.saccadeOnset(end+1,:)= zeros(1,numCells);
        EPdata.analysis.moveTrial(end+1,:)= zeros(1,numCells);
        EPdata.analysis.badTrials(end+1,:)= zeros(1,numCells);
        EPdata.analysis.badChans(end+1,:,:)= zeros(1,numCells,numChans);
        
        newChanData=zeros(numChans,numPoints,numCells,1,numSGLfacs);
        if ~isempty(EPdata.facVecT)
            newChanData=newChanData(:,1,:,:,:);
        end;
        if ~isempty(EPdata.facVecS)
            newChanData=newChanData(1,:,:,:,:);
        end;
        EPdata.data(:,:,:,end+1,:)=newChanData;
        if ~isempty(EPdata.facData)
            newFacData=zeros(numChans,numPoints,numCells,1,numCMBfacs);
            EPdata.facData(:,:,:,end+1,:)=newFacData;
        end;
        if isfield(EPdata,'noise')
            EPdata.noise(:,:,:,end+1,:)=zeros(numChans,numPoints,numCells,1,numSGLfacs);
        end;
        if isfield(EPdata,'std')
            EPdata.std(:,:,:,end+1,:)=zeros(numChans,numPoints,numCells,1,numSGLfacs);
        end;
        numSubs=numSubs+1;
    end;
    theSub=find(strcmp(betweenName,EPdata.subNames));
    
    for theColumn=EPmain.anova.data.leftColumn:EPmain.anova.data.rightColumn
        theCell=find(strcmp(ANOVAdata.cellNames{theColumn},EPdata.cellNames));
        theChan=find(strcmp(ANOVAdata.areaNames{theColumn},EPdata.chanNames));
        if ~isempty(EPdata.facVecS)
            theChan=1;
        else
            if strcmp(ANOVAdata.changrp,'autoPCA')
                theChan=[1:numChans];
            end;
        end;
        if theCell && theChan %if a corresponding cell and channel are present in the data then add the trimmed grand ave
            [B,index] = sort(ANOVAdata.data(ANOVAdata.index(subList),theColumn));
            numTrim=floor(EPmain.preferences.anova.trimming*length(subList));
            index=index(1+numTrim:length(subList)-numTrim); %determine which subjects should go into the grand average for this area/cell
            EPdata.data(theChan,:,theCell,theSub,theFactor)=mean(EPdata.data(theChan,:,theCell,ANOVAdata.index(subList(index)),theFactor),4);
            if isfield(EPdata,'noise')
                EPdata.noise(theChan,:,theCell,theSub,theFactor)=mean(EPdata.noise(theChan,:,theCell,ANOVAdata.index(subList(index)),theFactor),4);
            end;
            
            if ~isempty(find(EPdata.avgNum(ANOVAdata.index(subList(index)),theCell) == -1))
                EPdata.avgNum(theSub,theCell) = -1; %bad number
            elseif ~isempty(find(EPdata.avgNum(ANOVAdata.index(subList(index)),theCell) == 0))
                EPdata.avgNum(theSub,theCell) = 0; %unknown number
            else
                EPdata.avgNum(theSub,theCell)=sum(EPdata.avgNum(ANOVAdata.index(subList(index)),theCell));
            end
            
            if ~isempty(find(EPdata.subNum(ANOVAdata.index(subList(index)),theCell) == -1))
                EPdata.subNum(theSub,theCell) = -1; %bad number
            elseif ~isempty(find(EPdata.subNum(ANOVAdata.index(subList(index)),theCell) == 0))
                EPdata.subNum(theSub,theCell) = 0; %unknown number
            else
                EPdata.subNum(theSub,theCell)=sum(EPdata.subNum(ANOVAdata.index(subList(index)),theCell));
            end
            
            %since this function is not allowed for single subject files, can assume are averaged data
            EPdata.analysis.blinkTrial(theSub,theCell)= sum(EPdata.analysis.blinkTrial(ANOVAdata.index(subList(index)),theCell));
            EPdata.analysis.saccadeTrial(theSub,theCell)= sum(EPdata.analysis.saccadeTrial(ANOVAdata.index(subList(index)),theCell));
            EPdata.analysis.saccadeOnset(theSub,theCell)= sum(EPdata.analysis.saccadeOnset(ANOVAdata.index(subList(index)),theCell));
            EPdata.analysis.moveTrial(theSub,theCell)= sum(EPdata.analysis.moveTrial(ANOVAdata.index(subList(index)),theCell));
            EPdata.analysis.badTrials(theSub,theCell)= sum(EPdata.analysis.badTrials(ANOVAdata.index(subList(index)),theCell));
            EPdata.analysis.badChans(theSub,theCell,:)= sum(EPdata.analysis.badChans(ANOVAdata.index(subList(index)),theCell,:));
        end;
    end;
    subCounter=subCounter+subjects(theBetween);
end;


[err]=ep_checkEPfile(EPdata);

if ~exist([EPdataset.EPwork filesep 'EPwork'],'dir')
    warndlg('The work directory cannot be found.')
    return
end;

delete([EPdataset.EPwork filesep 'EPwork' filesep EPdataset.dataset(whichData).dataName '.mat']);
EPdataset.dataset=EPdataset.dataset(setdiff([1:length(EPdataset.dataset)],whichData));
EPdataset=ep_saveEPdataset(EPdataset,EPdata,length(EPdataset.dataset)+1,'no');
EPdataset.dataset=[EPdataset.dataset(1:whichData-1) EPdataset.dataset(end) EPdataset.dataset(whichData:end-1)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function ANOVATable(src,eventdata)
%process entry into ANOVA factor table.

global EPmain

theFactor=find(src == EPmain.handles.anova.factor);
theLevel=find(src == EPmain.handles.anova.levels);
if theFactor
    theEntry=get(EPmain.handles.anova.factor(theFactor),'string');
    if length(theEntry)==3
        EPmain.anova.data.factor{theFactor}=theEntry;
    else
        EPmain.anova.data.factor{theFactor}=[];
    end;
end;

if theLevel
    theEntry=get(EPmain.handles.anova.levels(theLevel),'string');
    EPmain.anova.data.levels{theLevel}=theEntry;
end;

ANOVAlevels=0;
for theFactor=1:6
    if ~isempty(EPmain.anova.data.levels{theFactor}) && ~isempty(EPmain.anova.data.factor{theFactor})
        if ANOVAlevels==0
            ANOVAlevels=1;
        end;
        ANOVAlevels=ANOVAlevels*length(EPmain.anova.data.levels{theFactor});
    end;
end;

EPmain.anova.data.totalLevels=ANOVAlevels;

ep('start');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function betweenName(src,eventdata)
%process entry of between factor name into ANOVA pane

global EPmain

theFactor=find(src == EPmain.handles.anova.betweenName);
if theFactor
    theEntry=get(EPmain.handles.anova.betweenName(theFactor),'string');
    if length(theEntry)==3
        EPmain.anova.data.betweenName{theFactor}=theEntry;
    else
        EPmain.anova.data.betweenName{theFactor}='';
    end;
end;

ep('start');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function betweenGroup(src,eventdata)
%process selection of between group level column in ANOVA pane

global EPmain

theFactor=find(src == EPmain.handles.anova.between);
if ~isempty(theFactor)
    temp=get(EPmain.handles.anova.between(theFactor),'value');
    if temp ~=0
        betweenNames=EPmain.anova.data.columnNames;
        betweenNames{end+1}='none';
        betweenRange=[setdiff([1:length(betweenNames)],[1:EPmain.anova.data.rightColumn,EPmain.anova.data.between(setdiff(1:6,theFactor))]) length(betweenNames)];
        EPmain.anova.data.between(theFactor)=betweenRange(temp);
    end;
    if isempty(temp)
        betweenNames=EPmain.anova.data.columnNames;
        betweenNames{end+1}='none';
        betweenRange=[setdiff([1:length(betweenNames)],[1:EPmain.anova.data.rightColumn,EPmain.anova.data.between(setdiff(1:6,theFactor))]) length(betweenNames)];
        EPmain.anova.data.between(theFactor)=betweenRange(temp);
    end;
end;
ep('start');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function quit(src,eventdata)
%quit from EP Toolkit

global EPdataset

button = questdlg('Delete Work Directory?');

switch button
    case 'Yes'
        rmdir([EPdataset.EPwork filesep 'EPwork'],'s');
    case 'No'
        
    case 'Cancel'
        return
end;

close('EP Toolkit');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function changeWindowDataset(src,eventdata)
%change the dataset in the window function

global EPmain EPdata EPdataset EPtable

temp=get(EPmain.handles.window.dataset,'Value');
if temp ~=0
    EPmain.window.dataset=EPmain.window.aveData(temp);
end;
if isempty(temp)
    EPmain.window.dataset=EPmain.window.aveData(temp);
end;
EPdata=ep_stripAdds(ep_loadEPdataset(EPdataset,EPmain.window.dataset));
EPmain.window.inCells=[];
EPmain.window.outCells=[];
for i=1:length(EPdata.cellNames)
    EPmain.window.inCells{i}=EPdata.cellNames{i};
    EPmain.window.outCells{i}=EPdata.cellNames{i};
end;

EPmain.window.specSelect=repmat(false,length(EPdata.subjectSpecNames),1);

EPmain.window.sampStart=1;
EPmain.window.sampEnd=1;

if EPmain.noTable
    if isfield(EPtable,'window')
        EPtable=rmfield(EPtable,'window');
    end;
end;

ep('start');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function checkViewSettings(src,eventdata)
%check the settings in the view pane to see if the Waves control should be enabled.

global EPmain

set(EPmain.handles.view.waves,'enable','on');

bottomVolt=str2num(get(EPmain.handles.view.bottomVolt,'string'));
topVolt=str2num(get(EPmain.handles.view.topVolt,'string'));
leftEpoch=str2num(get(EPmain.handles.view.leftEpoch,'string'));
rightEpoch=str2num(get(EPmain.handles.view.rightEpoch,'string'));
marker1=str2num(get(EPmain.handles.view.marker1,'string'));
marker2=str2num(get(EPmain.handles.view.marker2,'string'));

if bottomVolt > topVolt
    set(EPmain.handles.view.waves,'enable','off');
end;

if leftEpoch > rightEpoch
    set(EPmain.handles.view.waves,'enable','off');
end;

if leftEpoch > rightEpoch
    set(EPmain.handles.view.waves,'enable','off');
end;

if ~isempty(marker1)
    if (marker1 < leftEpoch) || (marker1 > rightEpoch)
        set(EPmain.handles.view.waves,'enable','off');
    end;
end;

if ~isempty(marker2)
    if (marker2 < leftEpoch) || (marker2 > rightEpoch)
        set(EPmain.handles.view.waves,'enable','off');
    end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function changeWork(src,eventdata)
%change work directory

global EPdataset EPmain

newDir = uigetdir(EPdataset.EPwork,'Select EPwork directory');
if newDir == 0
    msg{1}='No directory selected.';
    [msg]=ep_errorMsg(msg);
    return
end;

seps=findstr(newDir,filesep);
if ~isempty(seps)
    theDir=newDir(seps(end)+1:end);
    newDir=newDir(1:seps(end));
else
    theDir=newDir;
    newDir='.';
end;

if ~strcmp(theDir,'EPwork')
    msg{1}='Directory not named EPwork.';
    [msg]=ep_errorMsg(msg);
    return
end;

if ~exist([newDir filesep 'EPwork' filesep 'EPdataset.mat'],'file')
    msg{1}='EPwork directory missing EPdataset file.';
    [msg]=ep_errorMsg(msg);
    return
end;

if ~exist([newDir filesep 'EPwork' filesep 'EPprefs.mat'],'file')
    msg{1}='EPwork directory missing EPprefs file.';
    [msg]=ep_errorMsg(msg);
    return
end;

eval(['load ' newDir filesep 'EPwork' filesep 'EPdataset.mat']);
load([newDir filesep 'EPwork' filesep 'EPprefs.mat']);
EPmain.preferences=prefs; 
EPdataset.EPwork=newDir;

ep('start');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function createWork(src,eventdata)
%create work directory in current directory

global EPdataset EPmain

if exist([pwd filesep 'EPwork'],'dir')
    msg{1}='EPwork directory already present.';
    [msg]=ep_errorMsg(msg);
    return
end;

mkdir([pwd filesep 'EPwork']);
EPdataset=[];
EPdataset.EPwork=pwd;
EPdataset.dataset=cell(0);
eval(['save ''' pwd filesep 'EPwork' filesep 'EPdataset'' EPdataset']);
prefs=EPmain.preferences;
eval(['save ''' pwd filesep 'EPwork' filesep 'EPprefs'' prefs']);

ep('start');
